###################################
#
# EVM Automate Method: add_user_to_group_email
# Notes: This method is used to email an admin to add an new user to the correct group
###################################
def log(level, message) 
  @method = 'add_user_to_group_email' 
  $evm.log(level, "#{@method} - #{message}") 
end

# dump_root 
def dump_root() 
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes") 
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")} 
  log(:info, "Root:<$evm.root> End $evm.root.attributes") 
  log(:info, "") 
end 

def email_user
  $evm.log('info', "#{@method} - email logic starting") if @debug
  
  # Getting user
  user = $evm.root['user']
  
  # If to is still nil use to_email_address from model
  to = nil
  to ||= user.email

  # Get from_email_address from model unless specified below
  from = nil
  from ||= $evm.object['from_email_address']
  
  # Get signature from model unless specified below
  signature = nil
  signature ||= $evm.object['signature']
  
  # Getting task
  task = $evm.root['service_template_provision_task']

  # Build subject
  subject = "Request ID #{task.miq_request_id} - You Have Requested Access to a Group"

  # Build email body
  body = "Hello #{user.first_name},"
  body += "<br><br>Your request to be added to the group: #{$evm.root['dialog_cf_group_name']} on #{task.created_on} has been submitted<br>"
  body += "<br><br>Once complete, your dashboard and services will be updated.<br>"
  body += "<br><br> Thank you,"
  body += "<br> #{signature}<br>"
  
  $evm.log("info", "#{@method} - Sending email to <#{to}> from <#{from}> subject: <#{subject}>") if @debug
  $evm.execute(:send_email, to, from, subject, body)
  
end

def email_admin
  $evm.log('info', "#{@method} - email logic starting") if @debug

  # If to is still nil use to_email_address from model
  to = nil
  to ||= $evm.object['to_email_address']

  # Get from_email_address from model unless specified below
  from = nil
  from ||= $evm.object['from_email_address']
  
  # Get signature from model unless specified below
  signature = nil
  signature ||= $evm.object['signature']
  
  #get operations email
  operations = nil
  operations ||= $evm.object['operations_email_address']
  
  # Getting user
  user = $evm.root['user']
  
  # Getting task
  task = $evm.root['service_template_provision_task']

  # Build subject
  subject = "Request ID #{task.miq_request_id} - A New User has Requested Access to a Group"

  # Build email body
  body = "Hello,"
  body += "<br><br>The user: #{user.first_name + ' ' + user.last_name} (#{user.email}) has submitted a request to be added to the group: #{$evm.root['dialog_cf_group_name']} on #{task.created_on}.<br>"
  body += "<br><br>Once verified, please add the user to the correct groups.<br>"
  body += "<br><br> Thank you,"
  body += "<br> #{signature}<br>"
  body += "<br>**********************************************************************************************************************<br>"
  body += "To change a Users Group follow the steps below:<br><br>"
  body += "<table><tr><td>1.</td><td>From the Navigation bar, go to <strong>Configure</strong> -> <strong>Configuration</strong></td></tr>"
  body += "<tr><td>2.</td><td>From the Left Panel, click on <strong>Access Control</strong></td></tr>"
  body += "<tr><td>3.</td><td>Expand the User list and click <strong>#{user.name}</strong></td></tr>"
  body += "<tr><td>4.</td><td>Click <strong>Configuration</strong> (Above the line: EVM User \"#{user.name}\") -> <strong>Edit this User</strong></td></tr>"
  body += "<tr><td>3.</td><td>Expand <strong>Group</strong> and select <strong>#{$evm.root['dialog_cf_group_name']}</strong></td></tr></table>"
  
  $evm.log("info", "#{@method} - Sending email to <#{to}> from <#{from}> subject: <#{subject}>") if @debug
  $evm.execute(:send_email, to, from, subject, body)
  $evm.execute(:send_email, operations, from, subject, body)
end

def remove_service
  service = $evm.vmdb('service').find_by_id($evm.root['service_template_provision_task'].destination_id)
  service.retire_now
end

begin
  $evm.log("info", "#{@method} - EVM Automate Method Started")
  dump_root

  # Email admin and user
  email_admin
  remove_service
  email_user
  $evm.log("info", "#{@method} - EVM Automate Method Ended")
  exit MIQ_OK

rescue => err
  $evm.log("error", "#{@method} - [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_STOP
end
