###################################
#
# EVM Automate Method: vm_email_remedy
#
# Notes: This method is used to send an email to remedy
#
###################################
begin
  @method = 'vm_email_remedy'
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Started")

  # Turn of verbose logging
  @debug = true

  obj   = $evm.object

  $evm.log("info", "Listing Object Attributes:")
  obj.attributes.sort.each { |k, v| $evm.log("info", "\t#{k}: #{v}")} if @debug
  $evm.log("info", "===========================================")

  #
  # Look in the current object for a VM
  #
  vm = $evm.object['vm']
  if vm.nil?
    vm_id = $evm.object['vm_id'].to_i
    vm = $evm.vmdb('vm', vm_id) unless vm_id == 0
  end

  #
  # Look in the Root Object for a VM
  #
  if vm.nil?
    vm = $evm.root['vm']
    if vm.nil?
      vm_id = $evm.root['vm_id'].to_i
      vm = $evm.vmdb('vm', vm_id) unless vm_id == 0
    end
  end

  #
  # No VM Found, exit
  #
  raise "VM not found" if vm.nil?


  process = $evm.object("process")
  policy_id = process.attributes["policy_id"].to_i
  policy_name = process.attributes["policy_name"]
  policy_description = process.attributes["policy_description"]
  $evm.log("info", "Policy ID:          #{policy_id}")
  $evm.log("info", "Policy Name:        #{policy_name}")
  $evm.log("info", "Policy Description: #{policy_description}")
  #policy    = $evm.vmdb('policy', policy_id)

  subject = "Remedy Ticket - Policy Violation"
  body =  "Schema: HD Incident"
  body << "<br/>Server: DemoRemedy"
  body << "<br/>Login: EVM_Demo"
  body << "<br/>Password: <mypassword>"
  body << "<br/>Action: Submit"
  body << "<br/>!Submitter!: EVM"
  body << "<br/>!Short Description!: VM policy violation VM Name: #{vm.name} Policy: #{policy_description}"
  body << "<br>"
  body << "<br><br>To view and manage your virtual machine go to <a href='https://192.168.254.147/vm/show/#{vm['id']}'>https://192.168.254.147/vm/show/#{vm['id']}</a>"

  obj['to']      = "evmadmin@company.com"
  obj['subject'] = subject
  obj['body']    = body

  #
  # Exit method
  #
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "<#{@method}>: [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_STOP
end
