###################################
#
# EVM Automate Method: update_migration_status
#
# Notes: This method updates migration status and calls Service Now if applicable
#
# Required inputs: status, status_state
#
###################################
begin
  @method = 'update_migration_status'
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Started")

  # Turn of verbose logging
  @debug = true

  #
  # Initialize variables
  #
  miq_task = $evm.root['vm_migrate_task']
  miq_task_id     = miq_task['id']

  # Get variables from Server object
  server = $evm.root['miq_server']
  evm_ipaddr    = server.ipaddress
  evm_hostname  = server.hostname
  evm_region    = server.region_number

  # Get State Machine
  state = $evm.current_object.class_name

  # Get current step
  step = $evm.current_object.current_field_name

  # Get status from input field status
  status = $evm.inputs['status']

  # Get status_state ['on_entry', 'on_exit', 'on_error'] from input field
  status_state = $evm.inputs['status_state']


  ###################################
  #
  # Update Status for on_entry,on_exit
  #
  ###################################
  if $evm.root['ae_result'] == 'ok'

    # Check to see if provisioning is complete
    if status == 'migration_complete'
      message = 'VM Migrated Successfully'
      miq_task.finished(message)
    end
    miq_task.message = status
  end


  ###################################
  #
  # Update Status for on_error
  #
  ###################################
  if $evm.root['ae_result'] == 'error'
    miq_task.message = status
  end


  ###################################
  #
  # Call Service-Now
  #
  ###################################
  if miq_task.options.has_key?(:ws_values)
    ws_values  = miq_task.options[:ws_values]

    # Set request_id from Service-Now unless nil set to provisioning id
    request_id = ws_values[:request_id] || ws_values[:requestid] || miq_task_id
    item_id    = ws_values[:item_id]    || ws_values[:itemid] || miq_task_id
    friendly_itemid = ws_values[:friendly_itemid]

    $evm.log("info","RequestID: <#{request_id}>") if @debug
    $evm.log("info","ItemID: <#{item_id}>") if @debug
    $evm.log("info","Friendly ItemID: <#{friendly_itemid}>") if @debug

    unless request_id.nil? || item_id.nil?

      message = status_state
      # state_retries = $evm.root['ae_state_retries'] || 0

      agent         = "EVM Automate Migration"
      topic         = "Item:#{friendly_itemid} | EVM:#{miq_task_id} | StateMachine:#{state}"
      name          = "Step:#{step} | State:#{status_state}"
      source        = "EVM Region:#{evm_region} | Hostname:#{evm_hostname} | IP:#{evm_ipaddr}"

      require 'uri'
      args = "agent=#{agent}&topic=#{topic}&source=#{source}&name=#{name}&id=#{miq_task_id}&request_id=#{request_id}&item_id=#{item_id}&message=#{message}&step=#{step}&status=#{status}"
      #args = "topic=#{topic}&id=#{id}&request_id=#{request_id}&item_id=#{item_id}&message=#{message}&step=#{step}&status=#{status}"
      args = URI.escape(args)

      $evm.log("info","Calling Service-Now with arguments: #{args}")
      $evm.instantiate("/Factory/ServiceNow/eccq_insert1?#{args}")
    end
  end

  #
  # Exit method
  #
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "<#{@method}>: [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_STOP
end
