###################################
#
# EVM Automate Method: update_provision_status
#
# Notes: This method updates provisioning status and calls Service-Now/Remedy if applicable
#
# Required inputs: status, status_state
#
###################################
begin
  @method = 'update_provision_status'
  $evm.log("info", "#{@method} - EVM Automate Method Started")

  # Turn of verbose logging
  @debug = true


  ###################################
  #
  # Method: call_servicenow
  #
  ###################################
  def call_servicenow(prov, ws_values, server, state, step, status, status_state)

    # Set request_id from Service-Now unless nil set to provisioning id
    request_id = ws_values[:request_id] || ws_values[:requestid] || prov['id']
    item_id    = ws_values[:item_id]    || ws_values[:itemid] || prov['id']
    friendly_itemid = ws_values[:friendly_itemid]

    $evm.log("info","#{@method} - RequestID:<#{request_id}> ItemID:<#{item_id}> Friendly ItemID:<#{friendly_itemid}>") if @debug

    unless request_id.nil? || item_id.nil?

      message = status_state
      # state_retries = $evm.root['ae_state_retries'] || 0

      agent         = "EVM Automate Provisioning"
      topic         = "Item:#{friendly_itemid} | EVM:#{prov['id']} | StateMachine:#{state}"
      name          = "Step:#{step} | State:#{status_state}"
      source        = "EVM Region:#{server.region_number} | Hostname:#{server.hostname} | IP:#{server.ipaddress}"

      require 'uri'
      args = "agent=#{agent}&topic=#{topic}&source=#{source}&name=#{name}&id=#{prov['id']}&request_id=#{request_id}&item_id=#{item_id}&message=#{message}&step=#{step}&status=#{status}"
      #args = "topic=#{topic}&id=#{id}&request_id=#{request_id}&item_id=#{item_id}&message=#{message}&step=#{step}&status=#{status}"
      args = URI.escape(args)

      $evm.log("info","#{@method} - Calling Service-Now with arguments: #{args}")
      $evm.instantiate("/Factory/ServiceNow/eccq_insert1?#{args}")
    end
  end


  ####################################
  #
  # Method: call_remedy
  #
  ####################################
  def call_remedy(prov, ws_values, server, state, step, status, status_state)
    parent_id = ws_values[:parent_id]
    $evm.log("info","#{@method} - Parent_ID:<#{parent_id}>") if @debug

    unless parent_id.nil?
      comment         =  "#{prov.get_option(:vm_target_name)} - #{status}"
      $evm.log("info","#{@method} - Step:<#{step}>") if @debug


      # Build provisioning object hash prov[:submitComments] that contains comments
      $evm.log("info","#{@method} - Building provisioning object hash prov['submit_comments'] that contains comments") if @debug
      prov['submit_comments'] = {
        :comment => comment,
        :subject => 'Status Update',
        :submitter => 'EVM',
        :parent_id => parent_id
      }

      $evm.log("info","#{@method} - Calling Remedy_SubmitComment with the following parameters:<#{prov['submit_comments'].inspect}>") if @debug
      $evm.instantiate("/Integration/Remedy/Remedy_SubmitComment")

      # Reset submitComments Hash
      prov['submit_comments'] = {}
    end
  end


  ###################################
  #
  # Method: boolean
  #
  ###################################
  def boolean(string)
    return true if string == true || string =~ (/(true|t|yes|y|1)$/i)
    return false if string == false || string.nil? || string =~ (/(false|f|no|n|0)$/i)

    # Return false if string does not match any of the above
    $evm.log("info","#{@method} - Invalid boolean string:<#{string}> detected. Returning false") if @debug
    return false
  end


  # Initialize variables
  prov   = $evm.root['miq_provision']

  # Set servicenow to true to always post status to service-now
  servicenow = false

  # Set remedy to true to always post status to service-now
  remedy = false

  # Get variables from Server object
  server = $evm.root['miq_server']

  # Get State Machine
  state = $evm.current_object.class_name

  # Get current step
  step = $evm.current_object.current_field_name

  # Get status from input field status
  status = $evm.inputs['status']

  # Get status_state ['on_entry', 'on_exit', 'on_error'] from input field
  status_state = $evm.inputs['status_state']

  $evm.log("info", "#{@method} - Server:<#{server.name}> Ae_Result:<#{$evm.root['ae_result']}> State:<#{state}> Step:<#{step}> Status_State:<#{status_state}> Status:<#{status}>")

  # Update Status for on_entry,on_exit
  if $evm.root['ae_result'] == 'ok'

    # Check to see if provisioning is complete
    if status == 'provision_complete'
      message      = 'VM Provisioned Successfully'
      prov.finished(message)
    end
    prov.message = status
  end


  # Update Status for on_error
  if $evm.root['ae_result'] == 'error'
    prov.message = status
  end


  # Call external services to post status
  if prov.options.has_key?(:ws_values)
    ws_values  = prov.options[:ws_values]

    # Check if ws_values[:servicenow] is set to true
    if boolean(ws_values[:servicenow]) || servicenow
      call_servicenow(prov, ws_values, server, state, step, status, status_state)
    end

    # Check if ws_values[:remedy] is set to true
    if boolean(ws_values[:remedy]) || remedy
      call_remedy(prov, ws_values, server, state, step, status, status_state)
    end
  end


  #
  # Exit method
  #
  $evm.log("info", "#{@method} - EVM Automate Method Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "#{@method} - [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_STOP
end
