###################################
#
# EVM Automate Method: vdi_wait_for_hostname
#
# Notes: This method waits for the VM's hostname, then starts the VM
#
###################################
begin
  @method = 'vdi_wait_for_hostname'
  $evm.log("info", "#{@method} - EVM Automate Method Started")

  # Turn of verbose logging
  @debug = true

  prov   = $evm.root["miq_provision"]
  result = 'ok'

  # Determine if the Desktop tab was enabled and a valid VDI Farm selected.
  vdi_farm = prov.vdi_farm
  vdi_vendor = vdi_farm.nil? ? nil : vdi_farm.vendor
  if vdi_vendor == "citrix"
    result = 'retry'
    full_hostname = prov.vm.hostnames.first
    unless full_hostname.nil?
      if full_hostname.to_s.split('.').length > 1
        result = 'ok'
      end
    end
    $evm.log("info", "#{@method} - VM Hostname:  #{full_hostname.inspect}") if @debug
  else
    result = 'ok'
  end

  $evm.log('info', "#{@method} - VDI Hostname check returned <#{result}>") if @debug

  case result
  when 'error'
    $evm.root['ae_result'] = 'error'
    reason = $evm.root['miq_provision'].message
    reason = reason[7..-1] if reason[0..6] == 'Error: '
    $evm.root['ae_reason'] = reason
  when 'retry'
    prov.vm.start unless prov.vm.power_state == 'on'
    prov.vm.refresh
    $evm.root['ae_result']         = 'retry'
    $evm.root['ae_retry_interval'] = '60.seconds'
  when 'ok'
    # Bump State
    $evm.root['ae_result'] = 'ok'
  end

  #
  # Exit method
  #
  $evm.log("info", "#{@method} - EVM Automate Method Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "#{@method} - [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
