###################################
#
# EVM Automate Method: ipam_release_ip_from_file
#
# Notes: This method is used to release an IP Address from an ASCII file living on an appliance
#
###################################
begin
  @method = 'ipam_release_ip_from_file'
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Started")

  # Turn of verbose logging
  @debug = true

  #
  # Find the VM
  #
  vm = $evm.root['vm']
  raise "VM not found" if vm.nil?


  vm_hostname = vm.name.to_s.gsub(/\W/, '_')
  $evm.log("info", "VM Hostname : <#{vm_hostname}>")
  raise "VM Hostname not found" if vm_hostname.nil?


  # Specify filename to read
  # File must contain the following construct
  # VLAN Name  IP Addr    Gateway       Router  hostname | free
  ###########  #########  ############, ####### ######
  # VM Network,10.1.1.10,255.255.255.0,10.1.1.1,[evm1234, free]
  fname = "/var/www/miq/iplist.csv"
  raise "File '#{fname}' does not exist" unless File.exist?(fname)


  # Regular Expression to match the following construct:
  # VLAN Name ,IP Addr  Subnet Mask  ,Gateway , [used,free]
  # VM Network,10.1.1.1,255.255.255.0,10.1.1.1,used
  regex_line = /^(vlan.*|vm\snet.*|VM2.*),(\d*).(\d*).(\d*).(\d*),(\d*).(\d*).(\d*).(\d*),(\d*).(\d*).(\d*).(\d*),(\w*)/i
  $evm.log("info", "regex_line: <#{regex_line}>") if @debug

  File.open(fname) do |file|
    file.each do |line|
      # convert everything to lowercase and strip all leading/trailing whitespaces
      line = line.strip

      # if the regular expression successfully matches else skip the line
      if regex_line =~ line
        vlan    = $1
        octet1  = $2
        octet2  = $3
        octet3  = $4
        octet4  = $5
        mask1   = $6
        mask2   = $7
        mask3   = $8
        mask4   = $9
        gate1   = $10
        gate2   = $11
        gate3   = $12
        gate4   = $13
        tmp_name  = $14
        ipaddr  = "#{octet1}.#{octet2}.#{octet3}.#{octet4}"
        submask = "#{mask1}.#{mask2}.#{mask3}.#{mask4}"
        gateway = "#{gate1}.#{gate2}.#{gate3}.#{gate4}"

        $evm.log("info", "Processing Line: <#{line}> VLAN: #{vlan} IP: <#{ipaddr}> Mask: <#{submask}> Gateway: <#{gateway}> Name: <#{tmp_name}>") if @debug

        # if the VM hostname matches the line then set status to free
        if tmp_name.eql?(vm_hostname)
          newline = "#{vlan},#{ipaddr},#{submask},#{gateway},free"
          $evm.log("info", "Replacing Line: <#{line},> with : <#{newline}> in file: <#{fname}")

          # Use sed to do an in-place subtitution
          `sed -i 's/#{vlan},#{octet1}\.#{octet2}\.#{octet3}\.#{octet4},#{mask1}\.#{mask2}\.#{mask3}\.#{mask4},#{gate1}\.#{gate2}\.#{gate3}\.#{gate4},#{tmp_name}/#{vlan},#{octet1}\.#{octet2}\.#{octet3}\.#{octet4},#{mask1}\.#{mask2}\.#{mask3}\.#{mask4},#{gate1}\.#{gate2}\.#{gate3}\.#{gate4},free/' #{fname}`
          exit_method(begin_time)
        else
          $evm.log("info", "Cannot replace line: <#{line}> Name: <#{tmp_name}> VM Name: <#{vm_hostname}>") if @debug
        end #if tmp_name
      else
        $evm.log("info", "Skipping Line: <#{line}> does not match criteria") if @debug
      end # if regex_line
    end # file.each do
  end # file open do

  #
  # Exit method
  #
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "<#{@method}>: [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
