###################################
#
# EVM Automate Method: VMware_HotAdd_CPU
#
# This method is used to modify vCPUs to an existing VM running on VMware
#
# Inputs: $evm.root['vm'], cpus
#
###################################
# Method for logging
def log(level, message)
  @method = 'VMware_HotAdd_CPU'
  @debug = true
  $evm.log(level, "#{@method} - #{message}") if @debug
end

begin
  log(:info, 'EVM Automate Method Started')

  # Get vm object from root
  vm = $evm.root['vm']
  raise MissingVMObject, "Missing $evm.root['vm'] object" unless vm.nil?

  # Get the number of cpus from root
  cpus = $evm.root['cpus'].to_i
  log(:info, "Detected cpus:<#{cpus}>")

  # Add disk to a VM
  unless cpus.zero?
    log(:info, "Setting number of vCPUs to #{cpus} on VM:<#{vm.name}>")
    vm.set_number_of_cpus(cpus.to_i, :sync => true)
  end

  #
  # Exit method
  #
  log(:info, 'EVM Automate Method Ended')
  exit MIQ_OK

    #
    # Set Ruby rescue behavior
    #
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
