###################################
#
# EVM Automate Method: VMWare_HotResize_Disk
#
# Notes: This method is used to increase the size of the VM's disk
#
# Inputs: $evm.root['vm'], disk_number, size
#
###################################
# Method for logging
def log(level, message)
  @method = 'VMWare_HotResize_Disk'
  $evm.log(level, "#{@method} - #{message}")
end

begin
  log(:info, "EVM Automate Method Started")

  def resizeDisk(vm, disk_number, new_disk_size_in_kb)
    log(:info, "Resize disk functionality currently under review")
  end


  # Get vm object from root
  vm = $evm.root['vm']
  raise "#{@method} - VM object not found" if vm.nil?

  # Get disk_number variable from root if nil convert to zero
  disk_number = $evm.root['disk_number'].to_i

  # Get size variable from root
  size = $evm.root['size'].to_i

  log(:info, "Detected VM:<#{vm.name}> disk_number:<#{disk_number.inspect}> size:<#{size.inspect}>")


  unless size.zero?

    case disk_number

    when 0
      # Convert disk 1 disk size into KB and add the number of GB converted into KB
      new_disk_size_in_kb = (vm.disk_1_size / 1024) + (size * 1024**2)
    when 1
      # Convert disk 2 disk size into KB and add the number of GB converted into KB
      new_disk_size_in_kb = (vm.disk_2_size / 1024) + (size * 1024**2)
    when 2
      # Convert disk 3 disk size into KB and add the number of GB converted into KB
      new_disk_size_in_kb = (vm.disk_3_size / 1024) + (size * 1024**2)
    else
      log(:info, "invalid disk_number detected")
      size = 0
    end

    unless size.zero?
      log(:info, "VM:<#{vm.name}> Increasing Disk #{disk_number} size to #{new_disk_size_in_kb / 1024**2}GB")
      resizeDisk(vm, disk_number, new_disk_size_in_kb)
    end
  end


  #
  # Exit method
  #
  log(:info, "EVM Automate Method Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
