###################################
#
# CFME Automate Method: GroupAdmin_AssignGroupAdmin
#
# Notes: This method tags the user specified with group_id
#
# Inputs: dialog_options['dialog_group_id'], dialog_options['dialog_user_id']
#
###################################
begin
  # Method for logging
  def log(level, message, update_message=false)
    @method = 'GroupAdmin_AssignGroupAdmin'
    $evm.log(level, "#{@method} - #{message}")
    $evm.root['service_template_provision_task'].message = message if $evm.root['service_template_provision_task'] && update_message
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  # remove_service
  def remove_service(service_template_provision_task)
    # Get destination service object for removal
    service = service_template_provision_task.destination
    unless service.nil?
      log(:info, "Removing Service:<#{service.name}> Id:<#{service.id}> from CFME database")
      service.remove_from_vmdb
    end
  end

  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root


  # get the task from root
  service_template_provision_task = $evm.root['service_template_provision_task']

  # Get destination service object
  service = service_template_provision_task.destination
  log(:info, "Detected Service:<#{service.name}> Id:<#{service.id}> Tasks:<#{service_template_provision_task.miq_request_tasks.count}>")

  # Get dialog options from task
  dialog_options = service_template_provision_task.dialog_options
  log(:info, "Inspecting Dialog Options:<#{dialog_options.inspect}>")
  #Inspecting Dialog Options:<{"dialog_group_id"=>619000000000013, "dialog_user_id"=>619000000000002>

  # Get user from Dialog
  unless dialog_options['dialog_user_id'].blank?
    user = $evm.vmdb('user').find_by_id($evm.root['dialog_user_id'])
    log(:info,"Found user:<#{user.name}> user.id:<#{user.id}> via $evm.root['dialog_user_id']")
  else
    user = $evm.root['user']
    log(:info,"Found user:<#{user.name}> user.id:<#{user.id}> via $evm.root['user']")
  end

  unless dialog_options['dialog_group_id'].blank?
    group = $evm.vmdb('miq_group').find_by_id($evm.root['dialog_group_id'])
    log(:info,"Found group:<#{group.description}> group.id:<#{group.id}> via $evm.root['dialog_group_id']")
  else
    group = user.miq_group
    log(:info,"Found group:<#{group.description}> group.id:<#{group.id}> via $evm.root['user'].miq_group")
  end

  category = 'group_admin'
  log(:info, "Assigning new tag:<#{category}/#{group.id}> to User:<#{user.name}>", true)
  user.tag_assign("#{category}/#{group.id}")

  # Exit method
  log(:info, "CFME Automate Method Ended")
  $evm.root['service_template_provision_task'].finished("#{@method} Completed Successfully")
  remove_service(service_template_provision_task) unless $evm.root['service_template_provision_task'].nil?
  exit MIQ_OK

  # Ruby rescue
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}", true)
  $evm.root['service_template_provision_task'].finished("#{err}") unless $evm.root['service_template_provision_task'].nil?
  remove_service($evm.root['service_template_provision_task']) unless $evm.root['service_template_provision_task'].nil?
  exit MIQ_ABORT
end
