###################################
#
# CFME Automate Method: GroupAdmin_AssignGroupAdmin_Email
#
# Notes: This method is used to email the service requester
#
# Model Notes:
# 1. to_email_address - used to specify an email address in the case where the
#    requester does not have a valid email address. To specify more than one email
#    address separate email address with commas. (I.e. admin@company.com,user@company.com)
# 2. from_email_address - used to specify an email address in the event the
#    requester replies to the email
# 3. signature - used to stamp the email with a custom signature
#
###################################
begin
  # Method for logging
  def log(level, message)
    @method = 'GroupAdmin_AssignGroupAdmin_Email'
    $evm.log(level, "#{@method} - #{message}")
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  # emailrequester
  def emailrequester(miq_request, service, dialog_options)
    log(:info, "Requester email logic starting")

    # Get user from Dialog
    unless dialog_options['dialog_user_id'].blank?
      user = $evm.vmdb('user').find_by_id($evm.root['dialog_user_id'])
      log(:info,"Found user:<#{user.name}> user.id:<#{user.id}> via $evm.root['dialog_user_id']")
    else
      user = $evm.root['user']
      log(:info,"Found user:<#{user.name}> user.id:<#{user.id}> via $evm.root['user']")
    end

    unless dialog_options['dialog_group_id'].blank?
      group = $evm.vmdb('miq_group').find_by_id($evm.root['dialog_group_id'])
      log(:info,"Found group:<#{group.description}> group.id:<#{group.id}> via $evm.root['dialog_group_id']")
    else
      group = user.miq_group
      log(:info,"Found group:<#{group.description}> group.id:<#{group.id}> via $evm.root['user'].miq_group")
    end

    # Get requester email else set to nil
    user_email = user.email || $evm.object['to_email_address']

    # Get Owner Email else set to nil
    owner_email = miq_request.options[:owner_email] rescue nil
    log(:info, "owner_mail:<#{owner_email}>")

    # if to is nil then use owner_email
    to = nil
    to ||= user_email || owner_email || $evm.object['to_email_address']

    # Get from_email_address from model unless specified below
    from = nil
    from ||= $evm.object['from_email_address']

    # Get signature from model unless specified below
    signature = nil
    signature ||= $evm.object['signature']

    # Set email subject
    subject = "Service request ID #{miq_request.id} - Your Service Request #{service.name} has completed successfully"

    # Build email body
    body = "Hello, "
    body += "<br><br>User: #{user.name} has been set to Group Admin for #{group.description}."
    body += "<br><br>Dialog parameters used: <br><br>"
    body += dialog_options.collect { |k, v| "#{k.to_s}=#{v}" if k.starts_with?('dialog') && ! v.blank? }.join('<br><br>')
    body += "<br> #{signature}"

    # Send email
    log(:info, "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
    $evm.execute(:send_email, to, from, subject, body)
  end

  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root()

  # get the task from root
  service_template_provision_task = $evm.root['service_template_provision_task']

  # get the request object from root
  miq_request = service_template_provision_task.miq_request
  log(:info, "miq_request id:<#{miq_request.id}> approval_state:<#{miq_request.approval_state} options[:dialog]:<#{miq_request.options[:dialog].inspect}>")

  # Get destination service object
  service = service_template_provision_task.destination
  log(:info, "Detected Service:<#{service.name}> Id:<#{service.id}> Tasks:<#{service_template_provision_task.miq_request_tasks.count}>")

  # Get dialog options from task
  dialog_options = service_template_provision_task.dialog_options
  log(:info, "Inspecting Dialog Options:<#{dialog_options.inspect}>")

  # Email Requester
  emailrequester(miq_request, service, dialog_options)

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Ruby rescue
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_STOP
end
