###################################
#
# CFME Automate Method: GroupAdmin_ListGroups_Dialog
#
# Notes:
#
# Inputs:
#
###################################
begin
  # Method for logging
  def log(level, message)
    @method = 'GroupAdmin_ListGroups_Dialog'
    $evm.log(level, "#{@method} - #{message}")
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  # process_tags - Dynamically create categories and tags
  def process_tags( category, single_value, tag, tag_description )
    # Convert to lower case and replace all non-word characters with underscores
    category_name = category.to_s.downcase.gsub(/\W/, '_')
    tag_name = tag.to_s.downcase.gsub(/\W/, '_')
    log(:info, "Converted category name:<#{category_name}> Converted tag name: <#{tag_name}>")
    # if the category exists else create it
    unless $evm.execute('category_exists?', category_name)
      log(:info, "Category <#{category_name}> doesn't exist, creating category")
      $evm.execute('category_create', :name => category_name, :single_value => single_value, :description => "#{category}")
    end
    # if the tag exists else create it
    unless $evm.execute('tag_exists?', category_name, tag_name)
      log(:info, "Adding new tag <#{tag_name}> description <#{tag_description}> in Category <#{category_name}>")
      $evm.execute('tag_create', category_name, :name => tag_name, :description => "#{tag_description}")
    end
  end

  # start
  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root

  user = $evm.root['user']
  group = user.miq_group

  log(:info,"Detected user:<#{user.name}> group:<#{group.description}>")

  groups_hash = {}
  groups = $evm.vmdb('miq_group').all.each {|group| groups_hash[group.description] = group.id unless group.description.starts_with?('EVM')}
  log(:info, "Inspecting groups_hash:<#{groups_hash.inspect}>")

  # build
  list_values = {
    'sort_by'    => :none,
    'sort_order' => :ascending,
    'data_type'  => :string,
    'required'   => false,
    'values'     => [[nil, nil]] + groups_hash.to_a
  }

  # dynamically create a list of group_admin tags
  category = 'group_admin'
  groups_hash.each {|group_id, group_name| process_tags(category, false, group_name, group_id)}

  $evm.log('info', "Dynamic drop-down parameters: [#{list_values}]")
  list_values.each { |k,v| $evm.object[k] = v }

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Ruby rescue
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
