###################################
#
# CFME Automate Method: GroupAdmin_Quota_Management
#
# Notes: This method tags the current group with values from the inputs below
#
# Inputs: dialog_options['dialog_quota_max_cpu'], dialog_options['dialog_quota_max_memory'], dialog_options['dialog_quota_max_storage']
#
###################################
begin
  # Method for logging
  def log(level, message, update_message=false)
    @method = 'GroupAdmin_Quota_Management'
    $evm.log(level, "#{@method} - #{message}")
    $evm.root['service_template_provision_task'].message = message if $evm.root['service_template_provision_task'] && update_message
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  # remove_service
  def remove_service(service_template_provision_task)
    # Get destination service object for removal
    service = service_template_provision_task.destination
    unless service.nil?
      log(:info, "Removing Service:<#{service.name}> Id:<#{service.id}> from CFME database")
      service.remove_from_vmdb
    end
  end

  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root

  # get the task from root
  service_template_provision_task = $evm.root['service_template_provision_task']

  # Get destination service object
  service = service_template_provision_task.destination
  log(:info, "Detected Service:<#{service.name}> Id:<#{service.id}> Tasks:<#{service_template_provision_task.miq_request_tasks.count}>")

  # Get dialog options from task
  dialog_options = service_template_provision_task.dialog_options
  log(:info, "Inspecting Dialog Options:<#{dialog_options.inspect}>")
  #Inspecting Dialog Options:<{"dialog_quota_max_cpu"=>nil, "dialog_quota_max_memory"=>"81920", "request"=>"clone_to_service"}>

  user = $evm.root['user']
  log(:info,"Found user:<#{user.name}> user.id:<#{user.id}> via $evm.root['user']")

  unless dialog_options['dialog_group_id'].blank?
    group = $evm.vmdb('miq_group').find_by_id($evm.root['dialog_group_id'])
    log(:info,"Found group:<#{group.description}> group.id:<#{group.id}> via $evm.root['dialog_group_id']")
  else
    group = user.miq_group
    log(:info,"Found group:<#{group.description}> group.id:<#{group.id}> via $evm.root['user'].miq_group")
  end

  unless dialog_options['dialog_quota_max_cpu'].blank?
    category = 'quota_max_cpu'
    log(:info, "Assigning new tag:<#{category}/#{dialog_options['dialog_quota_max_cpu']}> to Group:<#{group.description}>", true)
    group.tag_assign("#{category}/#{dialog_options['dialog_quota_max_cpu']}")
  end

  unless dialog_options['dialog_quota_max_memory'].blank?
    category = 'quota_max_memory'
    log(:info, "Assigning new tag:<#{category}/#{dialog_options['dialog_quota_max_memory']}> to Group:<#{group.description}>", true)
    group.tag_assign("#{category}/#{dialog_options['dialog_quota_max_memory']}")
  end

  unless dialog_options['dialog_quota_max_storage'].blank?
    category = 'quota_max_storage'
    log(:info, "Assigning new tag:<#{category}/#{dialog_options['dialog_quota_max_storage']}> to Group:<#{group.description}>", true)
    group.tag_assign("#{category}/#{dialog_options['dialog_quota_max_storage']}")
  end

  # Exit method
  log(:info, "CFME Automate Method Ended")
  $evm.root['service_template_provision_task'].finished("#{@method} Completed Successfully")
  remove_service(service_template_provision_task) unless $evm.root['service_template_provision_task'].nil?
  exit MIQ_OK

  # Ruby rescue
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}", true)
  $evm.root['service_template_provision_task'].finished("#{err}") unless $evm.root['service_template_provision_task'].nil?
  remove_service($evm.root['service_template_provision_task']) unless $evm.root['service_template_provision_task'].nil?
  exit MIQ_ABORT
end
