###################################
#
# CFME Automate Method: Automate_Scheduler
#
# Notes: This method allows you to schedule an automate instantiation
#
# Inputs: $evm.root['dialog_instance_path'], $evm.root['dialog_interval'], $evm.root['dialog_job_name']
#
###################################
begin
  def log(level, msg, update_message=false)
    @method = 'Automate_Scheduler'
    $evm.log(level,"#{@method}: #{msg}")
    $evm.root['service_template_provision_task'].message = msg if $evm.root['service_template_provision_task'] && update_message
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  # basic retry logic
  def retry_method(retry_time=1.minute)
    log(:info, "Sleeping for #{retry_time} seconds", true)
    $evm.root['ae_result'] = 'retry'
    $evm.root['ae_retry_interval'] = retry_time
    exit MIQ_OK
  end

  # remove_service
  def remove_service(service)
    unless service.nil?
      log(:info, "Removing Service:<#{service.name}> Id:<#{service.id}> from CFME database")
      service.remove_from_vmdb
    end
  end

  # name_service - name the service to avoid duplicate names
  def name_service(service, new_service_name=nil)
    unless new_service_name.blank?
      new_service_name = "#{new_service_name}-#{Time.now.strftime('%Y%m%d-%H%M%S')}"
      log(:info, "Changing Service name:<#{service.name}> to <#{new_service_name}>")
    else
      new_service_name = "#{service.name}-#{Time.now.strftime('%Y%m%d-%H%M%S')}"
      log(:info, "Changing Service name:<#{service.name}> to <#{new_service_name}>")
    end
    service.name = new_service_name
  end

  # process_tags - Dynamically create categories and tags
  def process_tags( category, single_value, tag )
    # Convert to lower case and replace all non-word characters with underscores
    category_name = category.to_s.downcase.gsub(/\W/, '_')
    tag_name = tag.to_s.downcase.gsub(/\W/, '_')
    log(:info, "Converted category name:<#{category_name}> Converted tag name: <#{tag_name}>")
    # if the category exists else create it
    unless $evm.execute('category_exists?', category_name)
      log(:info, "Category <#{category_name}> doesn't exist, creating category")
      $evm.execute('category_create', :name => category_name, :single_value => single_value, :description => "#{category}")
    end
    # if the tag exists else create it
    unless $evm.execute('tag_exists?', category_name, tag_name)
      log(:info, "Adding new tag <#{tag_name}> in Category <#{category_name}>")
      $evm.execute('tag_create', category_name, :name => tag_name, :description => "#{tag}")
    end
  end

  # Beginning
  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root

  task = $evm.root['service_template_provision_task']
  service = task.destination

  # get dialog parameters from $evm.root
  instance_path = $evm.root['dialog_instance_path']
  raise "$evm.root['dialog_instance_path']" if instance_path.nil?
  interval = $evm.root['dialog_interval'].to_i
  raise "$evm.root['dialog_interval'] missing" if interval.zero?

  category = 'automation_jobs'
  job_name = $evm.root['dialog_job_name']
  raise "$evm.root['dialog_job_name'] missing" if job_name.nil?
  job_name = job_name.to_s.downcase.gsub(/\W/, '_')

  # Name this service
  name_service(service, job_name)

  # create a tag for none
  process_tags(category, true, 'none')

  # create tag for job_name
  process_tags(category, true, job_name)

  log(:info, "Detected service:#{service.name} id:#{service.id} instance_path:#{instance_path} interval:#{interval} job_name:#{job_name}", true)

  if service.tagged_with?(category, job_name)
    log(:info, "Instantiating:#{instance_path}", true)
    $evm.instantiate("#{instance_path}")
    if interval == 1
      interval = 1.minute
    else
      interval = interval.minutes
    end
    log(:info, "Assinging service:#{service.name} category:#{category}> tag:#{job_name}")
    service.tag_assign("#{category}/#{job_name}")
    retry_method(interval)
  else
    log(:info, "Ending scheduler for job:<#{job_name}>", true)
    remove_service(service) unless task.nil?
  end

  # Exit method
  log(:info, "CFME Automate Method Ended")
  task.finished("Automation Scheduler Task Completed")
  exit MIQ_OK

  # Ruby rescue
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  $evm.root['service_template_provision_task'].finished("#{err}")
  exit MIQ_ABORT
end
