###################################
#
# CFME Automate Method: Dialog_AutomateExport
#
# This method is dynamically provides for a service dialog drop down that will
# export a selected automate class.
#
###################################
begin
  # Method for logging
  def log(level, message, update_message=true)
    @method = 'Dialog_AutomateExport'
    $evm.log(level, "#{@method}: #{message}")
    $evm.root['service_template_provision_task'].message = message if $evm.root['service_template_provision_task'] && update_message
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  def run_linux_admin(cmd)
    require 'linux_admin'
    log(:info, "Executing #{cmd}")
    result = LinuxAdmin.run!(cmd)
    log(:info, "Inspecting output: #{result.output.inspect}", false)
    log(:info, "Inspecting error: #{result.error.inspect}", false)
    log(:info, "Inspecting exit_status: #{result.exit_status.inspect}", false)
    #script/rails runner script/rake evm:automate:export NAMESPACE=nimbus/automation CLASS=methods FILE=methods.xml
    return result
  end

  def setup_dialog()

    # sort_by: value / description / none
    dialog_field["sort_by"] = "description"

    # sort_order: ascending / descending
    #dialog_field["sort_order"] = "ascending"

    # data_type: string / integer
    dialog_field["data_type"] = "integer"

    # required: true / false
    #dialog_field["required"] = "true"
    #dialog_field["values"] = {10 => "ten", 20 => "twenty", 30 => "thirty", 40 => "forty"}
    #dialog_field["values"] = [[1, "one"], [2, "two"], [10, "ten"], [50, "fifty"]]

    #dialog_field["default_value"] = 10
  end

  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root

  if $evm.root['dialog_automate_klass'].nil?
    get_classes_cmd = "rake evm:automate:list_class"
    result = run_linux_admin(get_classes_cmd)
    raise "Command: #{get_classes_cmd} failed with #{result.error.inspect} exit_status: #{result.exit_status.inspect}" unless result.exit_status.zero?

    class_array = result.output.gsub(/\r|\t|\s+|\n?/, '').split(',')
    log(:info, "Inspecting class_array: #{class_array.sort.inspect}", false)

    klasses = []
    class_array.each do |klass|
      next if klass.include?('Object')
      klasses << "#{klass}"
    end

    dialog_field = $evm.object
    dialog_field["values"] = klasses.sort

    log(:info, "Inspeting dialog_field: #{dialog_field.inspect}", false)
  else
    stp_task = $evm.root['service_template_provision_task']
    log(:info, "===========================================", false)
    stp_task.attributes.sort.each { |k, v| log(:info, "$evm.root['service_template_provision_task'] Attributes - #{k}: #{v}", false)}

    regex = /^(\w.*)\/(\w*)$/
    if $evm.root['dialog_automate_klass'] =~ regex
      namespace = $1
      klass = $2
      export_cmd = "rake evm:automate:export CLASS=#{klass} NAMESPACE=#{namespace}"
      result = run_linux_admin(export_cmd)
      raise "Command: #{get_classes_cmd} failed with #{result.error.inspect} exit_status: #{result.exit_status.inspect}" unless result.exit_status.zero?
      stp_task.message = "#{result.output}"
    else
      raise "#{$evm.root['dialog_automate_klass']} does not meet regex match"
    end
  end

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Ruby rescue
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  $evm.root['service_template_provision_task'].finished("#{err}") unless $evm.root['service_template_provision_task'].nil?
  exit MIQ_ABORT
end
