###################################
#
# CFME Automate Method: SignificantEvent
#
# Notes: Dump the objects in storage to the automation.log
#
###################################
require 'socket'
require 'uri'

begin
  #########################
  #
  # Method: log
  # Description: Send messages to the log and create a body for email
  #
  ##########################
  def log(level, message)
    $body = "#{$body}#{message}<br>"

    @method = 'SignificantEvent'
    $evm.log(level, "#{@method} - #{message}")
  end

  #########################
  #
  # Method: lab_env?
  # Description: Determine if this is a lab environment
  #
  ##########################
  def lab_env?
    begin
      log(:info, "Is this a lab environment")
      hostname = Socket.gethostbyname(Socket.gethostname).first
      hostname.downcase!
      return hostname.include?("corplab")
    rescue SignalException, StandardError => e
      log(:info, "Unable to tell if this is the lab, defaulting to prod #{e}")
      return false
    end
  end

  #########################
  #
  # Method: dumpRoot
  # Description: Dump Root information
  #
  ##########################
  def dumpRoot
    log(:info, "Root:<$evm.root> Begin Attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attributes - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End Attributes")
    log(:info, "")
  end


  #########################
  #
  # Method: dumpServer
  # Inputs: $evm.root['miq_server']
  # Description: Dump MIQ Server information
  #
  ##########################
  def dumpServer
    log(:info, "Server:<#{$evm.root['miq_server'].name}> Begin Attributes")
    $evm.root['miq_server'].attributes.sort.each { |k, v| log(:info, "Server:<#{$evm.root['miq_server'].name}> Attributes - #{k}: #{v.inspect}")}
    log(:info, "Server:<#{$evm.root['miq_server'].name}> End Attributes")
    log(:info, "")
  end


  #########################
  #
  # Method: dumpUser
  # Inputs: $evm.root['user']
  # Description: Dump User information
  #
  ##########################
  def dumpUser
    user = $evm.root['user']
    unless user.nil?
      att_types = [ 'email', 'last_name', 'name', 'id', 'userid' ]
      log(:info, "User:<#{user.name}> Begin Attributes [user.attributes]")
      user.attributes.sort.each { |k, v|
        if att_types.include?(k.to_s)
          log(:info, "User:<#{user.name}> Attributes - #{k}: #{v.inspect}")
        end}
      log(:info, "User:<#{user.name}> End Attributes [user.attributes]")
      log(:info, "")

      #log(:info, "User:<#{user.name}> Begin Associations [user.associations]")
      #user.associations.sort.each { |assc| log(:info, "User:<#{user.name}> Associations - #{assc}")}
      #log(:info, "User:<#{user.name}> End Associations [user.associations]")
      #log(:info, "")

      #unless user.tags.nil?
      #  log(:info, "User:<#{user.name}> Begin Tags [user.tags]")
      #  user.tags.sort.each { |tag_element| tag_text = tag_element.split('/'); log(:info, "User:<#{user.name}> Category:<#{tag_text.first.inspect}> Tag:<#{tag_text.last.inspect}>")}
      #  log(:info, "User:<#{user.name}> End Tags [user.tags]")
      #  log(:info, "")
      #end

      vcn_types = [ 'ldap_group', 'miq_group_description', 'miq_user_role_name' ]
      log(:info, "User:<#{user.name}> Begin Virtual Columns [user.virtual_column_names]")
      user.virtual_column_names.sort.each { |vcn| 
        if vcn_types.include?(vcn.to_s)
          log(:info, "User:<#{user.name}> Virtual Columns - #{vcn}: #{user.send(vcn).inspect}")
        end} rescue nil
      log(:info, "User:<#{user.name}> End Virtual Columns [user.virtual_column_names]")
      log(:info, "")
    end
  end


  #########################
  #
  # Method: dumpGroup
  # Inputs: $evm.root['user'].miq_group
  # Description: Dump User's Group information
  #
  ##########################
  def dumpGroup
    user = $evm.root['user']
    unless user.nil?
      miq_group = user.miq_group
      unless miq_group.nil?
        log(:info, "Group:<#{miq_group.description}> Begin Attributes [miq_group.attributes]")
        miq_group.attributes.sort.each { |k, v| log(:info, "Group:<#{miq_group.description}> Attributes - #{k}: #{v.inspect}")} unless $evm.root['user'].miq_group.nil?
        log(:info, "Group:<#{miq_group.description}> End Attributes [miq_group.attributes]")
        log(:info, "")

        log(:info, "Group:<#{miq_group.description}> Begin Associations [miq_group.associations]")
        miq_group.associations.sort.each { |assc| log(:info, "Group:<#{miq_group.description}> Associations - #{assc}")}
        log(:info, "Group:<#{miq_group.description}> End Associations [miq_group.associations]")
        log(:info, "")

        unless miq_group.tags.nil?
          log(:info, "Group:<#{miq_group.description}> Begin Tags [miq_group.tags]")
          miq_group.tags.sort.each { |tag_element| tag_text = tag_element.split('/'); log(:info, "Group:<#{miq_group.description}> Category:<#{tag_text.first.inspect}> Tag:<#{tag_text.last.inspect}>")}
          log(:info, "Group:<#{miq_group.description}> End Tags [miq_group.tags]")
          log(:info, "")
        end

        #log(:info, "Group:<#{miq_group.description}> Begin Virtual Columns [miq_group.virtual_column_names]")
        #miq_group.virtual_column_names.sort.each { |vcn| log(:info, "Group:<#{miq_group.description}> Virtual Columns - #{vcn}: #{miq_group.send(vcn).inspect}")} rescue nil
        #log(:info, "Group:<#{miq_group.description}> End Virtual Columns [miq_group.virtual_column_names]")
        #log(:info, "")
      end
    end
  end

  #########################
  #
  # Method: dumpHost
  # Inputs: $evm.root['host']
  # Description: Dump Host information
  #
  ##########################
  def dumpHost(host)
    log(:info, "Host:<#{host.name}> Begin Attributes [host.attributes]")
    host.attributes.sort.each { |k, v| log(:info, "Host:<#{host.name}> Attributes - #{k}: #{v.inspect}")}
    log(:info, "Host:<#{host.name}> Attributes - Username: #{host.authentication_userid}")
    log(:info, "Host:<#{host.name}> Attributes - Password: #{host.authentication_password}")
    log(:info, "Host:<#{host.name}> End Attributes [host.attributes]")
    log(:info, "")

    log(:info, "Host:<#{host.name}> Begin Associations [host.associations]")
    host.associations.sort.each { |assc| log(:info, "Host:<#{host.name}> Associations - #{assc}")}
    log(:info, "Host:<#{host.name}> End Associations [host.associations]")
    log(:info, "")

    log(:info, "Host:<#{host.name}> Begin Hardware [host.hardware]")
    host.hardware.attributes.each { |k,v| log(:info, "Host:<#{host.name}> Hardware - #{k}: #{v.inspect}")}
    log(:info, "Host:<#{host.name}> End Hardware [host.hardware]")
    log(:info, "")

    log(:info, "Host:<#{host.name}> Begin Lans [host.lans]")
    host.lans.each { |lan| lan.attributes.sort.each { |k,v| log(:info, "Host:<#{host.name}> Lan:<#{lan.name}> - #{k}: #{v.inspect}")}}
    log(:info, "Host:<#{host.name}> End Lans [host.lans]")
    log(:info, "")

    log(:info, "Host:<#{host.name}> Begin Switches [host.switches]")
    host.switches.each { |switch| switch.attributes.sort.each { |k,v| log(:info, "Host:<#{host.name}> Swtich:<#{switch.name}> - #{k}: #{v.inspect}")}}
    log(:info, "Host:<#{host.name}> End Switches [host.switches]")
    log(:info, "")

    log(:info, "Host:<#{host.name}> Begin Operating System [host.operating_system]")
    host.operating_system.attributes.sort.each { |k, v| log(:info, "Host:<#{host.name}> Operating System - #{k}: #{v.inspect}")}
    log(:info, "Host:<#{host.name}> End Operating System [host.operating_system]")
    log(:info, "")

    log(:info, "Host:<#{host.name}> Begin Guest Applications [host.guest_applications]")
    host.guest_applications.each { |guest_app| guest_app.attributes.sort.each { |k, v| log(:info, "Host:<#{host.name}> Guest Application:<#{guest_app.name}> - #{k}: #{v.inspect}")}}
    log(:info, "Host:<#{host.name}> End Guest Applications [host.guest_applications]")
    log(:info, "")

    unless host.tags.nil?
      log(:info, "Host:<#{host.name}> Begin Tags [host.tags]")
      host.tags.sort.each { |tag_element| tag_text = tag_element.split('/'); log(:info, "Host:<#{host.name}> Category:<#{tag_text.first.inspect}> Tag:<#{tag_text.last.inspect}>")}
      log(:info, "Host:<#{host.name}> End Tags [host.tags]")
      log(:info, "")
    end

    log(:info, "Host:<#{host.name}> Begin Virtual Columns [host.virtual_column_names]")
    host.virtual_column_names.sort.each { |vcn| log(:info, "Host:<#{host.name}> Virtual Columns - #{vcn}: #{host.send(vcn).inspect}")}
    log(:info, "Host:<#{host.name}> End Virtual Columns [host.virtual_column_names]")
    log(:info, "")
  end


  #########################
  #
  # Method: dumpVM
  # Inputs: $evm.root['vm']
  # Description: Dump VM information
  #
  ##########################
  def dumpVM(vm)
    #log(:info, "VM:<#{vm.name}> Begin Attributes [vm.attributes]")
    #vm.attributes.sort.each { |k, v| log(:info, "VM:<#{vm.name}> Attributes - #{k}: #{v.inspect}")}
    #log(:info, "VM:<#{vm.name}> End Attributes [vm.attributes]")
    #log(:info, "")

    #log(:info, "VM:<#{vm.name}> Begin Associations [vm.associations]")
    #vm.associations.sort.each { |assc| log(:info, "VM:<#{vm.name}> Associations - #{assc}")}
    #log(:info, "VM:<#{vm.name}> End Associations [vm.associations]")
    #log(:info, "")

    #log(:info, "VM:<#{vm.name}> Begin Hardware Attributes [vm.hardware]")
    #vm.hardware.attributes.each { |k,v| log(:info, "VM:<#{vm.name}> Hardware - #{k}: #{v.inspect}")}
    #log(:info, "VM:<#{vm.name}> End Hardware Attributes [vm.hardware]")
    #log(:info, "")

    #log(:info, "VM:<#{vm.name}> Begin Hardware Associations [vm.hardware.associations]")
    #vm.hardware.associations.sort.each { |assc| log(:info, "VM:<#{vm.name}> hardware Associations - #{assc}")}
    #log(:info, "VM:<#{vm.name}> End hardware Associations [vm.hardware.associations]")
    #log(:info, "")

    #log(:info, "VM:<#{vm.name}> Begin Neworks [vm.hardware.nics]")
    #vm.hardware.nics.each { |nic| nic.attributes.sort.each { |k,v| log(:info, "VM:<#{vm.name}> VLAN:<#{nic.device_name}> - #{k}: #{v.inspect}")}}
    #log(:info, "VM:<#{vm.name}> End Networks [vm.hardware.nics]")
    #log(:info, "")

    #unless vm.ext_management_system.nil?
    #  log(:info, "VM:<#{vm.name}> Begin EMS [vm.ext_management_system]")
    #  vm.ext_management_system.attributes.sort.each { |ems_k, ems_v| log(:info, "VM:<#{vm.name}> EMS:<#{vm.ext_management_system.name}> #{ems_k} - #{ems_v.inspect}")}
    #  log(:info, "VM:<#{vm.name}> End EMS [vm.ext_management_system]")
    #  log(:info, "")
    #end

    unless vm.owner.nil?
      owner_types = [ 'name', 'email', 'userid' ]
      log(:info, "VM:<#{vm.name}> Begin Owner [vm.owner]")
      vm.owner.attributes.each { |k,v|
        if owner_types.include?(k.to_s)
          log(:info, "VM:<#{vm.name}> Owner - #{k}: #{v.inspect}")
        end}
      log(:info, "VM:<#{vm.name}> End Owner [vm.owner]")
      log(:info, "")
    end

    unless vm.operating_system.nil?
      os_types = [ 'product_name' ]
      log(:info, "VM:<#{vm.name}> Begin Operating System [vm.operating_system]")
      vm.operating_system.attributes.sort.each { |k, v|
        if os_types.include?(k.to_s)
          log(:info, "VM:<#{vm.name}> Operating System - #{k}: #{v.inspect}")
        end}
      log(:info, "VM:<#{vm.name}> End Operating System [vm.operating_system]")
      log(:info, "")
    end

    #unless vm.guest_applications.nil?
    #  log(:info, "VM:<#{vm.name}> Begin Guest Applications [vm.guest_applications]")
    #  vm.guest_applications.each { |guest_app| guest_app.attributes.sort.each { |k, v| log(:info, "VM:<#{vm.name}> Guest Application:<#{guest_app.name}> - #{k}: #{v.inspect}")}} unless vm.guest_applications.nil?
    #  log(:info, "VM:<#{vm.name}> End Guest Applications [vm.guest_applications]")
    #  log(:info, "")
    #end

    #unless vm.snapshots.nil?
    #  log(:info, "VM:<#{vm.name}> Begin Snapshots [vm.snapshots]")
    #  vm.snapshots.each { |ss| ss.attributes.sort.each { |k, v| log(:info, "VM:<#{vm.name}> Snapshot:<#{ss.name}> - #{k}: #{v.inspect}")}} unless vm.snapshots.nil?
    #  log(:info, "VM:<#{vm.name}> End Snapshots [vm.snapshots]")
    #  log(:info, "")
    #end

    #unless vm.storage.nil?
    #  log(:info, "VM:<#{vm.name}> Begin VM Storage [vm.storage]")
    #  vm.storage.attributes.sort.each { |stor_k, stor_v| log(:info, "VM:<#{vm.name}> Storage:<#{vm.storage.name}> #{stor_k} - #{stor_v.inspect}")}
    #  log(:info, "VM:<#{vm.name}> End VM Storage [vm.storage]")
    #  log(:info, "")
    #end

    unless vm.tags.nil?
      log(:info, "VM:<#{vm.name}> Begin Tags [vm.tags]")
      vm.tags.sort.each { |tag_element| tag_text = tag_element.split('/'); log(:info, "VM:<#{vm.name}> Category:<#{tag_text.first.inspect}> Tag:<#{tag_text.last.inspect}>")}
      log(:info, "VM:<#{vm.name}> End Tags [vm.tags]")
      log(:info, "")
    end

    vcn_types = [ 'host_name', 'hostnames', 'ipaddresses' ]
    log(:info, "VM:<#{vm.name}> Begin Virtual Columns [vm.virtual_column_names]")
    vm.virtual_column_names.sort.each { |vcn|
      if vcn_types.include?(vcn.to_s)
        log(:info, "VM:<#{vm.name}> Virtual Columns - #{vcn}: #{vm.send(vcn).inspect}")
      end}
    log(:info, "VM:<#{vm.name}> End Virtual Columns [vm.virtual_column_names]")
    log(:info, "")
  end


  #########################
  #
  # Method: dumpCluster
  # Inputs: $evm.root['ems_cluster']
  # Description: Dump Cluster information
  #
  ##########################
  def dumpCluster(cluster)
    log(:info, "Cluster:<#{cluster.name}> Begin Attributes [cluster.attributes]")
    cluster.attributes.sort.each { |k, v| log(:info, "Cluster:<#{cluster.name}> Attributes - #{k}: #{v.inspect}")}
    log(:info, "Cluster:<#{cluster.name}> End Attributes [cluster.attributes]")
    log(:info, "")

    log(:info, "Cluster:<#{cluster.name}> Begin Associations [cluster.associations]")
    cluster.associations.sort.each { |assc| log(:info, "Cluster:<#{cluster.name}> Associations - #{assc}")}
    log(:info, "Cluster:<#{cluster.name}> End Associations [cluster.associations]")
    log(:info, "")

    unless cluster.tags.nil?
      log(:info, "Cluster:<#{cluster.name}> Begin Tags [cluster.tags]")
      cluster.tags.sort.each { |tag_element| tag_text = tag_element.split('/'); log(:info, "Cluster:<#{cluster.name}> Category:<#{tag_text.first.inspect}> Tag:<#{tag_text.last.inspect}>")}
      log(:info, "Cluster:<#{cluster.name}> End Tags [cluster.tags]")
      log(:info, "")
    end

    log(:info, "Cluster:<#{cluster.name}> Begin Virtual Columns [cluster.virtual_column_names]")
    cluster.virtual_column_names.sort.each { |vcn| log(:info, "Cluster:<#{cluster.name}> Virtual Columns - #{vcn}: #{cluster.send(vcn)}")}
    log(:info, "Cluster:<#{cluster.name}> End Virtual Columns [cluster.virtual_column_names]")
    log(:info, "")
  end


  #########################
  #
  # Method: dumpEMS
  # Inputs: $evm.root['ext_managaement_system']
  # Description: Dump Provider information
  #
  ##########################
  def dumpEMS(ems)
    log(:info, "Provider:<#{ems.name}> Begin Attributes [ems.attributes]")
    ems.attributes.sort.each { |k, v| log(:info, "Provider:<#{ems.name}> Attributes - #{k}: #{v.inspect}")}
    log(:info, "Provider:<#{ems.name}> Attributes - Username: #{ems.authentication_userid}")
    log(:info, "Provider:<#{ems.name}> Attributes - Password: #{ems.authentication_password}")
    log(:info, "Provider:<#{ems.name}> Attributes - Encrypted Password: #{ems.authentication_password_encrypted}")
    log(:info, "Provider:<#{ems.name}> End Attributes [ems.attributes]")
    log(:info, "")

    log(:info, "Provider:<#{ems.name}> Begin Associations [ems.associations]")
    ems.associations.sort.each { |assc| log(:info, "Provider:<#{ems.name}> Associations - #{assc}")}
    log(:info, "Provider:<#{ems.name}> End Associations [ems.associations]")
    log(:info, "")

    log(:info, "Provider:<#{ems.name}> Begin Provider Folders [ems.ems_folders]")
    ems.ems_folders.each { |ef| ef.attributes.sort.each { |k,v| log(:info, "Provider:<#{ems.name}> Provider Folder:<#{ef.name}> #{k}: #{v.inspect}")}}
    log(:info, "Provider:<#{ems.name}> End Provider Folders [ems.ems_folders]")
    log(:info, "")

    unless ems.security_groups.nil?
      log(:info, "Provider:<#{ems.name}> Begin Security Groups [ems.security_groups]")
      ems.security_groups.each { |sg| sg.attributes.sort.each { |k,v| log(:info, "Provider:<#{ems.name}> Provider Folder:<#{sg.name}> #{k}: #{v.inspect}")}}
      log(:info, "Provider:<#{ems.name}> End Security Groups [ems.security_groups]")
      log(:info, "")
    end

    unless ems.tags.nil?
      log(:info, "Provider:<#{ems.name}> Begin Tags [ems.tags]")
      ems.tags.sort.each { |tag_element| tag_text = tag_element.split('/'); log(:info, "Provider:<#{ems.name}> Category:<#{tag_text.first.inspect}> Tag:<#{tag_text.last.inspect}>")}
      log(:info, "Provider:<#{ems.name}> End Tags [ems.tags]")
      log(:info, "")
    end

    log(:info, "Provider:<#{ems.name}> Begin Virtual Columns [ems.virtual_column_names]")
    ems.virtual_column_names.sort.each { |vcn| log(:info, "Provider:<#{ems.name}> Virtual Columns - #{vcn}: #{ems.send(vcn)}")}
    log(:info, "Provider:<#{ems.name}> End Virtual Columns [ems.virtual_column_names]")
    log(:info, "")
  end


  #########################
  #
  # Method: dumpService
  # Inputs: $evm.root['service']
  # Description: Dump Service information
  #
  ##########################
  def dumpService(service)
    att_types = [ 'created_at', 'description', 'name', 'id', 'evm_owner_id' ]
    log(:info, "Service:<#{service.name}> Begin Attributes [service.attributes]")
    service.attributes.sort.each { |k, v|
      if att_types.include?(k.to_s)
        log(:info, "Service:#{service.name} Attributes - #{k}: #{v.inspect}")
      end}
    log(:info, "Service:<#{service.name}> End Attributes [service.attributes]")
    log(:info, "")

    #log(:info, "Service:<#{service.name}> Begin Associations [service.associations]")
    #service.associations.sort.each { |assc| log(:info, "Service:#{service.name} Associations - #{assc}")}
    #log(:info, "Service:<#{service.name}> End Associations [service.associations]")
    #log(:info, "")

    log(:info, "Service:<#{service.name}> Begin vms [service.vms]")
    service.vms.each { |vm| log(:info, "Service:#{service.name} VM - name:#{vm.name} guid:<#{vm.guid}>")}
    log(:info, "Service:<#{service.name}> End vms [service.vms]")
    log(:info, "")

    #log(:info, "Service:<#{service.name}> Begin direct_vms [service.direct_vms]")
    #service.direct_vms.each { |vm| log(:info, "Service:#{service.name} Direct VM - name:#{vm.name} guid:<#{vm.guid}>")}
    #log(:info, "Service:<#{service.name}> End direct_vms [service.direct_vms]")
    #log(:info, "")

    #log(:info, "Service:<#{service.name}> Begin indirect_vms [service.indirect_vms]")
    #service.indirect_vms.each { |vm| log(:info, "Service:#{service.name} Indirect VM - name:{#vm.name} guid:<#{vm.guid}>")}
    #log(:info, "Service:<#{service.name}> End indirect_vms [service.indirect_vms]")
    #log(:info, "")

    #log(:info, "Service:<#{service.name}> Begin all_service_children [service.all_service_children]")
    #og(:info, "Service:<#{service.name}> All Service Children:<#{service.all_service_children.inspect}>")
    #log(:info, "Service:<#{service.name}> End all_service_children [service.all_service_children]")
    #log(:info, "")

    #log(:info, "Service:<#{service.name}> Begin direct_service_children [service.direct_service_children]")
    #log(:info, "Service:<#{service.name}> Direct Service Children:<#{service.direct_service_children.inspect}>")
    #log(:info, "Service:<#{service.name}> End all_service_children [service.direct_service_children]")
    #log(:info, "")

    unless service.tags.nil?
      log(:info, "Service:<#{service.name}> Begin Tags [service.tags]")
      service.tags.sort.each { |tag_element| tag_text = tag_element.split('/'); log(:info, "Service:<#{service.name}> Category:<#{tag_text.first.inspect}> Tag:<#{tag_text.last.inspect}>")}
      log(:info, "Service:<#{service.name}> End Tags [service.tags]")
      log(:info, "")
    end

    vcn_types = [ 'evm_owner_email', 'evm_owner_name', 'evm_owner_userid', 'owning_ldap_group' ]
    log(:info, "Service:<#{service.name}> Begin Virtual Columns [service.virtual_column_names]")
    service.virtual_column_names.sort.each { |vcn|
      if vcn_types.include?(vcn.to_s)
        log(:info, "Service:<#{service.name}> Virtual Columns - #{vcn}: #{service.send(vcn)}")
      end} rescue nil
    log(:info, "Service:<#{service.name}> End Virtual Columns [service.virtual_column_names]")
    log(:info, "")
  end


  #########################
  #
  # Method: dumpStorage
  # Inputs: $evm.root['storage']
  # Description: Dump Storage information
  #
  ##########################
  def dumpStorage(storage)
    log(:info, "Storage:<#{storage.name}> Begin Attributes [storage.attributes]")
    storage.attributes.sort.each { |k, v| log(:info, "Storage:<#{storage.name}> Attributes - #{k}: #{v.inspect}")}
    log(:info, "Storage:<#{storage.name}> End Attributes [storage.attributes]")
    log(:info, "")

    log(:info, "Storage:<#{storage.name}> Begin Associations [storage.associations]")
    storage.associations.sort.each { |assc| log(:info, "Storage:<#{storage.name}> Associations - #{assc}")}
    log(:info, "Storage:<#{storage.name}> End Associations [storage.associations]")
    log(:info, "")

    unless storage.tags.nil?
      log(:info, "Storage:<#{storage.name}> Begin Tags [storage.tags]")
      storage.tags.sort.each { |tag_element| tag_text = tag_element.split('/'); log(:info, "Storage:<#{storage.name}> Category:<#{tag_text.first.inspect}> Tag:<#{tag_text.last.inspect}>")}
      log(:info, "Storage:<#{storage.name}> End Tags [storage.tags]")
      log(:info, "")
    end

    log(:info, "Storage:<#{storage.name}> Begin Virtual Columns [storage.virtual_column_names]")
    storage.virtual_column_names.sort.each { |vcn| log(:info, "Storage:<#{storage.name}> Virtual Columns - #{vcn}: #{storage.send(vcn)}")}
    log(:info, "Storage:<#{storage.name}> End Virtual Columns [storage.virtual_column_names]")
    log(:info, "")
  end


  log(:info, "CFME Automate Method Started")

  # to_email_address from owner.email then from model if nil
  to = nil
  to ||= $evm.object['to_email_address']
  to ||= "TechCloudforms_Support@intuit.com"

  from = nil
  from ||= $evm.object['from_email_address']
  from ||= "cloudforms@intuit.com"

  if lab_env?
    from = "cloudforms_lab@intuit.com"
  end

  # List the types of object we will try to detect
  #obj_types = %w{ vm host storage ems_cluster ext_management_system service }
  obj_types = %w{ vm service host ems_cluster ext_management_system, storage}
  obj_type = $evm.root.attributes.detect { |k,v| obj_types.include?(k)}

  eventType = "Unknown Event"

  # If obj_type is NOT nil
  unless obj_type.nil?
    rootobj = obj_type.first
    obj = obj_type.second
    case rootobj
    when 'vm' then eventType = "VM Retirement"
    when 'service' then eventType = "Service Retirement"
    end
  end

  # set hard coded email stuff
  subject = "Cloudforms Significant Event Notification - #{eventType}"
  $body = "Significant Event Notice - #{eventType}<br>***************************************<br><br>"

  # dump root object attributes
  #dumpRoot

  # dump miq_server object attributes
  #dumpServer

  # dump user attributes
  dumpUser

  # dump miq_group attributes
  dumpGroup

  # If obj_type is NOT nil
  unless obj_type.nil?
    #rootobj = obj_type.first
    #obj = obj_type.second
    log(:info, "Detected Object:<#{rootobj}>")
    log(:info, "")

    case rootobj
    when 'host' then dumpHost(obj)
    when 'vm' then dumpVM(obj)
    when 'ems_cluster' then dumpCluster(obj)
    when 'ext_management_system' then dumpEMS(obj)
    when 'service' then dumpService(obj)
    when 'storage' then dumpStorage(obj)
    end
  end

  $body = "#{$body}<br>end.<br>"
  #$body = URI.escape($body)
  $evm.execute('send_email', to, from, subject, $body)
  log(:info, "Sent email to <#{to}> from <#{from}> subject: <#{subject}>")

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Set Ruby rescue behavior
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
