###################################
#
# CFME Automate Method: approve_request
#
# Author: Jason Dillaman
#
# Notes: Approves an incoming automation request
#
# Inputs: automation_request
#
# Mods:
#
###################################

AUTOMATION_URI_WHITELIST = [
    "Customer/Intuit/Automation/StateMachines/BulkServiceProvision/Default",
    "Customer/Intuit/Automation/StateMachines/Crust/ListQueue",
	"Customer/Intuit/Automation/StateMachines/Crust/ReconfigureVm",
	"Customer/Intuit/Automation/StateMachines/Crust/ApproveRequest",
    "Customer/Intuit/Automation/StateMachines/Crust/RetireService",
    "Customer/Intuit/Automation/StateMachines/HPSA/AddSoftwarePolicy",
    "Customer/Intuit/Automation/Methods/AddStorageNIC",
    "Customer/Intuit/integration/ExpandDisk/Actions/initiate_resize_second_disk",
    "Customer/Intuit/Provisioning/StateMachines/VMProvisionRollback/Default",
    "Customer/Intuit/Integration/StorageServices/StateMachine/AttachStorageRegional",
    "Customer/Intuit/Integration/StorageServices/StateMachine/DetachStorageRegional"]
ADMIN_AUTOMATION_URI_WHITELIST = [
    "Intuit/Automation/StateMachines/RetireVM/Default",
    "Intuit/Automation/StateMachines/ParkVM/Default"]

def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def validate_entry_point(user, request)
  ae_url = "#{request.options[:namespace]}/#{request.options[:class_name]}/#{request.options[:instance_name]}"
  unless (AUTOMATION_URI_WHITELIST + ADMIN_AUTOMATION_URI_WHITELIST).any? { |uri| ae_url == uri }
    msg = "Invalid automation entry point: [#{ae_url}]"
    request.deny('admin', msg)
    error(msg)
  end
  if ADMIN_AUTOMATION_URI_WHITELIST.find(ae_url) && user.userid != 'admin'
    msg = "Unauthorized user for automation entry point: [#{user.userid}][#{ae_url}]"
    request.deny('admin', msg)
    error(msg)
  end
end

begin
  dump_root()

  user = $evm.root['user']
  request = $evm.root['miq_request']

  validate_entry_point(user, request)

  $evm.log(:info, "AUTO-APPROVING")
  request.approve("admin", "Auto-Approved")

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
