###################################
#
# CFME Automate Method: ExecuteOnError
# Notes: This method is executed if a state ends with ae_result = error in the StateMachine
#
###################################
begin
  # Method for logging
  def log(level, message)
    @method = 'ExecuteOnError'
    $evm.log(level, "#{@method} - #{message}")
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  # emailrequester
  def emailrequester(miq_request, service, dialog_options)
    log(:info, "Requester email logic starting")

    # Get requester object
    requester = miq_request.requester

    group = requester.miq_group

    # Get requester email else set to nil
    requester_email = requester.email || $evm.object['to_email_address']

    # Get Owner Email else set to nil
    owner_email = miq_request.options[:owner_email] rescue nil
    log(:info, "Requester email:<#{requester_email}> Owner Email:<#{owner_email}>")

    # if to is nil then use requester_email or owner_email
    to = nil
    to ||= requester_email || owner_email || $evm.object['to_email_address']

    # Get from_email_address from model unless specified below
    from = nil
    from ||= $evm.object['from_email_address']

    # Get signature from model unless specified below
    signature = nil
    signature ||= $evm.object['signature']

    # Set email subject
    subject = "Service request ID #{miq_request.id} - Your Service Request #{service.name} has failed"

    # Build email body
    body = "Hello, "
    body += "<br>Group: #{group.description} quota modification has has failed due to the following reason:"
    body += "<br>#{miq_request.message}"
    body += "<br><br>Quota parameters used: <br>"
    body += dialog_options.collect { |k, v| "#{k.to_s}=#{v}" if k.starts_with?('dialog_quota') && ! v.blank? }.join('<br>')
    body += "<br> #{signature}"

    # Send email
    log(:info, "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
    $evm.execute(:send_email, to, from, subject, body)
  end

  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root()

  # get the task from root
  service_template_provision_task = $evm.root['service_template_provision_task']
  
  # get the request object from root
  miq_request = service_template_provision_task.miq_request
  log(:info, "miq_request id:<#{miq_request.id}> approval_state:<#{miq_request.approval_state} options[:dialog]:<#{miq_request.options[:dialog].inspect}>")

  # Get destination service object
  service = service_template_provision_task.destination
  log(:info, "Detected Service:<#{service.name}> Id:<#{service.id}> Tasks:<#{service_template_provision_task.miq_request_tasks.count}>")

  # Get dialog options from task
  dialog_options = service_template_provision_task.dialog_options
  log(:info, "Inspecting Dialog Options:<#{dialog_options.inspect}>")

  # Email Requester
  emailrequester(miq_request, service, dialog_options)

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Set Ruby rescue behavior
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
