###################################
#
# CFME Automate Method: get_tenants
# Author: Nick Lane
#
# Notes: Get a list of Tenants from Cisco ACI
#
# Inputs: 
#
# Mods:
#
###################################

require 'restclient'
require 'json'

def log(level, msg)
  @method = 'get_tenants'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "ERROR: #{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg)
  log(:error, "RETRY: #{msg}")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def get_token(username, password, server)
	log(:info, "Getting Login Token")
	uri = "http://#{server}/api/aaaLogin.json?gui-token-request=yes"
	body = {
		"aaaUser"=> {
			"attributes" => {
				"name" => username,
				"pwd"  => password
			}
		}
	}.to_json

	log(:info, "Attempting to acquire Token from #{server} with body: #{body}")
	login_response = RestClient.post(uri, body)
	log(:info, "Got response: #{login_response.headers}")
	response_hash = JSON.parse(login_response)

	imdata = response_hash['imdata'].first

	log(:info, "DATA: #{imdata}")

	url_token = imdata['aaaLogin']['attributes']['urlToken'].chomp
	@apic_cookie = imdata['aaaLogin']['attributes']['token'].chomp

	log(:info, "TOKEN: #{url_token}")
	return url_token
end

def get_epgs(server, token)
	log(:info, "Getting Tenants")
	#uri = "https://#{server}/api/node/mo/info.json"

	tenant = 'ihb'

	uri = "https://#{server}/api/mo/uni/tn-#{tenant}.xml?target-subtree-class=fvAEPg&query-target=subtree&rsp-subtree-class=tagInst&rsp-subtree=full"
	log(:info, "Attempting to get tenants with this request: #{uri}")
	begin
		epg_response = RestClient::Request.execute(
		:method => :get,
		:url => uri,
		:headers => {
			:accept => 'text/html,application/xhtml+xml,application/xml,application/json',
			'APIC-challenge' => token,
			'devcookie' => @apic_cookie
			}
		)
	rescue => err
		log(:info, "ERROR: #{err.inspect}")
	end

	log(:info, "Got Response: #{epg_response}")
	epg_data = epg_response['imdata'].shift
	log(:info, "Tenant Data: #{tenant_data}")
	epgs = Array.new
	epg_data.each {|epg|
		epgs.push epg['DnDomainMapEntry']['attributes']['dn']
	}
	log(:info, "EPGS: #{epgs}")

	return epgs
end

def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri.to_s)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

begin
	log(:info, "Entering Get EPGs")

	######
	#I'm setting this to IHPLAB for now until we can determine how best to figure out which environment we're in
	######

	intuit_dc = "ihplab"
	config = {}
  	retrieve_configuration(config, 'intuit_dc', intuit_dc)

  	username = config['username']
  	password = config['password']
  	server   = config['aci_server']

  	token = get_token(username, password, server)

  	epgs = get_epgs(server, token)

  	log(:info, "Got Token: #{token}")
rescue => err
	 error("[#{err}]\n#{err.backtrace.join("\n")}")
end
