###################################
#
# CFME Automate Method: Chef_Bootstrap
#
# Notes: This method uses a kinfe wrapper to bootstrap a Chef client
#
###################################
begin
  # Method for logging
  def log(level, message)
    @method = 'Chef_Bootstrap'
    $evm.log(level, "#{@method}: #{message}")
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  # basic retry logic
  def retry_method(retry_time=1.minute)
    log(:info, "Sleeping for #{retry_time} seconds")
    $evm.root['ae_result'] = 'retry'
    $evm.root['ae_retry_interval'] = retry_time
    exit MIQ_OK
  end

  def run_linux_admin(cmd)
    require 'linux_admin'
    log(:info, "Executing #{cmd}")
    result = LinuxAdmin.run!(cmd)
    log(:info, "Inspecting output: #{result.output.inspect}")
    log(:info, "Inspecting error: #{result.error.inspect}")
    log(:info, "Inspecting exit_status: #{result.exit_status.inspect}")
    return result
  end

  def copyFileFromHostToGuest(vm, vmrun_path, ems, guest_username, guest_password, srcfile, destfile)
    cmd = "#{vmrun_path}"
    cmd += " -h https://#{ems.ipaddress}:443/sdk"
    cmd += " -u \"#{ems.authentication_userid}\""
    cmd += " -p \"#{ems.authentication_password}\""
    cmd += " -gu \"#{guest_username}\""
    cmd += " -gp \"#{guest_password}\""
    cmd += " copyFileFromHostToGuest"
    cmd += " \"[#{vm.storage_name}]#{vm.location}\""
    cmd += " #{srcfile} #{destfile}"
    log(:info, "Calling external cmd:<#{cmd}>")
    results = system(cmd)
    return results
  end

  ####################################
  #
  # Method: runProgramInGuest
  # Notes: call vmrun with parameters
  # Returns: true|false
  #
  ####################################
  def runProgramInGuest(vm, vmrun_path, ems, guest_username, guest_password, guest_program, guest_options)
    cmd  = "#{vmrun_path}"
    cmd += " -h https://#{ems.ipaddress}:443/sdk"
    cmd += " -u \"#{ems.authentication_userid}\""
    cmd += " -p \"#{ems.authentication_password}\""
    cmd += " -gu \"#{guest_username}\""
    cmd += " -gp \"#{guest_password}\""
    cmd += " runProgramInGuest"
    cmd += " \"[#{vm.storage_name}]#{vm.location}\""
    cmd += " \"#{guest_program}\""
    cmd += " #{guest_options}" unless guest_options.blank?
    log(:info, "Calling external cmd:<#{cmd}>")
    results = system(cmd)
    #$evm.log("info","VMrun command <runProgramInGuest> completed with results:<#{results}>")
    return results
  end

  def fileExistsInGuest(vm, vmrun_path, ems, guest_username, guest_password, guest_program)
    cmd  = "#{vmrun_path}"
    cmd += " -h https://#{ems.ipaddress}:443/sdk"
    cmd += " -u \"#{ems.authentication_userid}\""
    cmd += " -p \"#{ems.authentication_password}\""
    cmd += " -gu \"#{guest_username}\""
    cmd += " -gp \"#{guest_password}\""
    cmd += " fileExistsInGuest"
    cmd += " \"[#{vm.storage_name}]#{vm.location}\""
    cmd += " \"#{guest_program}\""

    log(:info, "Calling external cmd:<#{cmd}>")
    results = system(cmd)
    #$evm.log("info","VMrun command <fileExistsInGuest> completed with results:<#{results}>")
    return results
  end

  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root

  username = nil || $evm.object['username']
  password = nil || $evm.object.decrypt('password')

  case $evm.root['vmdb_object_type']

  when 'miq_provision'
    prov = $evm.root['miq_provision']
    log(:info, "Provision:<#{prov.id}> Request:<#{prov.miq_provision_request.id}> Type:<#{prov.type}>")

    # get vm object from miq_provision
    vm = prov.vm
    raise "$evm.root['miq_provision'].vm not found" if vm.nil?

    # Since this is provisioning we need to put in retry logic to wait until IP Addresses are populated.
    unless vm.ipaddresses.blank?
      log(:info, "VM:<#{vm.name}> IP addresses:<#{vm.ipaddresses.inspect}> present.")
    else
      log(:warn, "VM:<#{vm.name}> IP addresses:<#{vm.ipaddresses.inspect}> not present.")
      retry_method()
    end

    tags = prov.get_tags
    log(:info, "Bootstrapping Chef client on VM:<#{vm.name}>")

    env = tags[:environment]
    if env.nil?
      log(:info, "Environment tag:<#{env}> missing. skipping method")
    else
      log(:info, "Environment tag:<#{env}> detected")
      cmd = "/var/www/miq/knife_wrapper_#{tr1} bootstrap #{vm.name}.mhint -x #{username} -P #{password}"
      result = run_linux_admin(cmd)
      unless result.exit_status.zero?
        log(:info, "Command: #{cmd} failed with #{result.error.inspect} exit_status: #{result.exit_status.inspect}" )
        retry_method()
      end
    end
  when 'vm'
    vmrun_path = nil
    vmrun_path ||= $evm.object['vmrun_path']
    vm = $evm.root['vm']
    ems = vm.ext_management_system

    log(:info, "Bootstrapping Chef client on VM:<#{vm.name}> using vmrun")
    #cmd = "/var/www/miq/knife_wrapper_tr1 bootstrap #{vm.name}.mhint -x #{username} -P #{password}"
    #result = run_linux_admin(cmd)
    bootstrap_script = "/var/www/miq/install_chef.sh"
    bootstrap_options = ""
    chef_roles = $evm.root['dialog_chef_roles'] rescue nil
    chef_cookbook = $evm.root['dialog_chef_cookbook'] rescue nil
    chef_options = []
    chef_options.push("role[#{chef_roles}]") unless chef_roles.blank?
    chef_options.push("recipe[#{chef_cookbook}]") unless chef_cookbook.blank?
    log(:info, "Got chef roles #{chef_roles} and cookbook #{chef_cookbook}")

    bootstrap_options =  "-r " + chef_options.join(",") if chef_options.length > 0


    log(:info, "Generated bootstrap command is #{bootstrap_script} #{bootstrap_options}")
    log(:info, "Starting Bootstrap via VIX")
    # Check if Chef is already bootstrapped
    unless vm.custom_get("CHEF_Bootstrapped").blank?
      log(:info, "Chef already bootstrapped")
      fileExists = fileExistsInGuest(vm, vmrun_path, ems, username, password, "/usr/bin/chef-client")
      log(:info, "File Exists #{fileExists}")
      if fileExists 
        results = runProgramInGuest(vm, vmrun_path, ems, username, password, "/usr/bin/chef-client", bootstrap_options)
        log(:info, "Chef results: #{results}")
        vm.custom_set("CHEF_Cookbook", chef_roles) unless chef_roles.blank?
        vm.custom_set("CHEF_Recipe", chef_cookbook) unless chef_cookbook.blank?
        exit MIQ_OK
      end
    end
    #fileExists = fileExistsInGuest(vm, vmrun_path, ems, username, password, bootstrap_script)
    fileExists = false
    unless fileExists
      results = runProgramInGuest(vm, vmrun_path, ems, username, password, "/bin/mkdir", "-p /var/www/miq")
      log(:error, "Failed to run /bin/mkdir -p /var/www/miq on #{vm.name}: <#{results}>, continuing anyway") unless results
      results = copyFileFromHostToGuest(vm, vmrun_path, ems, username, password, bootstrap_script, bootstrap_script)
      raise "Failed to copy #{bootstrap_script} to #{vm.name}: #{results}" unless results
    end
    results = runProgramInGuest(vm, vmrun_path, ems, username, password, bootstrap_script, "")
    raise "Chef bootstrap failed <#{results}> (#{bootstrap_script} #{bootstrap_options}" unless results
    results = runProgramInGuest(vm, vmrun_path, ems, username, password, "/usr/bin/chef-client", bootstrap_options) unless bootstrap_options.blank?
    log(:info, "Applied chef stuff #{bootstrap_options} with result #{results}")
    vm.custom_set("CHEF_Bootstrapped", "#{Time.now}")
    vm.custom_set("CHEF_Cookbook", chef_roles) unless chef_roles.blank?
    vm.custom_set("CHEF_Recipe", chef_cookbook) unless chef_cookbook.blank?
    log(:info, "Completed Bootstrap via VIX")
  else
    log(:info, "Invalid $evm.root['vmdb_object_type']:<#{$evm.root['vmdb_object_type']}>")
  end

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Ruby rescue
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
