###################################
#
# CFME Automate Method: create_automation_request
#
# Author: Jason Dillaman
#
# Notes: Initiate an automation SOAP request to CFME
#
# Inputs: intuit_dc
#         ae_namespace
#         ae_class
#         ae_instance
#         ip_address
#         parameters
#
# Changes:
#   2015-12-08 Daniel Trieu update for Savon2
#
###################################

require 'savon'

# logging convenience method
def log(level, msg)
  @method = 'create_automation_request'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

# dump_current
def dump_current()
  log(:info, "Current:<$evm.current> Begin $evm.current.attributes")
  $evm.current.attributes.sort.each { |k, v| log(:info, "Current:<$evm.current> Attribute - #{k}: #{v}")}
  log(:info, "Current:<$evm.current> End $evm.current.attributes")
  log(:info, "")
end

begin
  dump_current()

  intuit_dc  = $evm.current['intuit_dc']
  ip_address = $evm.current['ip_address']
  username   = $evm.current['cfme_username']
  password   = $evm.current.decrypt('cfme_password')
  error("Missing CFME credentials for intuit data center: [#{intuit_dc}]") if username.blank? || password.blank?

  #client = Savon.client("https://#{ip_address}/vmdbws/wsdl")
  #client.http.auth.basic(username, password)
  #client.http.auth.ssl.verify_mode = :none

  # Configure Savon
  client = Savon.client(
      wsdl: "https://#{ip_address}/vmdbws/wsdl",
      basic_auth: [username, password],
      ssl_verify_mode: :none
  )

  begin
    #response = client.request("CreateAutomationRequest") do
    #  soap.body = {
    #    "version"    => "1.0",
    #    "uri_parts"  => "namespace=#{$evm.current['ae_namespace']}|class=#{$evm.current['ae_class']}|instance=#{$evm.current['ae_instance']}",
    #    "parameters" => $evm.current['parameters'],
    #    "requester"  => ""
    #  }
    #end

    response = client.call(
        :create_automation_request,
        message: {
            "version"    => "1.0",
            "uri_parts"  => "namespace=#{$evm.current['ae_namespace']}|class=#{$evm.current['ae_class']}|instance=#{$evm.current['ae_instance']}",
            "parameters" => $evm.current['parameters'],
            "requester"  => ""
        }
    )
    
    $evm.current['request_id'] = (response.to_hash[:create_automation_request_response] || {})[:return].to_i
    $evm.current['result'] = 'ok'
    $evm.current['reason'] = ''
  rescue HTTPClient::TimeoutError, Savon::HTTPError => err
    log(:warn, "Failed, will retry: [#{err}]")
    $evm.current['result'] = 'retry'
    $evm.current['reason'] = err.to_s
  rescue Savon::SOAPFault => err
    error(err.message)
  end
  
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
