###################################
#
# CFME Automate Method: create_service_provision_request
#
# Author: Jason Dillaman
#
# Notes: Creates a new Service Provision Request 
#
# Inputs: service_template_name
#
# Mods:
#
###################################

def dump_current()
  $evm.log(:info, "Current:<$evm.current> Begin $evm.current.attributes")
  $evm.current.attributes.sort.each { |k, v| $evm.log(:info, "Current:<$evm.current> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Current:<$evm.current> End $evm.current.attributes")
  $evm.log(:info, "")
end

def error(msg)
  $evm.log(:error, msg)
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg
  exit MIQ_OK
end

def execute_rails_runner(rails_ruby_code)
  $evm.object['rails_ruby_code'] = rails_ruby_code
  result = instantiate("/#{$evm.current_namespace}/#{$evm.current_class}/RailsRunner")
  error("Failed to create Service Provision Request: [#{result['rails_errors']}]") unless result['rails_errors'].blank?
  return result['rails_output']
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute: [#{uri}]") unless result
  error(result['reason']) unless result['result'] == 'ok'
  return result
end

def generate_rails_ruby_code(userid, service_template, dialog_values)
  code = <<EOF
require 'json'

dialog_values = {}
JSON.parse('#{JSON.dump(dialog_values)}').each do |k,v|
  dialog_values["dialog_\#{k}"] = v
end

values = {
  :dialog => dialog_values,
  :src_id => #{service_template.id}
}
request = ServiceTemplateProvisionRequest.create_request(values, '#{userid}', false)
result = request.id
puts result

EOF
end

def get_dialog_values()
  dialog_values = {}
  $evm.current.attributes.each do |k,v|
    next unless match = k.match(/dialog_(.+)/)
    dialog_values[match[1]] = v
  end
  dialog_values
end

def get_service_template(service_template_name)
  service_template = $evm.vmdb('service_template').find_by_name(service_template_name)
  error("Unable to locate service template: [#{$evm.current['service_template_name']}]") unless service_template
  service_template
end

begin
  dump_current()
  
  if $evm.root['user'].nil?
    $evm.log(:info, "$evm.root User is nil. Using $evm.current instead.")    
    userid = $evm.root['userid']
  else
    $evm.log(:info, "Using $evm.root for userid.")    
    userid = $evm.root['user'].userid
  end
  $evm.log(:info, "Using Userid: #{userid}")
  service_template = get_service_template($evm.current['service_template_name'])
  dialog_values = get_dialog_values()
  
  rails_ruby_code = generate_rails_ruby_code(userid, service_template, dialog_values)
  result = execute_rails_runner(rails_ruby_code)
  $evm.log(:info, "Result of execute_rails_runner: #{result}")
  $evm.current['result'] = 'ok'
  $evm.root['miq_provision_id'] = result
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
