###################################
#
# CFME Automate Method: create_vm_provision_request
#
# Author: Jason Dillaman, Daniel Trieu
#
# Notes: Initiate VM provision SOAP request to CFME
#
# Inputs: template_fields: name=<template name>|request_type=template
#         vm_fields: number_of_vms=1|vm_name=example|provision_type=vmware
#         requester: user_name=admin|owner_first_name=cloud|owner_last_name=user|owner_email=cloud.admin@example.com
#         tags: tag1=A|tag2=B
#         values: value1=A|value2=B
#         ems_custom_attributes: value1=A|value2=B
#         miq_custom_attributes: value1=A|value2=B
#
# Changes:
#   2015-12-02 Daniel Trie updated Savon calls to proper Savon2.0
#
###################################

require 'savon'

def error(msg)
  $evm.log(:error, "CloudForms VM Provision Request Failed: [#{msg}]")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

# dump_current
def dump_current()
  $evm.log(:info, "Current:<$evm.current> Begin $evm.current.attributes")
  $evm.current.attributes.sort.each { |k, v| $evm.log(:info, "Current:<$evm.current> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Current:<$evm.current> End $evm.current.attributes")
  $evm.log(:info, "")
end

begin
  dump_current()

  intuit_dc  = $evm.current['intuit_dc']
  ip_address = $evm.current['ip_address']
  username   = $evm.current['cfme_username']
  password   = $evm.current.decrypt('cfme_password')
  error("Missing CFME credentials for intuit data center: [#{intuit_dc}]") if username.blank? || password.blank?

  client = Savon.client(
      wsdl: "https://#{ip_address}/vmdbws/wsdl",
      basic_auth: [username, password],
      ssl_verify_mode: :none,
      read_timeout: 300
  )

  begin

    response = client.call(
        :vm_provision_request,
        message: {
            "version"        => "2.0",
            "templateFields" => $evm.current['template_fields'],
            "vmFields"       => $evm.current['vm_fields'],
            "requester"      => $evm.current['requester'],
            "tags"           => $evm.current['tags'],
            "options"        => {
                "values"                => $evm.current['values'],
                "ems_custom_attributes" => $evm.current['ems_custom_attributes'],
                "miq_custom_attributes" => $evm.current['miq_custom_attributes']
            }
        }
    )

    $evm.current['request_id'] = (response.to_hash[:vm_provision_request_response] || {})[:return][:id].to_i
    $evm.current['result'] = 'ok'
    $evm.current['reason'] = ''
  rescue HTTPClient::TimeoutError, Savon::HTTPError => err
    $evm.log(:warn, "CloudForms VM Provision Request Failed, will retry: [#{err}]\n#{err.backtrace.join("\n")}")
    $evm.current['result'] = 'retry'
    $evm.current['reason'] = err.to_s
  rescue Savon::SOAPFault => err
    $evm.log(:info, "SOAPFault Error: [#{err}]\n#{err.backtrace.join("\n")}")
    error(err.message)
  end

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
