###################################
#
# CFME Automate Method: get_current_intuit_dc
#
# Author: Jason Dillaman
#
# Notes: Retrieve the intuit_dc tag for the current CFME appliance
#
###################################

require 'socket'

def error(msg)
  $evm.log(:error, msg)
  $evm.root['result'] = 'error'
  $evm.root['reason'] = "#{msg}"
  exit MIQ_OK
end

def get_hostname()
  hostname = Socket.gethostname
  error("Unable to determine current CloudForms appliance hostname") if hostname.blank?
  hostname
end

def get_miq_server(hostname)
  miq_server = $evm.vmdb('miq_server').find_by_hostname(hostname)
  error("Unable to locate CloudForm appliance appliance: [#{hostname}]") unless miq_server
  miq_server
end

def get_miq_server_vm(miq_server)
  vm_id = miq_server.vm_id
  error("CloudForms server to VM relationship not established: [#{miq_server.name}]") if vm_id.blank?
  vm = $evm.vmdb('vm').find_by_id(vm_id)
  error("Unable to locate VM associated to CloudForms server: [#{miq_server.name}]") unless vm
  vm
end

def get_intuit_dc(vm)
  intuit_dc = vm.tags(:intuit_dc).first
  intuit_dc = vm.ext_management_system.tags(:intuit_dc).first if intuit_dc.blank?
  error("Unable to locate intuit_dc tag for vCenter: [#{miq_server.ext_management_system.name}]") if intuit_dc.blank?
  $evm.log(:info, "Determined current CloudForms intuit_dc: [#{intuit_dc}]")
  intuit_dc
end

begin
  hostname_dc_map = {
      'itmcfgui01.corplab.intuit.net' => 'ihplab',
      'itmcfgui02.corplab.intuit.net' => 'ihplab',
      'itmcfrqui01.corplab.intuit.net' => 'ihplab',
      'itmcfrqui02.corplab.intuit.net' => 'ihplab',
      'itmcfrlui01.corplab.intuit.net' => 'ihplab',
      'mprdcfmews301.corp.intuit.net' => 'qdc',
      'mprdcfmews401.corp.intuit.net' => 'lvdc',
      'mprdcfmews321.corp.intuit.net' => 'qdc',
      'mprdcfmews322.corp.intuit.net' => 'qdc',
      'mprdcfmews421.corp.intuit.net' => 'lvdc',
      'mprdcfmews422.corp.intuit.net' => 'lvdc',
      'mprdcfmewsa21.corp.intuit.net' => 'edc',
      'mprdcfmewsa22.corp.intuit.net' => 'edc',
      'mprdcfmewsb21.corp.intuit.net' => 'sdc',
      'mprdcfmewsb22.corp.intuit.net' => 'sdc'
  }

  hostname = get_hostname()

  if hostname_dc_map[hostname]
    intuit_dc = hostname_dc_map[hostname]
    $evm.log(:info, "Identified DC #{intuit_dc} for host #{hostname}")
  else
    miq_server = get_miq_server(hostname)
    vm = get_miq_server_vm(miq_server)
    intuit_dc = get_intuit_dc(vm)
  end

  $evm.current['result'] = 'ok'
  $evm.current['intuit_dc'] = intuit_dc

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
