###################################
#
# CFME Automate Method: get_current_region_number
#
# Author: Jason Dillaman
#
# Notes: Retrieve the intuit_dc tag for the current CFME appliance
#
###################################

require 'socket'

DEFAULT_REGION_FACTOR = 1_000_000_000_000

def error(msg)
  $evm.log(:error, msg)
  $evm.root['result'] = 'error'
  $evm.root['reason'] = "#{msg}"
  exit MIQ_OK
end

def region_number(object)
  object.id.to_i / DEFAULT_REGION_FACTOR
end

def get_hostname()
  hostname = Socket.gethostname
  error("Unable to determine current CloudForms appliance hostname") if hostname.blank?
  hostname
end

def get_miq_server(hostname)
  miq_server = $evm.vmdb('miq_server').find_by_hostname(hostname)
  error("Unable to locate CloudForm appliance appliance: [#{hostname}]") unless miq_server
  miq_server
end

begin
  hostname = get_hostname()
  miq_server = get_miq_server(hostname)
  region_number = region_number(miq_server)
  
  $evm.current['result'] = 'ok'
  $evm.current['region_number'] = region_number

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
