###################################
#
# CFME Automate Method: get_region_endpoints
#
# Author: Jason Dillaman
#
# Notes: Locates the CFME Web Service Endpoints for local regions
#
# Inputs: vmdb_object_type
#
# Mods:
#
###################################

DEFAULT_REGION_FACTOR = 1_000_000_000_000

def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

# dump_root
def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def get_vmdb_object_region_and_intuit_dc()
  service = [$evm.current, $evm.root].detect { |x| !x['vmdb_object_type'].blank? }
  error("Missing required vmdb_object_type parameter") unless service
  
  vmdb_object = service[service['vmdb_object_type']]
  error("Missing required vmdb_object: [#{service['vmdb_object_type']}]") unless vmdb_object
  
  ems = (vmdb_object.respond_to?(:ext_management_system) ? 
    vmdb_object.ext_management_system : vmdb_object)  
  intuit_dc = ems.tags(:intuit_dc).first
  error("Unable to locate Intuit Data Center tag on vCenter: [#{ems.name}]") if intuit_dc.blank?
    
  return region_number(vmdb_object), intuit_dc
end

def region_number(object)
  object.id.to_i / DEFAULT_REGION_FACTOR
end

def get_region_web_service_servers(region)
  servers = $evm.vmdb('miq_server').all.select do |server| 
    region_number(server) == region && server.has_active_webservices
  end.shuffle
  error("Unable to find CloudForms web service for region: [#{region}]") if servers.blank?
  servers
end

begin
  # dump all root attributes to the log
  dump_root

  region, intuit_dc = get_vmdb_object_region_and_intuit_dc()
  servers = get_region_web_service_servers(region)

  $evm.current['intuit_dc'] = intuit_dc
  $evm.current['region_endpoint_ip_addresses'] = servers.collect { |server| server.ipaddress }
  $evm.current['result'] = 'ok'
  $evm.log(:info, "CloudForms Web Service Endpoints: [#{$evm.current['intuit_dc']}]#{$evm.current['region_endpoint_ip_addresses'] }")

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
