###################################
#
# CFME Automate Method: rails_runner
#
# Author: Jason Dillaman
#
# Notes: Wraps the rails runner to execute arbitrary code within the
#        context of the CloudForms Ruby-on-Rails application.  This
#        allows full access to the CFME database and associated model.
#
# Inputs: rails_ruby_code via $evm.parent
#
# Mods:
#
###################################

require 'io/wait'
require 'open3'
require 'tempfile'
require 'timeout'

DEFAULT_TIMEOUT = 120
RAILS_PATH      = '/var/www/miq/vmdb/script/rails'

def error(msg)
  $evm.log(:error, msg)
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def process_pipe(pipe, log_type)
  data = ""
  while pipe && pipe.ready? do
    data << pipe.read
  end
  data.split('\n').each do | line |
    $evm.log(log_type, "Rails output: [#{line}]")
  end
  return data
end

def rails_runner(ruby_code)
  output = ""
  errors = ""

  file = Tempfile.new('ae_rails')
  file.write(ruby_code)
  file.flush
  begin
    begin
      # Wrap in bash to free rails from the bundle environment
      stdin, stdout, stderr, wait_thr = 
        Open3.popen3('bash', '-cl', [RAILS_PATH, 'r', file.path].join(' '))
      stdin.close
      Timeout.timeout(DEFAULT_TIMEOUT) do
        begin
          output << process_pipe(stdout, 'info')
          errors << process_pipe(stderr, 'error')
        end until wait_thr.join(1)
        exit MIQ_ABORT if wait_thr.value != 0
      end
    rescue Timeout::Error
      Process.kill('KILL', wait_thr.pid)
      error('Timed out waiting for Rails')
    ensure
      output << process_pipe(stdout, 'info')
      errors << process_pipe(stderr, 'error')
      stdout.close if stdout
      stderr.close if stderr
    end
  ensure
    file.close!
  end
  return output, errors
end

begin
  rails_ruby_code = $evm.object['rails_ruby_code'] || ($evm.parent && $evm.parent['rails_ruby_code'])
  unless rails_ruby_code.blank?
    $evm.log('info', "Invoking custom Ruby Rails code: [#{rails_ruby_code}]")
    output, errors = rails_runner(rails_ruby_code)
    $evm.log('info', "Ruby Rails result: [stdout: #{output}] [stderr: #{errors}]")
    $evm.current['rails_output'] = output unless output.blank?
    error(errors) unless errors.blank?
    $evm.current['result'] = 'ok'
  end
  exit MIQ_OK
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
