##################################
#
# CFME Automate Method: delete_from_escalation_qb
#
# Author: Carsten Clasohm
#
# Notes: Deletes an escalation record from the Escalation Quickbase
#
# Inputs: intuit_dc, rid
#
###################################

require 'httpclient'
require 'nokogiri'
require 'uri'

APP_TOKEN = 'c5upj75cfp4iqccgabs9ub65rrkz'

# Methods for logging
def log(level, message)
  @method = 'delete_from_escalation_qb'
  $evm.log(level, "#{@method}: #{message}")
end

def error(msg)
  log(:error, "#{msg}")
  log(:error, "Error received deleting VM record from Escalation QB, exiting. #{msg}.")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def retry_method(msg)
  $evm.log(:warn,"Error received deleting VM record from Escalation QB, retrying. [#{msg}]")
  $evm.current['result'] = 'retry'
   $evm.current['reason'] = msg
  exit MIQ_OK
end

def escape_uri(value)
  URI.escape("#{value}", Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri.to_s)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

def authenticate(http, username, password, base_url)
  args = "act=API_Authenticate&"
  args+= "username=#{escape_uri(username)}&"
  args+= "password=#{escape_uri(password)}"
  resp = http.get(base_url+args, :follow_redirect=>true).content
  results = Nokogiri::XML.parse(resp)
  token = results.xpath("/qdbapi/ticket")[0].content
  token
end

def delete_from_escalation_qb_table(http, token, escalation_table_url, record_id)
  args = "act=API_DeleteRecord&"
  args+= "ticket=#{token}&"
  args+= "apptoken=#{APP_TOKEN}&"
  args+= "rid=#{record_id}"
  args = URI.escape(args)
  resp = http.get(escalation_table_url+args, :follow_redirect=>true).content
  resp
end

begin
  intuit_dc = $evm.current['intuit_dc']
  record_id = $evm.current['record_id']

  config = {}
  retrieve_configuration(config, 'intuit_dc', intuit_dc)

  http = HTTPClient.new(config['proxy_url'])

  token = authenticate(http,
                       config['eqb_username'], 
                       config['eqb_password'],
                       config['eqb_base_url'])

  log(:info, "Deleting record #{record_id}")

  resp = delete_from_escalation_qb_table(http,
                                         token,
                                         config['escalation_table_url'],
                                         record_id)
  xml_resp = Nokogiri.parse(resp)
  log(:info, "xml_resp = #{xml_resp.inspect}")

  errcode = xml_resp.xpath("qdbapi/errcode")[0].content
  errtext = xml_resp.xpath("qdbapi/errtext")[0].content
  unless errcode == "0"
    retry_method("Error #{errcode} deleting record #{record_id}: #{errtext}")
  end

  $evm.current['result'] = 'ok'
  $evm.current['reason'] = ''

rescue StandardError, SignalException => err
  retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
end
