###################################
#
# CFME Automate Method: wait_for_powered_on
#
# Author: Nick Lane
#
# Notes: Downloads the HPSA agent installer, transfers it to the new
#        VM, and executes the installer
#
# Inputs: vm: the target VM
#
# Mods:
#
###################################




def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def error(msg)
  $evm.log(:error, "HPSA agent installation failed: [#{msg}]")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end


def get_virtual_machine()
  case $evm.root['vmdb_object_type']
  when 'miq_provision'
    vm = $evm.root['miq_provision'].vm
  when 'vm'
    vm = $evm.root['vm']
  when 'automation_task'
    vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

begin

  dump_root()
  vm = get_virtual_machine()

#  $evm.log(:info, "vm inspect #{vm.inspect}")
  state = vm.power_state


  $evm.log(:info, "vm state #{state}")

  unless state.include? ("on")
    vm.refresh
    $evm.log(:info, "Waiting for VM to reboot")
    #$evm.root['ae_result'] = 'retry'
    #$evm.root['ae_retry_interval'] = '5.seconds'
    sleep(5)
    raise("Retry")
  else
    $evm.log(:info, "VM is on, resizing the second disk in the OS.")
  end
  exit MIQ_OK
rescue => err
  $evm.log(:info, "#{err.inspect}")
  if err.message == "Retry"
    $evm.log(:info, "Retrying Wait for Powered Off")
    retry
  else
    error("[#{err}]\n#{err.backtrace.join("\n")}")
  end
end
