###################################
#
# CFME Automate Method: park_ip_address
#
# Author: Dave Costakos, Carsten Clasohm
#
# Notes: Release IP address in Men & Mice
#
# Inputs: vm
#
# Mods:
#
###################################

require 'savon'
require 'rubygems'
require 'httpi'

# logging convenience method
def log (level, msg) 
  @method = 'park_ip_address'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)


    log(:error, "Received error from Men and Mice, exiting #{msg}.")
    $evm.current['result'] = 'error'
    $evm.current['reason'] = msg
    exit MIQ_OK
end

def retry_method(msg, retry_time=1.minute)
  # There are some errors from Men & Mice that are only intermittent. We
  # wait a little bit and retry.
  $evm.log(:warn,"Received error from Men and Mice, retrying #{msg}.")
  $evm.current['result'] = 'retry'
  $evm.current['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

def get_session_id(central, username, password)
  log(:info, "get_session_id: entering method")
  #response = @client.request :login do |soap|
  #  soap.body = {
  #    'server' => central,
  #    'loginName' => username,
  #    'password' => password
  #  }
  #end

  response = @client.call(
    :login,
    message: {
        'server' => central,
        'loginName' => username,
        'password' => password
    }
  )
  log(:info, "get_session_id: #{response.to_hash[:login_response]}")
  log(:info, "get_session_id: exiting method")
  return response.to_hash[:login_response][:session]
end

def get_dns_zone_ref(zonename)
  log(:info, "get_dns_zone_ref(#{zonename}): entering method")
  #response = @client.request :get_dns_zones do |soap|
  #  soap.body = {
  #    'session' => @session,
  #    'filter' => "type:^Master$ name:^#{zonename}$"
  #  }
  #end

  response = @client.call(
    :get_dns_zones,
    message: {
        'session' => @session,
        'filter' => "type:^Master$ name:^#{zonename}$"
    }
  )

  log(:info, "get_dns_zone_ref(#{zonename}): Response Details #{response.to_hash[:get_dns_zones_response].inspect}")
  zone = response.to_hash[:get_dns_zones_response][:dns_zones][:dns_zone]
  log(:info, "get_dns_zone_ref(#{zonename}): Got zone #{zone}")
  log(:info, "get_dns_zone_ref(#{zonename}): exiting method")
  return zone[:ref]
end

def get_dns_record(zone_ref, ipaddr)
  log(:info, "get_dns_record(#{zone_ref},#{ipaddr}): entering method")
  filter = "#{filter} data:^#{ipaddr}$" unless ipaddr.nil?
  log(:info, "get_dns_record(#{zone_ref},#{ipaddr}): Using filter #{filter}")
  #response = @client.request :get_dns_records do |soap|
  #  soap.body = {
  #    'session' => @session,
  #    'dnsZoneRef' => zone_ref,
  #    'filter' => filter,
  #  }
  #end

  response = @client.call(
    :get_dns_records,
    message: {
        'session' => @session,
        'dnsZoneRef' => zone_ref,
        'filter' => filter,
    }
  )
  log(:info, "get_dns_record(#{zone_ref},#{ipaddr}): #{response.to_hash[:get_dns_records_response].inspect}")
  return nil if response.to_hash[:get_dns_records_response][:total_results] == 0
  return nil if response.to_hash[:get_dns_records_response][:dns_records].nil?
  log(:info, "DNS NAME: #{response.to_hash[:get_dns_records_response][:dns_records][:dns_record][:name]}")
  return "parked" if response.to_hash[:get_dns_records_response][:dns_records][:dns_record][:name].include? "ipparked"
  hash = response.to_hash[:get_dns_records_response][:dns_records][:dns_record]
  if "#{hash.class}" == "Hash"
    return hash[:ref]
  else
    error("Reponse is of type #{hash.class}, probably multiple DNS entries with this IP address")
  end
  return hash[:ref]
end

def get_ipam_record(ipaddr)
  log(:info, "get_ipam_record(#{ipaddr}): entering method")
  #response = @client.request :get_ipam_record do |soap|
  #  soap.body = {
  #    'session' => @session,
  #    'addrRef' => ipaddr,
  #  }
  #end

  response = @client.call(
    :get_ipam_record,
    message: {
        'session' => @session,
        'addrRef' => ipaddr,
    }
  )
  log(:info, "get_ipam_record(#{ipaddr}): #{response.inspect}")
  #return nil if response.to_hash[:get_dns_records_response][:total_results] == 0
  #return nil if response.to_hash[:get_dns_records_response][:dns_records].nil?
  #log(:info, "DNS NAME: #{response.to_hash[:get_dns_records_response][:dns_records][:dns_record][:name]}")
  #return "parked" if response.to_hash[:get_dns_records_response][:dns_records][:dns_record][:name].include? "ipparked"
  #hash = response.to_hash[:get_ipam_record_response]
  #log(:info, "IPAM HASH(#{ipaddr}): #{hash.inspect}")
  #if "#{hash.class}" == "Hash"
  #  return hash[:ref]
  #else
  #  error("Reponse is of type #{hash.class}, probably multiple DNS entries with this IP address")
  #end
  return response
end

def update_ipam_record(orig_record, new_record)
  log(:info, "update_ipam_record(#{orig_record}): entering method")
  log(:info, "Session #{@session} client #{@client}")
  begin
    #response = @client.request :set_ipam_record do |soap|
    #  soap.body = {
    #    'session' => @session,
    #    'ipamRecordBefore' => {
    #        'addrRef' => orig_record[:addr_ref],
    #        'address' => orig_record[:address],
    #        'claimed' => orig_record[:claimed],
    #        'dnsHosts'=> {
    #          'dnsHost' => {
    #            'dnsRecord' => {
    #              'ref'  => orig_record[:dns_hosts][:dns_host][:dns_record][:ref],
    #              'name' => orig_record[:dns_hosts][:dns_host][:dns_record][:name],
    #              'type' => orig_record[:dns_hosts][:dns_host][:dns_record][:type],
    #              'ttl'  => orig_record[:dns_hosts][:dns_host][:dns_record][:ttl],
    #              'data' => orig_record[:dns_hosts][:dns_host][:dns_record][:data],
    #              'enabled' => orig_record[:dns_hosts][:dns_host][:dns_record][:enabled],
    #              'dnsZoneRef' => orig_record[:dns_hosts][:dns_host][:dns_record][:dns_zone_ref]
    #              },
    #              'ptrStatus' => orig_record[:dns_hosts][:dns_host][:ptr_status],
    #              'relatedRecords' => orig_record[:dns_hosts][:dns_host][:related_records]
    #            },
    #          },
    #        'dhcpReservations' => orig_record[:dhcp_reservations],
    #        'dhcpLeases' => orig_record[:dhcp_leases],
    #        'discoveryType' => orig_record[:discovery_type],
    #        'lastSeenDate' => orig_record[:last_seen_date],
    #        'lastDiscoveryDate' => orig_record[:last_discovery_date],
    #        'lastKnownClientIdentifier' => orig_record[:last_know_client_identifier],
    #        'customProperties' => orig_record[:custom_properties]
    #      },
    #      'ipamRecordAfter' => {
    #        'addrRef' => new_record[:addr_ref],
    #        'address' => new_record[:address],
    #        'claimed' => new_record[:claimed],
    #        'dnsHosts'=> {
    #          'dnsHost' => {
    #            'dnsRecord' => {
    #              'ref'  => new_record[:dns_hosts][:dns_host][:dns_record][:ref],
    #              'name' => new_record[:dns_hosts][:dns_host][:dns_record][:name],
    #              'type' => new_record[:dns_hosts][:dns_host][:dns_record][:type],
    #              'ttl'  => new_record[:dns_hosts][:dns_host][:dns_record][:ttl],
    #              'data' => new_record[:dns_hosts][:dns_host][:dns_record][:data],
    #              'enabled' => new_record[:dns_hosts][:dns_host][:dns_record][:enabled],
    #              'dnsZoneRef' => new_record[:dns_hosts][:dns_host][:dns_record][:dns_zone_ref]
    #              },
    #              'ptrStatus' => new_record[:dns_hosts][:dns_host][:ptr_status],
    #              'relatedRecords' => new_record[:dns_hosts][:dns_host][:related_records]
    #            },
    #          },
    #        'dhcpReservations' => new_record[:dhcp_reservations],
    #        'dhcpLeases' => new_record[:dhcp_leases],
    #        'discoveryType' => new_record[:discovery_type],
    #        'lastSeenDate' => new_record[:last_seen_date],
    #        'lastDiscoveryDate' => new_record[:last_discovery_date],
    #        'lastKnownClientIdentifier' => new_record[:last_know_client_identifier],
    #        'customProperties' => new_record[:custom_properties]
    #      },
    #  }
    #  #'ipamRecordBefore' => orig_record,
    #  #'ipamRecordAfter' => new_record
    #  log(:info, "SOAP BODY: #{soap.body}")
    #end

    response = @client.call(
      :set_ipam_record,
      message: {
          'session' => @session,
          'ipamRecordBefore' => {
              'addrRef' => orig_record[:addr_ref],
              'address' => orig_record[:address],
              'claimed' => orig_record[:claimed],
              'dnsHosts'=> {
                  'dnsHost' => {
                      'dnsRecord' => {
                          'ref'  => orig_record[:dns_hosts][:dns_host][:dns_record][:ref],
                          'name' => orig_record[:dns_hosts][:dns_host][:dns_record][:name],
                          'type' => orig_record[:dns_hosts][:dns_host][:dns_record][:type],
                          'ttl'  => orig_record[:dns_hosts][:dns_host][:dns_record][:ttl],
                          'data' => orig_record[:dns_hosts][:dns_host][:dns_record][:data],
                          'enabled' => orig_record[:dns_hosts][:dns_host][:dns_record][:enabled],
                          'dnsZoneRef' => orig_record[:dns_hosts][:dns_host][:dns_record][:dns_zone_ref]
                      },
                      'ptrStatus' => orig_record[:dns_hosts][:dns_host][:ptr_status],
                      'relatedRecords' => orig_record[:dns_hosts][:dns_host][:related_records]
                  },
              },
              'dhcpReservations' => orig_record[:dhcp_reservations],
              'dhcpLeases' => orig_record[:dhcp_leases],
              'discoveryType' => orig_record[:discovery_type],
              'lastSeenDate' => orig_record[:last_seen_date],
              'lastDiscoveryDate' => orig_record[:last_discovery_date],
              'lastKnownClientIdentifier' => orig_record[:last_know_client_identifier],
              'customProperties' => orig_record[:custom_properties]
          },
          'ipamRecordAfter' => {
              'addrRef' => new_record[:addr_ref],
              'address' => new_record[:address],
              'claimed' => new_record[:claimed],
              'dnsHosts'=> {
                  'dnsHost' => {
                      'dnsRecord' => {
                          'ref'  => new_record[:dns_hosts][:dns_host][:dns_record][:ref],
                          'name' => new_record[:dns_hosts][:dns_host][:dns_record][:name],
                          'type' => new_record[:dns_hosts][:dns_host][:dns_record][:type],
                          'ttl'  => new_record[:dns_hosts][:dns_host][:dns_record][:ttl],
                          'data' => new_record[:dns_hosts][:dns_host][:dns_record][:data],
                          'enabled' => new_record[:dns_hosts][:dns_host][:dns_record][:enabled],
                          'dnsZoneRef' => new_record[:dns_hosts][:dns_host][:dns_record][:dns_zone_ref]
                      },
                      'ptrStatus' => new_record[:dns_hosts][:dns_host][:ptr_status],
                      'relatedRecords' => new_record[:dns_hosts][:dns_host][:related_records]
                  },
              },
              'dhcpReservations' => new_record[:dhcp_reservations],
              'dhcpLeases' => new_record[:dhcp_leases],
              'discoveryType' => new_record[:discovery_type],
              'lastSeenDate' => new_record[:last_seen_date],
              'lastDiscoveryDate' => new_record[:last_discovery_date],
              'lastKnownClientIdentifier' => new_record[:last_know_client_identifier],
              'customProperties' => new_record[:custom_properties]
          },
      }
    )
  rescue => err
    $evm.log(:info, "ERR: #{err.inspect}")
  end
  log(:info, "DEBUG: #{response.inspect}")
  #log(:info, "update_upam_record(#{re}): exiting method")
end

def logout
  log(:info, "logout: entering method")
  log(:info, "logout: Logging out #{@session}") unless @session.nil?
  if @client.nil? || @session.nil?
    log(:info, "logout: exiting method")
    return
  end
  #response = @client.request :logout do |soap|
  #  soap.body = {
  #    'session' => @session
  #  }
  #end
  
  @client.call(
    :logout,
    message: {
        'session' => @session
    }
  )
  @session = nil
  log(:info, "logout: exiting method")
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute #{uri}") unless result

  case result['result']
    when 'error'
      error(result['reason'])
    when 'retry'
      retry_method(result['reason'])
  end

  result
end

def get_virtual_machine()
  case $evm.root['vmdb_object_type']
  when 'vm'
    vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

# --- BEGIN MAIN -----#
begin
  # Parameters that were collected by the ReleaseIPAddress instance, which
  # gets them from a Integration/MenAndMice/Configuration/intuit_dc
  # instance.

  intuit_dc = $evm.root['intuit_dc']
  result = $evm.instantiate("/Intuit/Integration/MenAndMice/Configuration/intuit_dc/#{intuit_dc}")
  wsdl_url = result['wsdl']
  central  = result['central']
  username = result['mm_username']
  password = result.decrypt('mm_password')

  # Parameters that were passed via $evm.instantiate()
  ip_addresses = $evm.root['ip_address']
  #dns_zone = $evm.current['dns_zone']

  error("Missing Men & Mice credentials for intuit_dc: [#{intuit_dc}]") if username.blank? || password.blank?

  @client = nil
  @session = nil

  save_session = @session
  # configure HTTPI
  HTTPI.log_level = :info
  HTTPI.log = false
  HTTPI.adapter = :net_http

  # Configure Savon
  #Savon.configure do |config|
  #  config.log = false
  #  config.log_level = :debug
  #end

  #@client = Savon::Client.new do |wsdl,http|
  #  wsdl.document = wsdl_url
  #end

  # Configure Savon
  @client = Savon.client(
      wsdl: wsdl_url,
      log: false,
      log_level: :debug
  )

  log(:info, "INFO: #{ip_addresses}")
#  ip_address = ip_addresses.first
  ip_addresses.each {|ip_address|
    @session = get_session_id(central, username, password)
    #zone_ref = get_dns_zone_ref(dns_zone)

    orig_record = Hash.new
    new_record = Hash.new
    record = get_ipam_record(ip_address)
    second_record  = get_ipam_record(ip_address)
    #record = get_dns_record(zone_ref, ip_address)
    orig_record = record.to_hash[:get_ipam_record_response][:ipam_record]
    log(:info, "OLD RECORD: #{orig_record}")
    new_record  = second_record.to_hash[:get_ipam_record_response][:ipam_record]
    orig_hostname = orig_record[:dns_hosts][:dns_host][:dns_record][:name]
    new_record[:dns_hosts][:dns_host][:dns_record][:name] = "ipparked-#{orig_hostname}"
    #log(:info, "OLD RECORD: #{orig_record}")
    log(:info, "NEW RECORD: #{new_record}")
    if record.nil?
      log(:info, "No DNS record found for IP #{ip_address}")
    elsif record == "parked"
      log(:info, "IP address parked, will not release IP address.")
    else
        update_ipam_record(orig_record, new_record)
    end
  }
  log(:info, "Main: exiting main method")
  # --- END MAIN -----#
  exit MIQ_OK
rescue StandardError, SignalException => err

    retry_method("[#{err}]\n#{err.backtrace.join("\n")}")
ensure
  begin
    log(:info, "Ensuring logout occurs")
    logout unless @session.nil?
  rescue
  end
end
