###################################
#
# CFME Automate Method: create_project_template
#
# Author: Bryan Antigua
#
# Notes: Creates a Project Template
#
# Inputs:
#
# Mods:
#
###################################

# Methods for logging
def log(level, message)
  @method = 'create_project_template'
  $evm.log(level, "#{@method} - #{message}")
end

def dump_current()
  $evm.log(:info, "Current:<$evm.current> Begin $evm.current.attributes")
  $evm.current.attributes.sort.each { |k, v| $evm.log(:info, "Current:<$evm.current> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Current:<$evm.current> End $evm.current.attributes")
  $evm.log(:info, "")
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def error(msg)
  $evm.log(:error, "Creating a Project Template failed: [#{msg}]")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end
 
# process_tags - Dynamically create categories and tags
def process_tags(service,category_name, tag, tag_description )
  # Convert to lower case and replace all non-word characters with underscores
  tag_name = tag.to_s.downcase.gsub(/\W/, '_')
  unless $evm.execute('tag_exists?', category_name, tag_name)
    $evm.log(:info, "Adding new tag <#{tag_name}> description <#{tag_description}> in Category <#{category_name}>")
    $evm.execute('tag_create', category_name, :name => tag_name, :description => "#{tag_description}")
  end
  
  service.tag_assign("#{category_name}/#{tag_name}")
end

begin
  $evm.log("info", "Automate Method Started")
  task = $evm.root['service_template_provision_task']
  service = task.destination
  dialog_options = task.dialog_options
  $evm.log(:info,"the task is #{task} and service is : #{service}")
  $evm.log(:info,"the options to this dialog are: #{dialog_options.inspect}")
  
  service.name = dialog_options['dialog_service_name'] + " (Project Template)"
  process_tags(service,'option_0_application_name', dialog_options['dialog_option_0_application_name'], dialog_options['dialog_option_0_application_name'])
  process_tags(service,'option_0_environment',dialog_options['dialog_option_0_environment'],dialog_options['dialog_option_0_environment'] )
  process_tags(service,'option_0_execution_zone',dialog_options['dialog_option_0_execution_zone'],dialog_options['dialog_option_0_execution_zone'] )
  process_tags(service,'option_0_sox_compliant', dialog_options['dialog_option_0_sox_complinat'],dialog_options['dialog_option_0_sox_complinat'] )
  process_tags(service,'option_0_owner_email', dialog_options['dialog_option_0_owner_email'],dialog_options['dialog_option_0_owner_email'])
  process_tags(service,'option_0_project_acronym',dialog_options['dialog_option_0_project_acronym'],dialog_options['dialog_option_0_project_acronym'] )
  process_tags(service,'intuit_service_name', dialog_options['dialog_service_name'],dialog_options['dialog_service_name'] )
  
  service.tag_assign('project_template/project_template')
  $evm.log(:info,"service is tagged with :#{service.tags()}")
  $evm.log("info", "Automate Method Ended")
  
  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
