###################################
#
# CFME Automate Method: get_bufg_l2_list	
#
# Author: Bryan Antigua
#
# Notes: Retrieves  available BUFG in level 2 from ServiceNow
#
# Inputs: intuit_dc
#
# Mods:
#
###################################

require 'json'
require 'httpclient'

def dump_current()
  $evm.log(:info, "Current:<$evm.current> Begin $evm.current.attributes")
  $evm.current.attributes.sort.each { |k, v| $evm.log(:info, "Current:<$evm.current> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Current:<$evm.current> End $evm.current.attributes")
  $evm.log(:info, "")
end

# Method for logging
def log(level, message)
  @method = 'get_bufg_l2_list'
  $evm.log(level, "#{@method} - #{message}")
end

# dump_root
def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def error(msg)
  $evm.log(:error, "Getting BUFG (Level 2) List failed: [#{msg}]")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance
  
  #$evm.log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance.attributes}]")
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

def get_bufg_list(username,password,domain_url,table_url,proxy)

  #authenticating for ServiceNow Call
  http =  HTTPClient.new(proxy)
  $evm.current['password'] = password
  http.set_auth(domain_url, username, $evm.current.decrypt('password'))
    
  #making GET request to ServiceNow
  resp = http.post(table_url).content
  json_response = JSON.parse(resp)
  records = json_response['records']
  
  if File.exist? '/tmp/bufg_list.json'
    saved_resp = File.read('/tmp/bufg_list.json')
    $evm.log(:info,"they do match - do nothing")
    if records.to_json.to_s != saved_resp.to_s
      $evm.log(:info,"they don't match")
      File.open('/tmp/bufg_list.json', 'w') { |file| file.write(records.to_json) }
    end
  else
    File.open('/tmp/bufg_list.json', 'w') { |file| file.write(records.to_json) }
  end
  
  records
end
  
def get_bufg_l2_list(username,password,domain_url,table_url,proxy) 

  records = get_bufg_list(username,password,domain_url,table_url,proxy)
  l2_records = []
  
  #filter l2 records
  records.each do |record|
    if record['u_parent'] =='482077c26f40210012ea77f16a3ee4eb' && record['u_name'] != "Intuit"
      l2_records << record
    end
  end
  
  l2_records
end

begin
  dump_current()
  
  intuit_dc = $evm.current['intuit_dc']
  error("Intuit DC must be provided") if intuit_dc.blank?
  
  config = {}
  retrieve_configuration(config, 'intuit_dc', intuit_dc)
  bufg_l2_list = get_bufg_l2_list(config['sn_username'], config['sn_password'], config['sn_domain_url'], config['sn_bufg_url'], config['proxy_url'])
  
  $evm.current['bufg_l2_list'] = bufg_l2_list
  $evm.current['result'] = 'ok'
  
  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
