###################################
#
# CFME Automate Method: get_bufg_l4_list	
#
# Author: Bryan Antigua
#
# Notes: Retrieves  available BUFG in level 4 from ServiceNow
#
# Inputs: intuit_dc
#
# Mods:
#
###################################

require 'json'
require 'httpclient'

def dump_current()
  $evm.log(:info, "Current:<$evm.current> Begin $evm.current.attributes")
  $evm.current.attributes.sort.each { |k, v| $evm.log(:info, "Current:<$evm.current> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Current:<$evm.current> End $evm.current.attributes")
  $evm.log(:info, "")
end

# Method for logging
def log(level, message)
  @method = 'get_bufg_l4_list'
  $evm.log(level, "#{@method} - #{message}")
end

# dump_root
def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def error(msg)
  $evm.log(:error, "Getting BUFG (Level 4) List failed: [#{msg}]")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end


def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance
  
  #$evm.log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance.attributes}]")
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

  def get_bufg_list(username,password,domain_url,table_url,proxy)
  records = nil
  
   if File.exists? '/tmp/bufg_list.json'
    records = JSON.parse(File.read('/tmp/bufg_list.json'))
    $evm.log(:info,"got from file!")
  else
    #authenticating for ServiceNow Call
    http =  HTTPClient.new(proxy)
    $evm.current['password'] = password
    http.set_auth(domain_url, username, $evm.current.decrypt('password'))
    
    #making GET request to ServiceNow
    resp = http.post(table_url).content
    json_response = JSON.parse(resp)
    records = json_response['records']
  end
  records
end
 
def get_bufg_l4_list(username,password,domain_url,table_url,proxy) 
  
  records = get_bufg_list(username,password,domain_url,table_url,proxy)
  l2_sys_ids = []
  #filter l2 records
  records.each do |record|
    if record['u_parent'] =='482077c26f40210012ea77f16a3ee4eb' && record['u_name'] != "Intuit"
      l2_sys_ids << record['sys_id']
    end
  end
  
  l3_sys_ids = []
  records.each do |record|
    if l2_sys_ids.include? record['u_parent']
      l3_sys_ids << record['sys_id']
    end
  end

  l4_records = []
  records.each do |record|
    if l3_sys_ids.include? record['u_parent']
      l4_records << record
    end
  end
  
  l4_records
end

begin
  dump_current()
  
  intuit_dc = $evm.current['intuit_dc']
  error("Intuit DC must be provided") if intuit_dc.blank?
  
  config = {}
  retrieve_configuration(config, 'intuit_dc', intuit_dc)
  bufg_l4_list = get_bufg_l4_list(config['sn_username'], config['sn_password'], config['sn_domain_url'], config['sn_bufg_url'], config['proxy_url'])
  
  $evm.current['bufg_l4_list'] = bufg_l4_list
  $evm.current['result'] = 'ok'
  
  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
