###################################
#
# CFME Automate Method: get_execution_subzone_list	
#
# Author: Bryan Antigua
#
# Notes: Retrieves available execution subzones from ServiceNow
#
# Inputs: intuit_dc
#
# Mods:
#
###################################

require 'json'
require 'httpclient'

EXEC_ZONE_1 = "Cloud9"

def dump_current()
  $evm.log(:info, "Current:<$evm.current> Begin $evm.current.attributes")
  $evm.current.attributes.sort.each { |k, v| $evm.log(:info, "Current:<$evm.current> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Current:<$evm.current> End $evm.current.attributes")
  $evm.log(:info, "")
end

# Method for logging
def log(level, message)
  @method = 'get_execution_subzone_list'
  $evm.log(level, "#{@method} - #{message}")
end

# dump_root
def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def error(msg)
  $evm.log(:error, "Getting Execution Subzone List failed: [#{msg}]")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end


def retrieve_configuration(config, class_name, instance_name)
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instantiate(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance
  
  #$evm.log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance.attributes}]")
  instance.attributes.each { |k, _| config[k] = instance.decrypt(k) }
end

 
def get_service_now_execution_subzone_list(username,password,domain_url,table_url,proxy) 
  #authenticating for ServiceNow Call
  http =  HTTPClient.new(proxy)
  $evm.current['password'] = password
  http.set_auth(domain_url, username, $evm.current.decrypt('password'))
  
  #making GET request to ServiceNow
  resp = http.post(table_url).content
  json_response = JSON.parse(resp)
  records = json_response['records']
end

begin
  dump_current()
  
  intuit_dc = $evm.current['intuit_dc']
  error("Intuit DC must be provided") if intuit_dc.blank?
  
  config = {}
  retrieve_configuration(config, 'intuit_dc', intuit_dc)
  execution_subzone_list = get_service_now_execution_subzone_list(config['sn_username'], config['sn_password'], config['sn_domain_url'], config['sn_execution_subzone_url'], config['proxy_url'])
  #remove Cloud9
  execution_subzone_list.delete_if{|zone| zone['label'].eql? EXEC_ZONE_1}
  
  $evm.current['execution_subzone_list'] = execution_subzone_list
  $evm.current['result'] = 'ok'
  
  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
