##################################
#
# CFME Automate Method: unread_storage_info
#
# Author: Daniel Trieu
#
# Notes: Delete storage services config, delete service
#
###################################
require 'restclient'
require 'json'

# logging convenience method
def log(level, msg)
  @method = 'unread_storage_info'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def to_bool(value)
  return true if value == true || value =~ (/^(true|t|yes|y|1)$/i)
  return false if value == false || value.blank? || value =~ (/^(false|f|no|n|0)$/i)
  error("Invalid value for Boolean: to_bool(#{value})")
end

def retrieve_configuration(class_name, instance_name)
  log(:info, "retrieve_configuration: #{class_name} #{instance_name}")
  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  instance = $evm.instance_find(instance_uri)
  error("Unable to retrieve required configuration: [#{instance_uri}]") unless instance

  log(:info, "Retrieved configuration: [#{class_name}/#{instance_name}[#{instance}]")
  instance[instance_name]
end

def create_configuration(new_config, class_name, instance_name)
  log(:info, "create_configuration: #{new_config} #{class_name} #{instance_name}")

  instance_uri = "#{$evm.current_namespace}/Configuration/#{class_name}/#{instance_name}"
  result = $evm.instance_create(instance_uri, new_config)
  error("Unable to create configuration") unless to_bool(result)

  result
end

def delete_configuration(class_name, instance_name)
  log(:info,"delete_configuration: #{class_name} #{instance_name}")

  log(:info, "NOTHING DONE")
end

def tag_service(service, category, tag_value)
  $evm.execute('category_create', :name => category, :single_value => false, :description=> category) unless $evm.execute('category_exists?', category)
  tag = tag_value.downcase unless tag_value.nil?

  log(:info, "Tag value: #{tag}")
  if $evm.execute('tag_exists?', category, tag)
    service.tag_assign(category+"/"+tag)
  else
    $evm.execute('tag_create', category, :name => tag, :description => tag)
    service.tag_assign(category+"/"+tag)
  end
end

begin
  log(:info, "Starting Method")

  service = $evm.root['service']

  storage_object_id		= service.tags(category='intuit_storage_object_id').first
  delete_configuration('intuit_ss', storage_object_id)

  log(:info, "Retiring service: #{service.name}")
  service.retire_now()

  log(:info, "Ending Method")
  exit MIQ_OK
rescue StandardError, SignalException => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
