###################################
#
# CFME Automate Method: get_vix_platform
#
# Author: Jason Dillaman
#
# Notes: Extracts a VM's product name and determines a VIX platform
#
# Inputs: vm: the target VM
#
# Mods:
#
###################################

# dump_root
def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def error(msg)
  $evm.log(:error, "VIX platform detection failed: [#{msg}]")
  $evm.current['result'] = 'error'
  $evm.current['reason'] = msg.to_s
  exit MIQ_OK
end

def get_vm()
  return $evm.root['vm'] unless $evm.root['vm'].blank?
  case $evm.root['vmdb_object_type']
    when 'miq_provision'
      vm = $evm.root['miq_provision'].vm
    when 'vm'
      vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

begin
  dump_root()
  vm = get_vm()

  product_name = vm.operating_system.product_name
  if product_name =~ /(rhel|linux|centos)/i
    vix_platform = 'linux'
  elsif product_name =~ /windows/i
    vix_platform = 'windows'
  elsif product_name =~ /other/i
      vix_platform = 'linux'
  else
    #vm.reload
    # Issue vm.refresh to get correct vix plaform. reload not working.
    $evm.log(:info, "Product name before vm.refresh: #{product_name}")
    vm.refresh
    $evm.log(:info, "Product name after vm.refresh: #{vm.operating_system.product_name}")
    error("Unable to determine VIX platform: [#{product_name}]")
  end

  $evm.log(:info, "Detected VIX plaform: [#{vix_platform}][#{product_name}]")
  $evm.current['result'] = 'ok'
  $evm.current['vix_platform'] = vix_platform

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
