require 'rubygems'
require 'json'
require 'net/http'
require 'uri'

begin

def log (level, msg)
    $evm.log(level, "<Fix BU Tag Automation>: #{msg}")
end

# dump_vm_tags
def dump_vm_tags( vm, vmtags )
  log(:info, "VM:<#{vm.name}> Begin dump_vm_tags")
  vmtags.sort.each { |k, v| log(:info, "VM:<#{vm.name}> tag - #{k}: #{v}")}
  log(:info, "VM:<#{vm.name}> End dump_vm_tags")
  log(:info, "")
end



def tag_vm vm,category,tag
  log(:info, "About to apply #{category} tag with #{tag} to VM #{vm.name}")
  unless @justlog
    tagDesc = tag
    #log(:info, "Tag value received - vm: #{vm.name}, category: #{category}, tag description: #{tagDesc}")
    #strip non alphanumeric characters
    tag = tag.gsub(/[^\p{Alnum}\p{Space}_]/u, '')
    #downcase and convert whitespace to underscore
    tag = tag.to_s.downcase.gsub(/\W/, '_')
    #limit to 1 "_" in a row
    tag = tag.gsub(/_+/,'_')
    #truncate to a max tag name length of 30 chars
    tag = tag[0..29]
    log(:info, "Tag value processed - vm: #{vm.name}, category: #{category}, tag: #{tag}, tag description: #{tagDesc}")
    begin
      if $evm.execute('tag_exists?', category, tag)
        vm.tag_assign(category+"/"+tag)
      else
        $evm.execute('tag_create', category, :name => tag, :description => tagDesc)
        vm.tag_assign(category+"/"+tag) 
      end    
      rescue Exception => e
        log(:info, "IMPORT FAIL @tag_vm: unable to complete metadata import for #{vm.name} - #{e}")
    end
  end
end

def tag_vm_with_bu vm,vm_glass_data
  log(:info, "Tagging VM #{vm.name}")
  
  begin
    meta_info = vm_glass_data["meta_info"]
    if !meta_info.nil?
        if !meta_info["bu"].nil?
          tag_vm vm,"intuit_bu",meta_info["bu"].upcase
          tag_vm vm,"intuit_filter_bu",meta_info["bu"].upcase
        else
          tag_vm vm,"intuit_bu", "Unknown"
          tag_vm vm,"intuit_filter_bu", "Unknown"
        end
    else
      log(:info, "Tagging fail @tag_non_pharos: meta_info is nil, no metadata import for #{vm.name}")
    end
  rescue Exception => e
    log(:info, "IMPORT FAIL @tag_non_pharos: unable to complete metadata import for #{vm.name} - #{e}")
    return
  end  

  log(:info, "Done tagging VM #{vm.name}")
end

def search_glass vm_name
  log(:info, "Glass search for #{vm_name}")     

  begin
      mh_hostname = @mh.hostname
      lab_host = mh_hostname.include?("corplab")
      app_url = $evm.object['glass_url']
      url = "#{app_url}glass/search/#{vm_name}" 
      enc_uri = URI.escape(url)

      if lab_host
        log(:info, "Searching using (lab proxy) url #{enc_uri}")
        proxy_addr = '10.152.252.17'
        proxy_port = 3128
        log(:info, "Searching using (labhost) url #{enc_uri}")
        resp = Net::HTTP::Proxy(proxy_addr, proxy_port).get_response(URI.parse(enc_uri))
      else
        log(:info, "Searching using (prodhost) url #{enc_uri}")

        log(:info, "Searching enc_url #{enc_uri}")
        resp = Net::HTTP.get_response(URI.parse(enc_uri))

      end
    rescue Exception => e
      log(:info, "Error. #{e}")
      log(:info, "Search Glass FAIL @end_of_search_glass: unable to complete BU tag fix for #{vm_name}")
      return ""
  end
  return JSON.parse(resp.body)
end

def query_glass_and_tag_vm vm
  log(:info, "Inside query_glass_and_tag_vm, Searching glass for #{vm.name}")
  begin
    vm_glass_data = search_glass vm.name
    tag_vm_with_bu vm,vm_glass_data unless vm_glass_data.empty? 
    if vm_glass_data.empty?
      log(:info, "Inside query_glass_and_tag_vm, no glass data found for VM #{vm.name}")
    end
  rescue Exception => e
    log(:info, "Error. #{e}")
  end
end

def fix_tags vm
  log(:info, "Begin fix_tags for #{vm.name}")
  begin
    #dump_vm_tags( vm, vm.tags )
    bu_found = false
    for tag in vm.tags.sort
      #log(:info, "VM Tag: #{tag}")
      if tag.start_with?('intuit_bu')
        bu_found = true
        bu_tag = tag
      end
    end

    if bu_found
      log(:info, "The VM #{vm.name} already has a BU tag: #{bu_tag} ")
      if (bu_tag.casecmp('intuit_bu/unknown'))
        log(:info, "However, the BU tag is invalid")
        query_glass_and_tag_vm vm
      end 
    else
      log(:info, "VM #{vm.name} found with no BU tag, fixing...")
      query_glass_and_tag_vm vm
    end
  rescue Exception => e
    log(:info, "Error. #{e}")
    log(:info, "Fail when checking for intuit_template: #{vm.name}")
  end
end

def process_vms
  begin
    @vmsToSkip = $evm.current['vm_count_to_skip']
    log(:info, "Automation will skip #{@vmsToSkip} VMs!")
  rescue Exception => e
    @vmsToSkip = 0
    log(:info, "Fail setting VM count to skip - forcing 0")
  end

  counter = 1
  @mh.vms.each do |vm|
    log(:info, "---------------------------------------------------------------------------")
    GC.stat
    if counter > @vmsToSkip
      log(:info, "Processing vm #{counter} named #{vm.name}")
      log(:info, "---------------------------------------------------------------------------")
      fix_tags vm
      ObjectSpace.garbage_collect
      sleep 1
    else
      log(:info, "Skipping vm #{counter} named #{vm.name}")
      log(:info, "---------------------------------------------------------------------------")
    end
    counter+=1

    #debug...
    #if counter > 20
      #log(:info, "DEBUG: ending process_vms after 20 VM")
      #return
    #end
    #end debug...
  end
end

###################################
# Method: boolean
# Returns: true/false
###################################
def boolean(string)
  return true if string == true || string =~ (/(true|t|yes|y|1)$/i)
  return false if string == false || string.nil? || string =~ (/(false|f|no|n|0)$/i)

  # Return false if string does not match any of the above
  log(:info, " - Invalid boolean string:<#{string}> detected. Returning false") if @debug
  return false
end


log(:info, "BU Tagging Automate Method Started")

@mh = nil
@mh = $evm.root['ext_management_system'] if $evm.root['vmdb_object_type']=='ext_management_system' 
log(:info, "Got Management host from $evm.root['ext_management_system']")
if @mh.nil?
  log(:info, "Management host object not found")
  raise "#{@method} - Management host object not found"
end
log(:info, "Management host Found:<#{@mh.name}>")

@justlog = nil
begin
  @justlog = boolean($evm.current['log_only'])
  if @justlog
    log(:info, "Automation will only log data NO TAGGING!")
  else
    log(:info, "Automation will log data AND TAG VMs TOO!")
  end
rescue Exception => e
  @justlog = true
  log(:info, "Fail setting just log data - forcing true")
end

#for testing always just log
#@justlog = true

process_vms

log("info", "BU Tagging Automate Method Ended")
exit MIQ_OK
end
