require 'rubygems'
require 'json'
require 'net/http'
require 'uri'

begin

def log (level, msg)
    $evm.log(level, "<Import MetaData Automation>: #{msg}")
end


def error(msg)
  log(:error, "<Import MetaData Automation>: #{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end


# dump_root
def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end


# dump_current
def dump_current()
  log(:info, "Current:<$evm.current> Begin $evm.current.attributes")
  $evm.current.attributes.sort.each { |k, v| log(:info, "Current:<$evm.current> Attribute - #{k}: #{v}")}
  log(:info, "Current:<$evm.current> End $evm.current.attributes")
  log(:info, "")
end


# dump_parent
def dump_parent()
  if $evm.parent
    log(:info, "Parent:<$evm.parent> Begin $evm.parent.attributes")
    $evm.parent.attributes.sort.each { |k, v| log(:info, "Parent:<$evm.parent> Attribute - #{k}: #{v}")}
    log(:info, "Parent:<$evm.parent> End $evm.parent.attributes")
    log(:info, "")
  end
end


# dump_vm
def dump_vm( vm )
  log(:info, "VM:<#{vm.name}> Begin vm.attributes")
  vm.attributes.sort.each { |k, v| log(:info, "VM:<#{vm.name}> Attribute - #{k}: #{v}")}
  log(:info, "VM:<#{vm.name}> End vm.attributes")
  log(:info, "")
end


def get_pharos_vm_details vm_name
  log(:info, "About to lookup webapp for pharos details for VM  #{vm_name}")
  app_url = $evm.object['glass_url']
  #glass_url= http://qycopspxy01.corp.intuit.net/ 
  url = "#{app_url}pharos/search/#{vm_name}" 
  enc_uri = URI.escape(url) 
    begin
        log(:info, "Searching using (prodhost) url #{enc_uri}")
        #url encoding required only here because the other lookup is based on id
        resp = Net::HTTP.get_response(URI.parse(enc_uri))
      rescue Exception => e
        log(:info, "Error. #{e}")
        log(:info, "Pharos search query failed")
        log(:info, "IMPORT FAIL @get_pharos_vm_details: unable to complete metadata import for #{vm_name}")
        vm_details = nil
    end
 
  log(:info, "resp inspect #{resp.inspect}")
  vm_details = JSON.parse(resp.body)
  return vm_details      
end


def pharos_vm_import vm
  log(:info, "Pharos vm import method called!")
  pharos_vm_details = get_pharos_vm_details vm.name
  if pharos_vm_details.nil? or pharos_vm_details.empty?
    log(:info, "Unable to retreive info from Pharos db, proceeding with non pharos vm import #{vm.name}")         
    non_pharos_vm_import vm
  else
     tag_pharos_imported vm,pharos_vm_details 
     log(:info, "Tagging complete,  #{vm.name} ")      
  end
end

  
def tag_vm vm,category,tag
  unless @justlog
    tagDesc = tag
    #log(:info, "Tag value received - vm: #{vm.name}, category: #{category}, tag description: #{tagDesc}")
    #strip non alphanumeric characters
    tag = tag.gsub(/[^\p{Alnum}\p{Space}_]/u, '')
    #downcase and convert whitespace to underscore
    tag = tag.to_s.downcase.gsub(/\W/, '_')
    #limit to 1 "_" in a row
    tag = tag.gsub(/_+/,'_')
    #truncate to a max tag name length of 30 chars
    tag = tag[0..29]
    log(:info, "Tag value processed - vm: #{vm.name}, category: #{category}, tag: #{tag}, tag description: #{tagDesc}")
    begin
      if $evm.execute('tag_exists?', category, tag)
        vm.tag_assign(category+"/"+tag)
      else
        $evm.execute('tag_create', category, :name => tag, :description => tagDesc)
        vm.tag_assign(category+"/"+tag) 
      end    
      rescue Exception => e
        log(:info, "IMPORT FAIL @tag_vm: unable to complete metadata import for #{vm.name} - #{e}")
        vm.tag_assign("metadata_import/import_failed")
    end
  end
end


def set_custom_attribute vm, attrib, attrib_value
  unless @justlog
    begin
      vm.custom_set(attrib, attrib_value) unless attrib_value.nil?    
    rescue Exception => e
      log(:info, "IMPORT FAIL @set_custom_attribute: unable to complete metadata import for #{vm.name} - #{e}")
      tag_vm vm,"metadata_import","import_failed"
    end
  end
end


def tag_pharos_imported vm,vm_pharos_data
    begin
      log(:info, "Started @tag_pharos_imported for #{vm.name}")

      if vm_pharos_data.nil?
        log(:info, "ERROR! vm_pharos_data is nil @tag_pharos_imported for #{vm.name}")
      else
        log(:info, "vm_pharos_data:#{vm_pharos_data}")
      end

      meta_info = vm_pharos_data["meta_info"]

      if !meta_info.nil?
        tag_vm vm,"intuit_bu",meta_info["bu"].upcase unless  meta_info["bu"].nil?
        tag_vm vm,"intuit_project_name",meta_info["project_name"] unless  meta_info["project_name"].nil?
        tag_vm vm,"intuit_software_component",meta_info["service_comp_name"] unless  meta_info["service_comp_name"].nil?
        tag_vm vm,"intuit_pharos_import_env",meta_info["env_type"] unless  meta_info["env_type"].nil?
        tag_vm vm,"intuit_security_level",meta_info["security_level"] unless  meta_info["security_level"].nil?
        tag_vm vm,"intuit_fd",meta_info["fault_domain"] unless  meta_info["fault_domain"].nil?
        tag_vm vm,"intuit_instance_size",meta_info["instance_size"] unless  meta_info["instance_size"].nil?
        tag_vm vm,"intuit_sz",meta_info["security_zone"] unless  meta_info["security_zone"].nil?
        tag_vm vm,"intuit_hs",meta_info["hosting_segment_name"] unless  meta_info["hosting_segment_name"].nil?
        
        #changing this to a custom attribute because if we set this as a tag, it will not try again (since the tag for this is the test for CF provisioning)
        set_custom_attribute vm, "Pharos_Template", meta_info["template"]
        set_custom_attribute vm, "Pharos_IP_Address", meta_info["ip_address"]
        set_custom_attribute vm, "Pharos_NFS_Hostname", meta_info["nfs_host_name"]
        set_custom_attribute vm, "Pharos_NFS_IP_Address", meta_info["nfs_ip_address"]
        set_custom_attribute vm, "Pharos_Created_By", meta_info["created_by"]
        set_custom_attribute vm, "Pharos_Created_On", meta_info["created_on"]

        set_custom_attribute vm, "Pharos_Tier_Name", meta_info["service_comp_name"]      
        set_custom_attribute vm, "Pharos_Service_Name", meta_info["service_name"]

        #this does not work yet
        #assign_vm_to_service vm,meta_info["service_name"],meta_info["service_name"] unless meta_info["service_name"].nil?
      else
        log(:info, "meta_info is nil @tag_pharos_imported for #{vm.name}")
      end

      storage_details = vm_pharos_data["storage"]
      if !storage_details.nil?
          set_custom_attribute vm, "Pharos_Storage_block_id_1", storage_details["block_id"]
          set_custom_attribute vm, "Pharos_Storage_full_path_1", storage_details["full_path"]
          set_custom_attribute vm, "Pharos_Storage_lun_path_1", storage_details["lun_path"]
          set_custom_attribute vm, "Pharos_Storage_cmode_cluster_1", storage_details["cmode_cluster"]
          set_custom_attribute vm, "Pharos_Storage_management_host_1", storage_details["management_host"]
          set_custom_attribute vm, "Pharos_Storage_Export_Type_1", storage_details["export_type"]
          set_custom_attribute vm, "Pharos_Storage_cmode_1", storage_details["cmode"]
      else
        log(:info, "storage_details is nil @tag_pharos_imported for #{vm.name}")
      end
    rescue Exception => e
      log(:info, "IMPORT FAIL @tag_pharos_imported unable to complete storage metadata import for #{vm.name} - #{e}")
      tag_vm vm,"metadata_import","import_failed"
      return  
    end

    tag_vm vm,"metadata_import","pharos_updated"        
end


def tag_non_pharos vm,vm_non_pharos
  log(:info, "Tagging non pharos VM #{vm.name}")
  
  begin
    meta_info = vm_non_pharos["meta_info"]
    if !meta_info.nil?
      tag_vm vm,"intuit_bu",meta_info["bu"].upcase unless  meta_info["bu"].nil?
      tag_vm vm,"intuit_project_name",meta_info["project_name"] unless  meta_info["project_name"].nil?
      tag_vm vm,"intuit_hs",meta_info["hosting_segment"] unless  meta_info["hosting_segment"].nil?
      tag_vm vm,"intuit_sz",meta_info["security_zone"] unless  meta_info["security_zone"].nil?
      set_custom_attribute vm, "Intuit_Non_Pharos_Owner", meta_info["owner"]
      unless meta_info["ip_address"].nil?
        ip_addresses = meta_info["ip_address"].split(";")
        ip_addresses.each_with_index do |value,index|
          set_custom_attribute vm, "Intuit_Non_Pharos_IP_Address_#{index+1}", value
        end
      end
    else
      log(:info, "Tagging fail @tag_non_pharos: meta_info is nil, no metadata import for #{vm.name}")
    end
  rescue Exception => e
    log(:info, "IMPORT FAIL @tag_non_pharos: unable to complete metadata import for #{vm.name} - #{e}")
    tag_vm vm,"metadata_import","import_failed"
    return
  end  

  begin
    storage_details = vm_non_pharos["storage"]
    unless storage_details.nil?
      st_index = 1
      storage_details.each_with_index do |value,index|
        set_custom_attribute vm, "Storage_full_path_#{index+1}", value["full_path"]
        set_custom_attribute vm, "Storage_cmode_cluster_#{index+1}", value["cmode_cluster"]
        set_custom_attribute vm, "Storage_management_host_#{index+1}", value["management_host"]
        set_custom_attribute vm, "Storage_Export_Type_#{index+1}", value["export_type"]
        set_custom_attribute vm, "Storage_cmode_#{index+1}", value["cmode"]
      end    
    end 
  rescue Exception => e
    log(:info, "IMPORT FAIL @tag_non_pharos: unable to complete storage metadata import for #{vm.name} - #{e}")
    tag_vm vm,"metadata_import","import_failed"
    return
  end

  tag_vm vm,"metadata_import","non_pharos_updated"     

  log(:info, "Done tagging non pharos VM #{vm.name}")
end


def search_glass vm_name
  log(:info, "Glass search for #{vm_name}")     
  begin
      app_url = $evm.object['glass_url']
      url = "#{app_url}glass/search/#{vm_name}" 
      enc_uri = URI.escape(url)
      log(:info, "Searching url #{url}")
      log(:info, "Searching enc_url #{enc_uri}")
      resp = Net::HTTP.get_response(URI.parse(enc_uri))
    rescue Exception => e
      log(:info, "Error. #{e}")
      log(:info, "IMPORT FAIL @end_of_search_glass: unable to complete metadata import for #{vm_name}")
      tag_vm vm,"metadata_import","import_failed"
  end
  return JSON.parse(resp.body)
end



def non_pharos_vm_import vm
  begin
    log(:info, "Inside non pharos vm import, Searching glass for #{vm.name}")

    vm_non_pharos = search_glass vm.name    
    tag_non_pharos vm,vm_non_pharos unless vm_non_pharos.empty? 
    if vm_non_pharos.empty?
      log(:info, "Inside non pharos vm import, failed trying to tag #{vm.name}")
      tag_vm vm,"metadata_import","import_failed"
    end
  rescue Exception => e
    log(:info, "Error. #{e}")
    log(:info, "Fail during non_pharos_vm_import")
    log(:info, "IMPORT FAIL @non_pharos_vm_import: unable to complete metadata import for #{vm.name}")
    tag_vm vm,"metadata_import","import_failed"
  end
end


def meta_data_import vm
  log(:info, "Begin meta data import for #{vm.name}")
  begin
    unless vm.tags.any? { |s| s.start_with?('intuit_template') }
      log(:info, "Non Cloudforms provisioned VM:  #{vm.name}")
      pharos_vm_import vm
    else
      log(:info, "This host was provisioned by Cloudforms, skipping: #{vm.name}")
    end   
  rescue Exception => e
    log(:info, "Error. #{e}")
    log(:info, "Fail when checking for intuit_template: #{vm.name}")
  end
end

#kiruba
#service assignment for pharos -> Not sure if this is really needed to assign names.
def name_service(service, new_service_name, new_service_description)
  unless new_service_name.blank?
    $evm.log(:info, "Changing Service name:<#{service.name}> to <#{new_service_name}>")
  else
    new_service_name = "#{service.name}-#{Time.now.strftime('%Y%m%d-%H%M%S')}"
    $evm.log(:info, "Changing Service name:<#{service.name}> to <#{new_service_name}>")
  end
  service.name = new_service_name
  service.description = new_service_description
end


def create_service_group service_name, service_description
  # How to create a service group programatically with only service_name & service_description ?
  #task = $evm.root['service_template_provision_task']
  #service = task.destination
  #name_service(service, service_name, service_description)
  return nil
end


def assign_vm_to_service vm,pharos_service_name, pharos_service_description
  log(:info, "Service lookup for : #{pharos_service_name.downcase}")
  service = $evm.vmdb('service').find_by_name(pharos_service_name.downcase)
  if service.nil?
    log(:info, "Service does not exist, needs to be created : #{pharos_service_name.downcase}") #create service names downcased.
    if !@justlog
      service = create_service_group pharos_service_name, pharos_service_description 
    end    
  end
  log(:info, "Service found, add the service to vm : #{service.name}  #{vm.name}") unless service.nil?
  if !@justlog
    vm.add_to_service(service) unless service.nil?
  end    
end



def report_memory
    #memory = 'Memory ' + `ps ax -o pid,rss | grep -E "^[[:space:]]*#{$$}"`.strip.split.map(&:to_i)[1].to_s + 'KB'
    #puts memory
    log(:info, 'Memory ' + `ps ax -o pid,rss | grep -E "^[[:space:]]*#{$$}"`.strip.split.map(&:to_i)[1].to_s + 'KB')
end


def process_vms
  begin
    @vmsToSkip = $evm.current['vm_count_to_skip']
    log(:info, "Import Single VM Automation will skip #{@vmsToSkip} VMs!")
  rescue Exception => e
    @vmsToSkip = 0
    log(:info, "Fail setting VM count to skip - forcing 0")
  end

  counter = 1
  @mh.vms.each do |vm|
    log(:info, "---------------------------------------------------------------------------")
    report_memory
    GC.stat
    if counter > @vmsToSkip
      log(:info, "Processing vm #{counter} named #{vm.name}")
      log(:info, "---------------------------------------------------------------------------")
      meta_data_import vm
      #GC#start
      ObjectSpace.garbage_collect
      sleep 1
    else
      log(:info, "Skipping vm #{counter} named #{vm.name}")
      log(:info, "---------------------------------------------------------------------------")
    end
    counter+=1
  end
end


def get_virtual_machine()
  vm = $evm.root['vm']
  error("Failed to retrieve VM") unless vm
  log(:info, "Located VM: [#{vm.name}]")
  #dump_vm( vm )
  vm
end

###################################
# Method: boolean
# Returns: true/false
###################################
def boolean(string)
  return true if string == true || string =~ (/(true|t|yes|y|1)$/i)
  return false if string == false || string.nil? || string =~ (/(false|f|no|n|0)$/i)

  # Return false if string does not match any of the above
  log(:info, " - Invalid boolean string:<#{string}> detected. Returning false") if @debug
  return false
end


log(:info, "Non CF Data Import Automate Method Started")

#dump_current()

@mh = nil
@mh = $evm.root['ext_management_system'] if $evm.root['vmdb_object_type']=='ext_management_system' 
log(:info, "Got Management host from $evm.root['ext_management_system']")
if @mh.nil?
  log(:info, "Management host object not found")
  raise "#{@method} - Management host object not found"
end
log(:info, "Management host Found:<#{@mh.name}>")

@justlog = nil
begin
  @justlog = boolean($evm.current['log_only'])
  if @justlog
    log(:info, "Import Single VM Automation will only log data NO TAGGING!")
  else
    log(:info, "Import Single VM Automation will log data AND TAG VMs TOO!")
  end
rescue Exception => e
  @justlog = true
  log(:info, "Fail setting just log data - forcing true")
end

process_vms

log("info", "Non CF Data Import Automate Method Ended")
exit MIQ_OK
end
