###################################
#
# CFME Automate Method: pharos_storage_cleanup
#
# Notes: This method calls a proxy service to delete network storage associated with a (Pharos) VM
#
###################################

require 'rubygems'
require 'json'
require 'net/http'
require 'uri'

# overfide of log
def log (level, msg)
    $evm.log(level, "<Delete Network Storage Automation>: #{msg}")
end

# overide of error
def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def get_region vc
#TODO: find a less code specific way to get the region
  #prod vCenter servers
  #QDC-E Prod       mprdvctis600.corp.intuit.net
  #QDC-A Prod       mprdvctis100.corp.intuit.net
  #QDC-A Pre-Prod   mprdvctis101.corp.intuit.net
  #QDC-C Prod       mprdvctis300.corp.intuit.net
  #QDC-C Pre-Prod   mprdvctis301.corp.intuit.net

  #LVDC-B Prod      mprdvctis700.corp.intuit.net
  #LVDC-A Prod      mprdvctis400.corp.intuit.net
  #LVDC-A Pre-Prod  mprdvctis401.corp.intuit.net

  # handle prod
  log(:info, "Region ID: #{vc.region_number}")
  if vc.region_number == 509
    log(:info, "Region: QDC")
    return "qdc"
  elsif vc.region_number == 702
    log(:info, "Region: LVDC")
    return "lvdc"
  else
    return "unknown"
  end
end

def storage_cleanup vm
  log(:info, "Starting storage cleanup")
  vm_tags = vm.tags.inspect
  log(:info, "VM Tags: #{vm_tags}")

  unless vm.ext_management_system.nil?
    vm_vcenter = vm.ext_management_system
    log(:info, "Found VMs vCenter: #{vm_vcenter.name}")
    region = get_region vm_vcenter
    if region.casecmp("unknown") == 0
      log(:info, "Unable to determine the region")
    end
    log(:info, "Found VMs region: #{region}")
  end

  begin
    #metadata_import/pharos_updated
    if vm_tags.include? "metadata_import/pharos_updated"
      log(:info, "VM is a Pharos VM")
      storage_block_id = nil
      storage_block_id ||= vm.custom_get("Pharos_Storage_block_id_1")
      log(:info, "Storage found: #{storage_block_id}")
    end    
  rescue SignalException, StandardError => e
    log(:info,"exception occured getting storage_block_id #{e}")
    exit MIQ_OK
  end
  
  begin
    hostname = vm.name

    #QDC delete url :   http://qycopspxy01.corp.intuit.net/pharos/storage_cleanup?host_name=#{hostname}&sb_id=#{sb_ids[0]}
    #LVDC delete url : http://lvcopspxy001.corp.intuit.net/pharos/storage_cleanup?host_name=#{hostname}&sb_id=#{sb_ids[0]}

    if region.casecmp("qdc") == 0
      url = "http://qycopspxy01.corp.intuit.net/pharos/storage_cleanup?host_name=#{hostname}&sb_id=#{storage_block_id}"
    else
      url = "http://lvcopspxy001.corp.intuit.net/pharos/storage_cleanup?host_name=#{hostname}&sb_id=#{storage_block_id}"
    end

    log(:info,"url to be called with #{hostname} and sb_id #{storage_block_id}")
    log(:info,"url: #{url}")
    resp = Net::HTTP.get_response(URI.parse(url))
    data = resp.body
    result = JSON.parse(data)
  rescue SignalException, StandardError => e
    log(:info,"exception occured calling storage cleanup service #{e}")
    exit MIQ_OK
  end  
  log(:info, "Status of the cleanup job #{result["status"]}")
  log(:info, "Details of the cleanup job #{result["msg"]}") 
end  

def get_virtual_machine()
  vm = $evm.root['vm']
  error("Failed to retrieve VM") unless vm
  log(:info, "Located VM: [#{vm.name}]")
  vm
end

begin
  ###################################
  log(:info, "pharos_storage_cleanup method started")

  target_vm = get_virtual_machine()
  storage_cleanup target_vm

  log("info", "pharos_storage_cleanup method ended")
  exit MIQ_OK
end
