###################################
#
# CFME Automate Method: discover_region_endpoints
#
# Author: Jason Dillaman
#
# Notes: Locates the CFME Web Service Endpoints for local regions
#
# Inputs: vm_provision_requests within task
#
# Mods:
#
###################################

DEFAULT_REGION_FACTOR = 1_000_000_000_000

def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

# dump_root
def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def region_number(object)
  object.id.to_i / DEFAULT_REGION_FACTOR
end

begin
  $evm.log(:info, 'CFME Automate Method Started')

  # dump all root attributes to the log
  #dump_root

  case $evm.root['vmdb_object_type']
  when 'service_template_provision_task'
    # executed via service catalog
    task = $evm.root['service_template_provision_task']
    vm_provision_requests = task.options[:vm_provision_requests]
  when 'service'
    vm_provision_requests = $evm.root[:vm_provision_requests]
  else
    raise "Invalid $evm.root['vmdb_object_type']:<#{$evm.root['vmdb_object_type']}>"
  end
  
  error("Missing VM provision request") unless vm_provision_requests
  
  vm_provision_requests.each do |vm_provision_request|
    $evm.log(:info, "Processing VM provision request: [#{vm_provision_request}]")
    
    intuit_hs = vm_provision_request[:tags][:intuit_hs]
    emses = $evm.vmdb('ext_management_system').all.select do |ems|
      ems.tagged_with?('intuit_hs', intuit_hs)
    end
    error("Unable to find vCenter for hosting segment: [#{intuit_hs}]") if emses.blank?
    region = region_number(emses.first)    
    
    intuit_dc = emses.first.tags(:intuit_dc).first
    error("Unable to locate Intuit Data Center tag on vCenter: [#{emses.first.name}]") if intuit_dc.blank?

    servers = $evm.vmdb('miq_server').all.select do |server| 
      region_number(server) == region && server.has_active_webservices
    end.shuffle
    error("Unable to find CloudForms web service for hosting segment: [#{intuit_hs}]") if servers.blank?
    
    vm_provision_request[:endpoint_intuit_dc] = intuit_dc
    vm_provision_request[:endpoint_ip_addresses] = servers.collect { |server| server.ipaddress }
    $evm.log(:info, "CloudForms Web Service Endpoints: [#{vm_provision_request[:endpoint_intuit_dc]}]#{vm_provision_request[:endpoint_ip_addresses] }")
  end
  
  $evm.log(:info, "VM Provision Requests: [#{vm_provision_requests}]")

  if task.nil?
    $evm.root[:vm_provision_requests] = vm_provision_requests
  else
    task.set_option(:vm_provision_requests, vm_provision_requests)
  end
  $evm.log(:info, 'CFME Automate Method Ended')
  exit MIQ_OK

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
