###################################
#
# CFME Automate Method: initiate_provision_request
#
# Author: Jason Dillaman
#
# Notes: Initiate VM provision SOAP requests to subordinate regions
#
# Inputs: vm_provision_requests within task
#
# Mods:
#
###################################

def escape_uri(value)
  URI.escape(value, Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

# dump_root
def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

begin
  $evm.log(:info, 'CFME Automate Method Started')

  # dump all root attributes to the log
  dump_root

  case $evm.root['vmdb_object_type']
  when 'service_template_provision_task'
    # executed via service catalog
    task = $evm.root['service_template_provision_task']
    vm_provision_requests = task.options[:vm_provision_requests]
  when 'service'
    vm_provision_requests = $evm.root[:vm_provision_requests]
  else
    raise "Invalid $evm.root['vmdb_object_type']:<#{$evm.root['vmdb_object_type']}>"
  end

  
  error("Missing VM provision request") unless vm_provision_requests
  
  vm_provision_requests.each do |vm_provision_request|
    unless vm_provision_request[:result].blank? || vm_provision_request[:result] == 'retry'
      $evm.log(:info, "Skipping request: [#{vm_provision_request}]")
      next
    end
    
    request = vm_provision_request[:request]
    intuit_dc = vm_provision_request[:endpoint_intuit_dc]
    vm_provision_request[:endpoint_ip_addresses].each do |ip_address|
      ae_uri = "/Intuit/Integration/CloudForms/Actions/CreateVmProvisionRequest?" \
               "intuit_dc=#{intuit_dc}&" \
               "ip_address=#{ip_address}&" \
               "template_fields=#{escape_uri(request[:template_fields])}&" \
               "vm_fields=#{escape_uri(request[:vm_fields])}&" \
               "requester=#{escape_uri(request[:requester])}&" \
               "tags=#{escape_uri(request[:tags])}&" \
               "values=#{escape_uri(request[:values])}&" \
               "ems_custom_attributes=#{escape_uri(request[:ems_custom_attributes])}&" \
               "miq_custom_attributes=#{escape_uri(request[:miq_custom_attributes])}"

      $evm.log(:info, "Intantiating Web Service Call: [#{ae_uri}]")
      result = $evm.instantiate(ae_uri)
      vm_provision_request[:result] = result ? result['result'] : 'error'
      vm_provision_request[:reason] = result ? result['reason'] : 'Unknown Error'
      vm_provision_request[:request_id] = result['request_id'] if result
      $evm.log(:info, "Web Service Results: [#{vm_provision_request[:result]}][#{vm_provision_request[:reason]}]")
      break if ['ok', 'error'].include?(vm_provision_request[:result])
    end
  end
  
  $evm.log(:info, "VM Provision Requests: [#{vm_provision_requests}]")
  
  if task.nil?
    $evm.root[:vm_provision_requests] = vm_provision_requests
  else
    task.set_option(:vm_provision_requests, vm_provision_requests)
  end

  retry_requests = vm_provision_requests.select { |request| request[:result] == 'retry' }
  error_requests = vm_provision_requests.select { |request| request[:result] == 'error' }
  if retry_requests.blank?
    $evm.root['ae_reason'] = ''
    $evm.root['ae_result'] = 'ok'
  else

    #####
    # THE COMMENTED CODE IS USELESS AND HAS BEEN COMMENTED OUT.
    #####
#    data_centers = retry_requests.collect do |request|
#      if request[:tags] == nil? || request[:tags] == blank?
#        intuit_hs = request[:values][:tag_intuit_hs]
#        tag = $evm.vmdb('classification').find_by_name("intuit_hs/#{intuit_hs}")
#        tag.description || intuit_hs
#      else  
#        intuit_hs = request[:tags][:intuit_hs]
#        tag = $evm.vmdb('classification').find_by_name("intuit_hs/#{intuit_hs}")
#        tag.description || intuit_hs
#      end
#    end
    
    error_msg = "#{error_requests.count} failed VM provision request#{error_requests.count > 1 ? 's' : ''}" unless error_requests.blank?
    retry_msg = "#{retry_requests.count} pending VM provision request#{retry_requests.count > 1 ? 's' : ''}"
    $evm.root['ae_reason'] = [error_msg, retry_msg].compact.join(', ')
    $evm.root['ae_result'] = 'retry'
    $evm.root['ae_retry_interval'] = '1.minute'
    $evm.log(:warn, $evm.root['ae_reason'])
    exit MIQ_OK
  end
  
  $evm.log(:info, 'CFME Automate Method Ended')
  exit MIQ_OK
  
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
