###################################
#
# EVM Automate Method: PostProvision
#
# Notes: This method is used to process tasks immediately after the VM has been provisioned
#
###################################
begin
  # Method for logging
  def log(level, message)
    @method = 'PostProvision'
    $evm.log(level, "#{@method} - #{message}")
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root

  # Get Variables
  prov = $evm.root["miq_provision"]
  log(:info, "Provisioning ID:<#{prov.id}> Provision Request ID:<#{prov.miq_provision_request.id}>")

  # Get provissioned VM from prov object
  vm = prov.vm
  unless vm.nil?
    log(:info, "VM:<#{vm.name}> has been provisioned")
  end

  #
  # Exit method
  #
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
