###################################
#
# CFME Automate Method: Set_VMName
#
# Notes: This is the default vmnaming method
# 1. If VM Name was not chosen during dialog processing then use vm_prefix
#    from dialog else use model and [:environment] tag to generate name
# 2. Else use VM name chosen in dialog
# 3. Then add 3 digit suffix to vm_name
# 4. Added support for dynamic service naming
#
###################################
begin
  # Method for logging
  def log(level, message)
    @method = 'Set_VMName'
    $evm.log(level, "#{@method} - #{message}")
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root

  prov = $evm.root['miq_provision_request'] || $evm.root['miq_provision'] || $evm.root['miq_provision_request_template']
  #log(:info, "Inspecting prov:<#{prov.inspect}>")

  vm_name = prov.get_option(:vm_name).to_s.strip
  number_of_vms_being_provisioned = prov.get_option(:number_of_vms)
  diamethod = prov.get_option(:vm_prefix).to_s.strip

  # If no VM name was chosen during dialog
  if vm_name.blank? || vm_name == 'changeme'
    vm_prefix = nil
    vm_prefix ||= $evm.object['vm_prefix']
    log(:info, "vm_name from dialog:<#{vm_name.inspect}> vm_prefix from dialog:<#{diamethod.inspect}> vm_prefix from model:<#{vm_prefix.inspect}>")

    # Get Provisioning Tags for VM Name
    tags = prov.get_tags
    log(:info, "Provisioning Object Tags: #{tags.inspect}")
    
    # Set a Prefix for VM Naming
    if diamethod.blank?
      vm_name = vm_prefix
    else
      vm_name = diamethod
    end
    log(:info,"VM Naming Prefix: <#{vm_name}>")

    # Check :environment tag
    env = tags[:environment]

    # If environment tag is not nil
    unless env.nil?
      log(:info,"Environment Tag: <#{env}> detected")
      # Get the first 3 characters of the :environment tag
      env_first = env[0,3]

      vm_name =  "#{vm_name}#{env_first}"
      log(:info,"Updating VM Name: <#{vm_name}>")
    end
    derived_name = "#{vm_name}$n{3}"
  else
    if number_of_vms_being_provisioned == 1
      derived_name = "#{vm_name}"
    else
      derived_name = "#{vm_name}$n{3}"
    end
  end

  $evm.object['vmname'] = derived_name
  log(:info, "VM Name: <#{derived_name}>")

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Set Ruby rescue behavior
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
