###################################
#
# CFME Automate Method: best_fit_with_scope
#
# Notes: This method is used to find all hosts, datastores that have the tag category
# prov_scope = 'all' &&|| prov_scope = <group-name>
#
###################################
begin
  # Method for logging
  def log(level, message)
    @method = 'best_fit_with_scope'
    $evm.log(level, "#{@method} - #{message}")
  end

  def error(msg)
    $evm.log(:error, "#{msg}")
    $evm.root['ae_result'] = 'error'
    $evm.root['ae_reason'] = msg
    $evm.root['message'] = msg
    exit MIQ_OK
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  # In the host, find all the VMs and sum up the allocated CPU counts.
  def countAllocatedCpuInHost(vms)
    cpu_count = 0
    vms.each do |vm|
      cpu_count += vm.num_cpu.to_i
    end
    return cpu_count
  end
  # In the host, find all the VMs and sum up the allocated memory.
  def sumAllocatedMemInHost(vms)
    mem_sum = 0
    vms.each do |vm|
      mem_sum += vm.mem_cpu.to_i
    end
    return mem_sum
  end
  def get_mem_oversubscription(dev)
    mem_oversubscription = dev.tags(category='intuit_mem_oversub').first
    mem_oversubscription.nil? ? nil : mem_oversubscription.to_i
  end
  def get_cpu_oversubscription(dev)
    cpu_oversubscription = dev.tags(category='intuit_cpu_oversub').first
    cpu_oversubscription.nil? ? nil : cpu_oversubscription.to_i
  end
  def get_clusters(options_hash, tags_hash, template)
    security_zone = tags_hash[:intuit_sz]
    security_level = tags_hash[:intuit_security_level]
    fault_domain = tags_hash[:intuit_fd]
    requested_cpus = options_hash[:number_of_sockets].first.to_i*options_hash[:cores_per_socket].first.to_i
    requested_mem = options_hash[:vm_memory].first.to_i
    ems = template.ext_management_system
    host_name = options_hash[:placement_host_name].first
    log(:info, "placement_host_name: #{host_name}")
    clusters = ems.ems_clusters.select do |cluster|
      $cluster_error = 0
      total_cpu_count = 0
      total_mem_count = 0
      requested_cpu_total = 1
      requested_mem_total = 1
      cluster_aggregate_mem = 1
      cluster_aggregate_cpu = 1
      #oversub_cpu_ratio = cluster.tagged_with?('intuit_cpu_oversub', (1..10).to_a) ? cluster.tags(category='intuit_cpu_oversub').first.to_i : 1
      #oversub_mem_ratio = cluster.tagged_with?('intuit_mem_oversub', (1..10).to_a) ? cluster.tags(category='intuit_mem_oversub').first.to_i : 1
      oversub_cpu_ratio = cluster.tags(category='intuit_cpu_oversub').first
      oversub_mem_ratio = cluster.tags(category='intuit_mem_oversub').first
      log(:info, "CPU before nil check: #{oversub_cpu_ratio}")
      log(:info, "MEM before nil check: #{oversub_mem_ratio}")
      oversub_cpu_ratio = 1 if oversub_cpu_ratio.nil?
      oversub_mem_ratio = 1 if oversub_mem_ratio.nil?
      if cluster.tagged_with?('intuit_ratio_scope', 'cluster')
        cluster.hosts.each {|host|
          vms = host.vms
          total_cpu_count += countAllocatedCpuInHost(vms)
          total_mem_count += sumAllocatedMemInHost(vms)
        }
        cluster_aggregate_cpu = cluster.aggregate_logical_cpus.to_i
        cluster_aggregate_mem = cluster.aggregate_memory.to_i
        requested_cpu_total = total_cpu_count + requested_cpus
        requested_mem_total = total_mem_count + requested_mem
      end
      log(:info, "Cluster name: #{cluster.name}")
      log(:info, "CPU Ratio Limit: #{oversub_cpu_ratio}")
      log(:info, "MEM Ratio Limit: #{oversub_mem_ratio}")
      log(:info, "CPU Total After Request: #{requested_cpu_total}")
      log(:info, "MEM Total After Request: #{requested_mem_total}")
      log(:info, "CPU Ratio After Request: #{(requested_cpu_total.fdiv(cluster_aggregate_cpu))}")
      log(:info, "MEM Ratio After Request: #{(requested_mem_total.fdiv(cluster_aggregate_mem))}")
      log(:info, "Math check: #{((requested_cpu_total.fdiv(cluster_aggregate_cpu)) <= oversub_cpu_ratio.to_i)}")

=begin
      result = cluster.tagged_with?('prov_scope', 'all') &&
        cluster.tagged_with?('intuit_sz', security_zone) &&
        cluster.tagged_with?('intuit_security_level', security_level) &&
        cluster.tagged_with?('intuit_fd', fault_domain) &&
        ((requested_cpu_total.fdiv(cluster_aggregate_cpu)) <= oversub_cpu_ratio.to_i) &&
        ((requested_mem_total.fdiv(cluster_aggregate_mem)) <= oversub_mem_ratio.to_i)
=end



      unless cluster.tagged_with?('prov_scope', 'all') && cluster.tagged_with?('intuit_sz', security_zone) && cluster.tagged_with?('intuit_security_level', security_level) && cluster.tagged_with?('intuit_fd', fault_domain)
        sz_check = cluster.tagged_with?('intuit_sz', security_zone)
        sl_check = cluster.tagged_with?('intuit_security_level', security_level)
        fd_check = cluster.tagged_with?('intuit_fd', fault_domain)
        log(:info, "Cluster tags: #{cluster.tags.inspect}")
        log(:info, "Prov scope? #{cluster.tagged_with?('prov_scope', 'all')}")
        log(:info, "Sec Zone #{security_zone}? #{sz_check}")
        log(:info, "Sec Level #{security_level}? #{sl_check}")
        log(:info, "Fault domain #{fault_domain}? #{fd_check}")
        $cluster_error = 1
        next
      end

      unless ((requested_cpu_total.fdiv(cluster_aggregate_cpu)) <= oversub_cpu_ratio.to_i)
        $cluster_error = 2
        next
      end
      unless ((requested_mem_total.fdiv(cluster_aggregate_mem)) <= oversub_mem_ratio.to_i)
        if $cluster_error == 2
          $cluster_error = 4
        else
          $cluster_error = 3
        end
        next
      end
      if $cluster_error == 0
        result = cluster
      end
      log(:info, "Cluster Error: #{$cluster_error} RESULT: #{result}")
      if result and not host_name.blank? and not cluster.hosts.detect { |h| h.name == host_name }
        log(:info, "Placement host #{host_name} not found, skipping cluster #{cluster.name}")
        result = false
      end
      result
    end.sort_by { |cluster| cluster.vms.count }
    log(:info, "Found #{clusters.count} Clusters")
    return clusters
    #  error("No target cluster available") if clusters.empty?
    #  cluster = clusters.first
    #  error("Unable to find suitable cluster") if cluster.nil?
    #  log(:info, "Using Cluster: #{cluster.name}")
    #  cluster.id
  end
  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root


  $cluster_error = nil
  # Get variables
  prov = $evm.root["miq_provision"]
  vm = prov.vm_template
  raise "VM not specified" if vm.nil?
  user = prov.miq_request.requester
  raise "User not specified" if user.nil?
  ems  = vm.ext_management_system
  raise "Provider not found for VM:<#{vm.name}>" if ems.nil?

  # Log all provisioning options and space required
  #log(:info, "Inspecting provisioning object: #{prov.options.inspect}")
  #log(:info, "VM=<#{vm.name}>, Space Required=<#{vm.provisioned_storage}>")

  log(:info, "Prov tags: #{prov.get_tags}")
  prov_tags = prov.get_tags
  prov_options = prov.options
  hosting_segment = prov_tags[:intuit_hs]
  security_zone = prov_tags[:intuit_sz]
  fault_domain = prov_tags[:intuit_fd]

  ws_values = prov.get_option(:ws_values) || {}

  #cluster_id = ws_values[:placement_cluster_id]
  host_name = ws_values[:placement_host_name]

  #cluster = $evm.vmdb('ems_cluster').find_by_id(cluster_id)
  clusters = get_clusters(prov_options, prov_tags, vm)
  cluster_message = nil
  if clusters.count < 1
    case $cluster_error
      when 1
        cluster_message = "No clusters found with the appropriate tags"
      when 2
        cluster_message = "Insufficient resources available on cluster: Not enough CPUs available to complete the provision request."
      when 3
        cluster_message = "Insufficient resources available on cluster: Not enough Memory available to complete the provision request."
      when 4
        cluster_message = "Insufficient resources available on cluster: Not enough CPUs and Memory available to complete the provision request."
    end
    prov.miq_request.set_option(:best_fit_error, "No clusters found: #{cluster_message}")
    error("No Clusters found: #{cluster_message}")
  end
  #$evm.vmdb('ems_cluster').find_by_name('TAX_ECO_FD1')

  #log(:info, "Using Cluster: #{cluster.name} from :cluster_id")

  #############################
  # Get Tags that are in scope
  # Default is to look for Hosts and Datastores tagged with prov_scope = All or match to Group
  #############################
  tags  = {}
  tags["prov_scope"] = ["all",user.normalized_ldap_group]

  log(:info, "VM=<#{vm.name}>, Space Required=<#{vm.provisioned_storage}>, group=<#{user.normalized_ldap_group}>")

  #############################
  # STORAGE LIMITATIONS
  #############################
  STORAGE_MAX_VMS      = 0
  storage_max_vms      = $evm.object['storage_max_vms']
  storage_max_vms      = storage_max_vms.strip.to_i if storage_max_vms.kind_of?(String) && !storage_max_vms.strip.empty?
  storage_max_vms      = STORAGE_MAX_VMS unless storage_max_vms.kind_of?(Numeric)
  STORAGE_MAX_PCT_USED = 100
  storage_max_pct_used = $evm.object['storage_max_pct_used']
  storage_max_pct_used = storage_max_pct_used.strip.to_i if storage_max_pct_used.kind_of?(String) && !storage_max_pct_used.strip.empty?
  storage_max_pct_used = STORAGE_MAX_PCT_USED unless storage_max_pct_used.kind_of?(Numeric)
  log(:info, "storage_max_vms:<#{storage_max_vms}> storage_max_pct_used:<#{storage_max_pct_used}>")

  #############################
  # Set host sort order here
  # options: :active_provioning_memory, :active_provioning_cpu, :current_memory_usage,
  #          :current_memory_headroom, :current_cpu_usage, :random
  #############################
  HOST_SORT_ORDER = [:active_provioning_memory, :current_memory_headroom, :random]
  STORAGE_SORT_ORDER = [:active_provisioning_vms, :random]
  #############################
  # Sort hosts
  #############################
  active_prov_data = prov.check_quota(:active_provisions)
  sort_data = []
  prov_cluster = prov_host = prov_storage = nil
  error_flag = nil


  clusters.each {|cluster|
    host = hosts = storage = nil
    sort_data = []
    min_registered_vms = nil

    log(:info, "Cluster Tags: #{cluster.tags}")

    cluster.tags.each {|c_tags|
      if c_tags.include? "intuit_storage_percent"
        storage_max_pct_used = cluster.tags(category='intuit_storage_percent').first.to_i
      end
    }

    log(:info, "Storage Percentage that can be allocated: #{storage_max_pct_used}")

    cluster.hosts.each do |h|
      unless host_name.blank? || h.name == host_name
        $evm.log(:info, "Skipping host due placement selection: [#{h.name}] != [#{host_name}]")
        next
      end

      sort_data << sd = [[], h.name, h]
      host_id = h.attributes['id'].to_i
      HOST_SORT_ORDER.each do |type|
        sd[0] << case type
                   # Multiply values by (-1) to cause larger values to sort first
                   when :active_provioning_memory
                     active_prov_data[:active][:memory_by_host_id][host_id]
                   when :active_provioning_cpu
                     active_prov_data[:active][:cpu_by_host_id][host_id]
                   when :current_memory_headroom
                     h.current_memory_headroom * -1
                   when :current_memory_usage
                     h.current_memory_usage
                   when :current_cpu_usage
                     h.current_cpu_usage
                   when :random
                     rand(1000)
                   else 0
                 end
        ##### End sort options
      end
    end

    sort_data.sort! {|a,b| a[0] <=> b[0]}
    hosts = sort_data.collect {|sd| sd.pop}
    log(:info, "Sorted host Order:<#{HOST_SORT_ORDER.inspect}> Results:<#{sort_data.inspect}>")

    #############################
    # Set storage sort order here
    # options: :active_provisioning_vms, :free_space, :free_space_percentage, :random
    #############################


    hosts.each do |h|
      log(:info, "Host being evaluated: #{h.name}")
      next unless h.power_state == "on"
      #############################
      # Only consider hosts that meet mem and cpu ratio
      #############################
      ratio_scope = nil
      mem_oversubscription = nil
      cpu_oversubscription = nil
      if !h.tags(category='intuit_ratio_scope').first.nil?
        ratio_scope = h.tags(category='intuit_ratio_scope').first
        if ratio_scope != 'cluster'
          mem_oversubscription = get_mem_oversubscription(h)
          cpu_oversubscription = get_cpu_oversubscription(h)
        end
      else
        if !h.ems_cluster.tags(category='intuit_ratio_scope').first.nil?
          ratio_scope = h.ems_cluster.tags(category='intuit_ratio_scope').first
          if ratio_scope != 'cluster'
            mem_oversubscription = get_mem_oversubscription(h.ems_cluster)
            cpu_oversubscription = get_cpu_oversubscription(h.ems_cluster)
          end
        else
          ratio_scope = 'host'
          mem_oversubscription = get_mem_oversubscription(h)
          cpu_oversubscription = get_cpu_oversubscription(h)
        end
      end
      log(:info, "Oversubscription Tags: <Scope: #{ratio_scope}, Mem: #{mem_oversubscription}, CPU: #{cpu_oversubscription}>")
      if ratio_scope != 'cluster'
        vms = h.vms
        socket_count = h.hardware.numvcpus
        cores_per_socket = h.hardware.cores_per_socket
        total_cores = socket_count * cores_per_socket
        log(:info, "Host Hardware Total CPU: #{total_cores}")
        total_mem = h.hardware.memory_cpu
        log(:info, "Host Hardware Total Memory (MB): #{total_mem}")
        allocated_cpu = countAllocatedCpuInHost(vms)
        log(:info, "Host Hardware Allocated CPU: #{allocated_cpu}")
        allocated_mem = sumAllocatedMemInHost(vms)
        log(:info, "Host Hardware Allocated Mem (MB): #{allocated_mem}")
        provision_req_sockets = prov.options[:number_of_sockets].first
        provision_req_cores = prov.options[:cores_per_socket].first
        provision_req_cpu = provision_req_sockets * provision_req_cores
        log(:info, "Provision Requested CPU: #{provision_req_sockets} sockets * #{provision_req_cores} cores")
        provision_req_mem = prov.options[:vm_memory].first.to_i
        log(:info, "Provision Requested Memory (MB): #{provision_req_mem}")
        if mem_oversubscription.nil?
          log(:info, "Skipping host #{h.name}, Memory: Req'd #{provision_req_mem + allocated_mem} MB > Available #{total_mem} MB") if provision_req_mem + allocated_mem > total_mem
          error_flag = 1
          next unless provision_req_mem + allocated_mem <= total_mem
        else
          log(:info, "Skipping host #{h.name}, Memory: Req'd #{provision_req_mem + allocated_mem} MB > Available #{total_mem * mem_oversubscription} MB") if provision_req_mem + allocated_mem > total_mem * mem_oversubscription
          error_flag = 1
          next unless provision_req_mem + allocated_mem <= total_mem * mem_oversubscription
        end
        if cpu_oversubscription.nil?
          log(:info, "Skipping host #{h.name}, CPU: Req'd #{provision_req_cpu + allocated_cpu} CPUs > Available #{total_cores} CPUs") if provision_req_cpu + allocated_cpu > total_cores
          error_flag = 2
          next unless provision_req_cpu + allocated_cpu <= total_cores
        else
          log(:info, "Skipping host #{h.name}, CPU: Req'd #{provision_req_cpu + allocated_cpu} CPUs > Available #{total_cores * cpu_oversubscription} CPUs") if provision_req_cpu + allocated_cpu > total_cores * cpu_oversubscription
          error_flag = 2
          next unless provision_req_cpu + allocated_cpu <= total_cores * cpu_oversubscription
        end
      end

      #############################
      # Only consider hosts that have the required tags
      #############################
      next unless tags.all? do |key, value|
        if value.kind_of?(Array)
          value.any? { |v| h.tagged_with?(key, v) }
        else
          h.tagged_with?(key, value)
        end
      end

      nvms = h.vms.length

      #############################
      # Only consider storages that have the tag category group=all
      #############################
      storages = h.storages.find_all do |s|
        tags.all? do |key, value|
          if value.kind_of?(Array)
            value.any? { |v| s.tagged_with?(key, v) }
          else
            s.tagged_with?(key, value)
          end
        end
      end

      log(:info, "Evaluating storages:<#{storages.collect {|s| s.name}.join(", ")}>")

      #############################
      # Filter out storages that do not have enough free space for the VM
      #############################
      active_prov_data = prov.check_quota(:active_provisions)
      storages = storages.find_all { |s|
        storage_id = s.attributes['id'].to_i
        actively_provisioned_space = active_prov_data[:active][:storage_by_id][storage_id]
        if s.free_space > vm.provisioned_storage + actively_provisioned_space
          #        $evm.log("info", "Active Provision Data inspect: [#{active_prov_data.inspect}]")
          #        $evm.log("info", "Active provision space requirement: [#{actively_provisioned_space}]")
          #        $evm.log("info", "Valid Datastore: [#{s.name}], enough free space for VM -- Available: [#{s.free_space}], Needs: [#{vm.provisioned_storage}]")
          true
        else
          log(:info, "Skipping Datastore:<#{s.name}>, not enough free space for VM:<#{vm.name}>. Available:<#{s.free_space}>, Needs:<#{vm.provisioned_storage}>")
          error_flag = 3
          false
        end
      }

      #############################
      # Filter out storages number of VMs is greater than the max number of VMs allowed per Datastore
      #############################
      storages = storages.find_all { |s|
        storage_id = s.attributes['id'].to_i
        active_num_vms_for_storage = active_prov_data[:active][:vms_by_storage_id][storage_id].length
        if (storage_max_vms == 0) || ((s.vms.size + active_num_vms_for_storage) < storage_max_vms)
          true
        else
          log(:info, "Skipping Datastore:<#{s.name}>, max number of VMs:<#{s.vms.size + active_num_vms_for_storage}> exceeded")
          error_flag = 4
          false
        end
      }

      #############################
      # Filter out storages where percent used will be greater than the max % allowed per Datastore
      #############################
      storages = storages.find_all { |s|
        storage_id = s.attributes['id'].to_i
        active_pct_of_storage  = ((active_prov_data[:active][:storage_by_id][storage_id]) / s.total_space.to_f) * 100
        request_pct_of_storage = (vm.provisioned_storage / s.total_space.to_f) * 100

        #      $evm.log("info", "Active Provision Data inspect: [#{s.name}]:[#{storage_id}] -- [#{active_prov_data.inspect}]")
        #      $evm.log("info", "Datastore Percent: [#{s.name}]:[#{storage_id}] -- Storage:[#{s.v_used_space_percent_of_total}]  Active:[#{active_pct_of_storage}]  Request:[#{request_pct_of_storage}]")

        if (storage_max_pct_used == 100) || ((s.v_used_space_percent_of_total + active_pct_of_storage) < storage_max_pct_used)
          #        $evm.log("info", "Current PCT of active provision: [#{active_pct_of_storage}]")
          #        $evm.log("info", "Valid Datastore: [#{s.name}], enough free space for VM -- Total Datastore Size: [#{s.total_space}], Available: [#{s.free_space}], Needs: [#{vm.provisioned_storage}]")
          true
        else
          log(:info, "Skipping Datastore:<#{s.name}> percent of used space #{s.v_used_space_percent_of_total + active_pct_of_storage} exceeded")
          error_flag = 3
          #        $evm.log("info", "Total Datastore Size: [#{s.total_space}], Total Percentage Required: ([#{s.v_used_space_percent_of_total}] + [#{active_pct_of_storage}])")
          false
        end
      }


      if min_registered_vms.nil? || nvms < min_registered_vms
        #############################
        # Sort storage to determine target datastore
        #############################
        sort_data = []
        storages.each_with_index do |s, idx|
          sort_data << sd = [[], s.name, idx]
          storage_id = s.attributes['id'].to_i
          STORAGE_SORT_ORDER.each do |type|
            sd[0] << case type
                       when :free_space
                         # Multiply values by (-1) to cause larger values to sort first
                         (s.free_space - active_prov_data[:active][:storage_by_id][storage_id]) * -1
                       when :free_space_percentage
                         active_pct_of_storage  = ((active_prov_data[:active][:storage_by_id][storage_id]) / s.total_space.to_f) * 100
                         s.v_used_space_percent_of_total + active_pct_of_storage
                       when :active_provioning_vms
                         active_prov_data[:active][:vms_by_storage_id][storage_id].length
                       when :random
                         rand(1000)
                       else 0
                     end
            ##### End sort options
          end
        end

        sort_data.sort! {|a,b| a[0] <=> b[0]}
        log(:info, "Sorted storage Order:<#{STORAGE_SORT_ORDER.inspect}>  Results:<#{sort_data.inspect}>")
        selected_storage = sort_data.first
        unless selected_storage.nil?
          selected_idx = selected_storage.last
          storage = storages[selected_idx]
          host    = h
        end

        #log(:info, "Found Host:<#{h.name}> with Tags:<#{h.tags.inspect}>")

        # Stop checking if we have found both host and storage
        break if host && storage
      end

    end # END - hosts.each
    log(:info, "BEFORE BREAK :: cluster=<#{cluster.name}> host=<#{host}> storage=<#{storage}>")
    prov_host = host
    prov_storage = storage
    prov_cluster = cluster
    if host && storage && cluster
      error_flag = 0
      break
    end
  } # END - clusters.each
  error("Invalid target host #{host_name}") unless host_name.blank? or not prov_host.nil?

  obj = $evm.object
  case error_flag
    when 0
      log(:info, "Selected Host:<#{prov_host.nil? ? "nil" : prov_host.name}>")
      obj["host"] = prov_host unless prov_host.nil?

      log(:info, "Selected Datastore:<#{prov_storage.nil? ? "nil" : prov_storage.name}>")
      obj["storage"] = prov_storage unless prov_storage.nil?

      ws_values[:placement_cluster_id] = prov_cluster.id
      prov.set_option(:ws_values, ws_values)
      # Set host and storage
      log(:info, "vm=<#{vm.name}> cluster=<#{prov_cluster.name}> host=<#{prov_host}> storage=<#{prov_storage}>")
    when 1
      prov.miq_request.set_option(:best_fit_error, "Insufficient Hosts Memory available to complete the provision request.")
      error("Insufficient Hosts Memory available to complete the provision request.")
    when 2
      prov.miq_request.set_option(:best_fit_error, "Insufficient Host CPUs available to complete the provision request.")
      error("Insufficient Host CPUs available to complete the provision request.")
    when 3
      prov.miq_request.set_option(:best_fit_error, "Insufficient space available on the Datastores to complete the provision request.")
      error("Insufficient space available on the Datastores to complete the provision request.")
    when 4
      prov.miq_request.set_option(:best_fit_error, "Max number of VMs has been reached on all Datastores, unable to complete the provision request.")
      error("Max number of VMs has been reached on all Datastores, unable to complete the provision request.")
  end

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

    # Set Ruby rescue behavior
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_OK
end
