###################################
#
# CFME Automate Method: check_provisioned
#
# Notes: This method checks to see if the VM has been provisioned by the
#        provider.
#
###################################

# Method for logging
def log(level, message)
  @method = 'check_provisioned'
  $evm.log(level, "#{@method} - #{message}")
end

begin
  log(:info, "CFME Automate Method Started")

  # Get current provisioning status
  prov = $evm.root['miq_provision']
  prov_status = prov['status']
  result = prov.status
  best_fit_message = prov.get_option(:best_fit_error)
  log(:info, "PROV #{best_fit_message}")
  log(:info, "Provision:<#{prov.id}> Request:<#{prov.miq_provision_request.id}> Type:<#{prov.type}>")
  log(:info, "Result:<#{result}> for state:<#{prov.state}> status:<#{prov_status}>")

  case result
  when 'error'
    $evm.root['ae_result'] = 'error'
    reason = $evm.root['miq_provision'].message
    reason = reason[7..-1] if reason[0..6] == 'Error: '
    reason = best_fit_message if best_fit_message
    $evm.root['ae_reason'] = reason
    log(:info, "AE_REASON #{$evm.root['ae_reason']}")
  when 'retry'
    $evm.root['ae_result']         = 'retry'
    $evm.root['ae_retry_interval'] = '1.minute'
  when 'ok'
    # Bump State
    $evm.root['ae_result'] = 'ok'

    # Prepare rollback in case we run into an error. We only do this after
    # the provider finished the provisioning request. Otherwise, we might
    # accidentally delete the wrong VM if the provisioning failed because a
    # VM with the same name already exists in the provider.
    rollback_parameters = prov.get_option(:rollback_parameters)
    rollback_parameters['vm_name'] = prov.get_option(:vm_target_name)
    rollback_parameters['ems_id'] = prov.get_option(:src_ems_id)
    prov.set_option(:rollback_parameters, rollback_parameters)
  end

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Set Ruby rescue behavior
rescue StandardError, SignalException=> err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_OK
end
