###################################
#
# CFME Automate Method: cleanup_escalation_qb
#
# Author: Carsten Clasohm
#
# Notes: 
#   Delete a record from Escalation QB. This is called if there is an error
#   during the provisioning process, to automatically remove the
#   half-provisioned VM.
#
# Inputs: record_id
#
###################################

# logging convenience method
def log(level, msg)
  @method = 'cleanup_escalation_qb'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute #{uri}") unless result
  
  case result['result']
  when 'error'
    error(result['reason'])
  when 'retry'
    retry_method(result['reason'])
  end
  
  result
end

begin
  log(:info, "Main: entering main method")

  task = $evm.root['automation_task']

  intuit_dc = task.options[:attrs][:intuit_dc]
  record_id = task.options[:attrs][:eqb_record_id]

  unless record_id.nil?
    ae_uri = "/Intuit/Integration/EscalationQB/Actions/DeleteFromEscalationQB?" + \
    URI.encode_www_form([['intuit_dc', intuit_dc],
                         ['record_id', record_id]])
    result = instantiate(ae_uri)

    log(:error, "failed to delete record #{record_id} from Escalation QB") if result.nil?
  end

  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''

  log(:info, "Main: exiting main method")
  exit MIQ_OK

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
