###################################
#
# CFME Automate Method: cleanup_hpsa
#
# Author: Carsten Clasohm
#
# Notes: 
#   Delete a VM from HPSA. This is called if there is an error during the
#   provisioning process, to automatically remove the half-provisioned VM.
#
# Inputs: hpsa_vm_id
#
# Mods:
#
###################################

# logging convenience method
def log(level, msg)
  @method = 'cleanup_hpsa'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute #{uri}") unless result
  
  case result['result']
  when 'error'
    error(result['reason'])
  when 'retry'
    retry_method(result['reason'])
  end
  
  result
end

begin
  log(:info, "Main: entering main method")

  task = $evm.root['automation_task']

  intuit_dc = task.options[:attrs][:intuit_dc]
  vm_id = task.options[:attrs][:hpsa_vm_id]

  log(:info, "hpsa_vm_id = #{vm_id}")

  unless vm_id.nil?
    vm = $evm.vmdb('vm', vm_id)

    if vm
      log(:info, "Calling decomission for VM #{vm_id}")

      ae_uri = "/Intuit/Integration/HPSA/Actions/Decommission?" + \
               URI.encode_www_form([['intuit_dc', intuit_dc],
                                    ['vmdb_object_type', 'vm'],
                                    ['Vm::vm', vm.id]])
      result = instantiate(ae_uri)

      log(:error, "failed to decomission from HPSA") if result.nil?
    else
      log(:warn, "Cannot find VM #{vm_id} during rollback")
    end
  end

  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''

  log(:info, "Main: exiting main method")
  exit MIQ_OK

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
