###################################
#
# CFME Automate Method: cleanup_vmdb
#
# Author: Carsten Clasohm
#
# Notes: 
#   Delete a VM from the VMDB. This is called if there is an error during
#   the provisioning process, to automatically remove the half-provisioned
#   VM.
#
# Inputs: vm_id
#
# Mods:
#
###################################

# logging convenience method
def log(level, msg)
  @method = 'cleanup_vmdb'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

begin
  log(:info, "Main: entering main method")

  task = $evm.root['automation_task']

  vm_id = task.options[:attrs][:vm_id]
  log(:info, "vm_id = #{vm_id}")

  unless vm_id.nil?
    vm = $evm.vmdb('vm', vm_id)

    if vm
      # Make sure that this VM was provisioned by us. This is a safeguard
      # for provisioning failures caused by naming collisions inside the
      # provider.
      if /\w*MIQ\sGUID/i =~ vm.v_annotation
        log(:info, "Deleting VM #{vm.name} from VMDB")
        vm.remove_from_vmdb
      else
        log(:warn, "VM #{vm.name} (#{vm.id}) missing MIQ GUID, not deleting during rollback")
      end
    else
      log(:warn, "Cannot find VM #{vm_id} during rollback")
    end
  end

  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''

  log(:info, "Main: exiting main method")
  exit MIQ_OK

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
