###################################
#
# CFME Automate Method: customize_request
#
# Notes: This method is used to Customize the Provisioning Request
#
# Customization Specification mapping for VMware provisioning
# Customization Template mapping for VMware PXE provisioning
# Customization Template mapping for RHEV clone provisioning
# Customization Template mapping for RHEV PXE provisioning
# Customization Template mapping for RHEV ISO provisioning
# Customization Template mapping for OpenStack provisioning
# Customization Template mapping for Amazon provisioning
#
###################################

  # Method for logging
  def log(level, message)
    @method = 'customize_request'
    $evm.log(level, "#{@method} - #{message}")
  end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
  end

  # set_customspec
  def set_customspec(prov, spec)
  prov.set_customization_spec(spec, true)
    log(:info, "Provisioning object updated - <:sysprep_custom_spec> with <#{prov.get_option(:sysprep_custom_spec).inspect}>")
    log(:info, "Provisioning object <:sysprep_spec_override> updated with <#{prov.get_option(:sysprep_spec_override)}>")
  end

  # process_vmware
  def process_vmware(mapping, prov )
    # Get information from the template platform
    template = prov.vm_template
    product  = template.operating_system['product_name'].downcase
    bitness = template.operating_system['bitness']
    log(:info, "Template:<#{template.name}> Vendor:<#{template.vendor}> Product:<#{product}> Bitness:<#{bitness}> mapping:<#{mapping}>")

    # Skip automatic customization spec mapping if one was chosen already or template product is 'other' or provision_type is clone_to_[template]
    #if prov.get_option(:sysprep_custom_spec) || product.include?("other") || prov.provision_type.include?("clone_to_template")
    if product.include?("other") || prov.provision_type.include?("clone_to_template")
      mapping = 0
    end

    case mapping

    when 0
      # Skip mapping
      log(:info, "Skipping #{prov.type} mapping:<#{mapping}>")

    when 1
      # Automatic customization specification mapping if template is RHEL,Suse or Windows
      if product.include?("red hat") || product.include?("suse") || product.include?("windows")
        spec = prov.vm_template.name # to match the template name
        set_customspec(prov, spec)

        # Set linux hostname stuff here
        prov.set_option(:linux_host_name, prov.get_option(:vm_target_name))
        log(:info, "Provisioning object updated - <:linux_host_name> with <#{prov.get_option(:linux_host_name)}>")
        prov.set_option(:vm_target_hostname, prov.get_option(:vm_target_name))
        log(:info, "Provisioning object updated - <:hostname> with <#{prov.get_option(:vm_target_hostname)}>")
      end

    when 2
      # Use this option to use a combination of product name and bitness to select your customization specification
      spec = nil # unknown type

      if product.include?("windows")
        spec = "cloudforms_windows" # Windows
      else 
        spec = "cloudforms_linux" # RHEL
      end
      log(:info, "VMware Custom Specification:<#{spec}> bitness:<#{bitness}>")

      # Set values in provisioning object
      set_customspec(prov, spec) unless spec.nil?
    when 3
      #
      # Enter your own VMware custom mapping here
      #
    else
      # Skip mapping
      log(:info, "Skipping #{prov.type} mapping:<#{mapping}>")
    end # end case
  end # end process_vmware


  #################################
  #
  # Method: process_vmware_pxe
  #
  #################################
  def process_vmware_pxe(mapping, prov )
    # Get information from the template platform
    template = prov.vm_template
    product  = template.operating_system['product_name'].downcase
    log(:info, "Template:<#{template.name}> Vendor:<#{template.vendor}> Product:<#{product}>")

    case mapping

    when 0
      # No mapping

    when 1
      if product.include?("windows")
        # find the windows image that matches the template name if a PXE Image was NOT chosen in the dialog
        if prov.get_option(:pxe_image_id).nil?

          pxe_image = prov.eligible_windows_images.detect { |pi| pi.name.casecmp(template.name)==0 }
          if pxe_image.nil?
            message "Failed to find matching PXE Image"
            prov.message = message
            log(:info, "Inspecting Eligible Windows Images:<#{prov.eligible_windows_images.inspect}>")
            raise message
          else
            log(:info, "Found matching Windows PXE Image ID:<#{pxe_image.id}> Name:<#{pxe_image.name}> Description:<#{pxe_image.description}>")
          end
          prov.set_windows_image(pxe_image)
          log(:info, "Provisioning object <:pxe_image_id> updated with <#{prov.get_option(:pxe_image_id).inspect}>")
        end
        # Find the first customization template that matches the template name if none was chosen in the dialog
        if prov.get_option(:customization_template_id).nil?
          cust_temp = prov.eligible_customization_templates.detect { |ct| ct.name.casecmp(template.name)==0 }
          if cust_temp.nil?
            message "Failed to find matching PXE Image"
            prov.message = message
            log(:info, "Inspecting Eligible Customization Templates:<#{prov.eligible_customization_templates.inspect}>")
            raise message
          end
          log(:info, "Found mathcing Windows Customization Template ID:<#{cust_temp.id}> Name:<#{cust_temp.name}> Description:<#{cust_temp.description}>")
          prov.set_customization_template(cust_temp)
          log(:info, "Provisioning object <:customization_template_id> updated with <#{prov.get_option(:customization_template_id).inspect}>")
        end
      else
        # find the first PXE Image that matches the template name if NOT chosen in the dialog
        if prov.get_option(:pxe_image_id).nil?
          pxe_image = prov.eligible_pxe_images.detect { |pi| pi.name.casecmp(template.name)==0 }
          log(:info, "Found Linux PXE Image ID:<#{pxe_image.id}> Name:<#{pxe_image.name}> Description:<#{pxe_image.description}>")
          prov.set_pxe_image(pxe_image)
          log(:info, "Provisioning object <:pxe_image_id> updated with <#{prov.get_option(:pxe_image_id).inspect}>")
        end
        # Find the first Customization Template that matches the template name if NOT chosen in the dialog
        if prov.get_option(:customization_template_id).nil?
          cust_temp = prov.eligible_customization_templates.detect { |ct| ct.name.casecmp(template.name)==0 }
          log(:info, "Found Customization Template ID:<#{cust_temp.id}> Name:<#{cust_temp.name}> Description:<#{cust_temp.description}>")
          prov.set_customization_template(cust_temp)
          log(:info, "Provisioning object <:customization_template_id> updated with <#{prov.get_option(:customization_template_id).inspect}>")
        end
      end
    when 3
      #
      # Enter your own VMware PXE custom mapping here
      #

    else
      # Skip mapping
      log(:info, "Skipping #{prov.type} mapping:<#{mapping}>")
    end # end case
  end # end process_vmware_pxe


  #################################
  #
  # Method: process_redhat_pxe
  #
  #################################
  def process_redhat_pxe(mapping, prov )
    # Get information from the template platform
    template = prov.vm_template
    product  = template.operating_system['product_name'].downcase
    log(:info, "Template:<#{template.name}> Vendor:<#{template.vendor}> Product:<#{product}>")

    case mapping

    when 0
      # No mapping

    when 1
      if product.include?("windows")
        # find the windows image that matches the template name if a PXE Image was NOT chosen in the dialog
        if prov.get_option(:pxe_image_id).nil?

          pxe_image = prov.eligible_windows_images.detect { |pi| pi.name.casecmp(template.name)==0 }
          if pxe_image.nil?
            message "Failed to find matching PXE Image"
            prov.message = message
            log(:info, "Inspecting Eligible Windows Images:<#{prov.eligible_windows_images.inspect}>")
            raise message
          else
            log(:info, "Found matching Windows PXE Image ID:<#{pxe_image.id}> Name:<#{pxe_image.name}> Description:<#{pxe_image.description}>")
          end
          prov.set_windows_image(pxe_image)
          log(:info, "Provisioning object <:pxe_image_id> updated with <#{prov.get_option(:pxe_image_id).inspect}>")
        end
        # Find the first customization template that matches the template name if none was chosen in the dialog
        if prov.get_option(:customization_template_id).nil?
          cust_temp = prov.eligible_customization_templates.detect { |ct| ct.name.casecmp(template.name)==0 }
          if cust_temp.nil?
            message "Failed to find matching PXE Image"
            prov.message = message
            log(:info, "Inspecting Eligible Customization Templates:<#{prov.eligible_customization_templates.inspect}>")
            raise message
          end
          log(:info, "Found mathcing Windows Customization Template ID:<#{cust_temp.id}> Name:<#{cust_temp.name}> Description:<#{cust_temp.description}>")
          prov.set_customization_template(cust_temp)
          log(:info, "Provisioning object <:customization_template_id> updated with <#{prov.get_option(:customization_template_id).inspect}>")
        end
      else
        # find the first PXE Image that matches the template name if NOT chosen in the dialog
        if prov.get_option(:pxe_image_id).nil?
          pxe_image = prov.eligible_pxe_images.detect { |pi| pi.name.casecmp(template.name)==0 }
          log(:info, "Found Linux PXE Image ID:<#{pxe_image.id}> Name:<#{pxe_image.name}> Description:<#{pxe_image.description}>")
          prov.set_pxe_image(pxe_image)
          log(:info, "Provisioning object <:pxe_image_id> updated with <#{prov.get_option(:pxe_image_id).inspect}>")
        end
        # Find the first Customization Template that matches the template name if NOT chosen in the dialog
        if prov.get_option(:customization_template_id).nil?
          cust_temp = prov.eligible_customization_templates.detect { |ct| ct.name.casecmp(template.name)==0 }
          log(:info, "Found Customization Template ID:<#{cust_temp.id}> Name:<#{cust_temp.name}> Description:<#{cust_temp.description}>")
          prov.set_customization_template(cust_temp)
          log(:info, "Provisioning object <:customization_template_id> updated with <#{prov.get_option(:customization_template_id).inspect}>")
        end
      end
    when 3
      #
      # Enter your own RHEV custom mapping here
      #

    else
      # Skip mapping
      log(:info, "Skipping #{prov.type} mapping:<#{mapping}>")
    end # end case
  end # end process_redhat


  #################################
  #
  # Method: process_redhat_iso
  #
  #################################
  def process_redhat_iso(mapping, prov )
    # Get information from the template platform
    template = prov.vm_template
    product  = template.operating_system['product_name'].downcase
    log(:info, "Template:<#{template.name}> Vendor:<#{template.vendor}> Product:<#{product}>")

    case mapping

    when 0
      # No mapping
    when 1
      if product.include?("windows")
        # Linux Support only for now
      else
        # Linux - Find the first ISO Image that matches the template name if NOT chosen in the dialog
        if prov.get_option(:iso_image_id).nil?
          iso_image = prov.eligible_iso_images.detect { |iso| iso.name.casecmp(template.name)==0 }
          if iso_image.nil?
            message "Failed to find matching ISO Image:<#{iso_image.inspect}>"
            prov.message = message
            log(:error, message)
            raise message
          else
            log(:info, "Found Linux ISO Image ID:<#{iso_image.id}> Name:<#{iso_image.name}> Description:<#{iso_image.description}>")
            prov.set_iso_image(iso_image)
            log(:info, "Provisioning object <:iso_image_id> updated with <#{prov.get_option(:iso_image_id).inspect}>")
          end
        else
          log(:info, "ISO Image selected from dialog: <#{prov.get_option(:iso_image_id).inspect}>")
        end

        log(:info, "prov.eligible_customization_templates:<#{prov.eligible_customization_templates.inspect}>")

        # Find the first Customization Template that matches the template name if NOT chosen in the dialog
        if prov.get_option(:customization_template_id).nil?
          #log(:info, "prov.eligible_customization_templates:<#{prov.eligible_customization_templates.inspect}>")
          #cust_temp = prov.eligible_customization_templates.detect { |ct| ct.name.casecmp(template.name)==0 }
          cust_temp = $evm.vmdb('customization_template').all.detect { |ct| ct.name.casecmp(template.name)==0 }
          if cust_temp.nil?
            message "Failed to find matching Customization Template:<#{cust_temp.inspect}>"
            prov.message = message
            log(:error, message)
            raise message
          else
            log(:info, "Found Customization Template ID:<#{cust_temp.id}> Name:<#{cust_temp.name}> Description:<#{cust_temp.description}>")
            prov.set_customization_template(cust_temp)
            log(:info, "Provisioning object <:customization_template_id> updated with <#{prov.get_option(:customization_template_id).inspect}>")
          end
        else
          log(:info, "Customization Template selected from dialog: <#{prov.get_option(:customization_template_id).inspect}>")
        end
      end
    when 2
      #
      # Enter your own RHEV ISO custom mapping here
      #

    else
      # Skip mapping
      log(:info, "Skipping #{prov.type} mapping:<#{mapping}>")
    end
  end

  #################################
  #
  # Method: process_redhat
  #
  #################################
  def process_redhat(mapping, prov )
    # Get information from the template platform
    template = prov.vm_template
    product  = template.operating_system['product_name'].downcase
    log(:info, "Template:<#{template.name}> Vendor:<#{template.vendor}> Product:<#{product}>")

    case mapping

    when 0
      # No mapping
    when 1
      #
    when 2
      #
      # Enter your own RHEV custom mapping here
      #

    else
      # Skip mapping
      log(:info, "Skipping #{prov.type} mapping:<#{mapping}>")
    end
  end

  #################################
  #
  # Method: process_amazon
  #
  #################################
  def process_amazon(mapping, prov )
    # Check for ws_values in prov object
    if prov.options.has_key?(:ws_values)
      ws_values = prov.options[:ws_values]
      log(:info, "Found incoming ws_values:<#{ws_values}>")
    end

    template = prov.vm_template
    log(:info, "template: #{template.name}")

    case mapping
    when 0
      # No mapping
    when 1
      provider = template.ext_management_system
      raise "Provider not found for template:<#{template.name}>" if provider.nil?
      instance_types = provider.flavors
      instance_types.each do |i|
        if i.name ==  ws_values[:instance_type]
          prov.set_option(:instance_type, [i.id,"#{i.name}':'#{i.description}"])
          log(:info, "Setting Instance type to: #{i.name}")
        end
      end
      keypairs = provider.key_pairs
      keypairs.each do |k|
        if k.name ==  ws_values[:guest_access_key_pair]
          prov.set_option(:guest_access_key_pair, [k.id,k.name])
          log(:info, "Setting Access Key type to: #{k.name}")
        end
      end # end keypairs.each do
    when 2
      log(:info, "prov.eligible_customization_templates:<#{prov.eligible_customization_templates.inspect}>")
      customization_templates = $evm.vmdb('customization_template').all
      log(:info, "Customization Templates:<#{customization_templates.inspect}>")

      # Find the Customization Template that matches the template name if NOT chosen in the dialog
      if prov.get_option(:customization_template_id).nil?
        #log(:info, "prov.eligible_customization_templates:<#{prov.eligible_customization_templates.inspect}>")
        #cust_temp = prov.eligible_customization_templates.detect { |ct| ct.name.casecmp(template.name)==0 }
        cust_temp = $evm.vmdb('customization_template').all.detect { |ct| ct.name.casecmp(template.name)==0 }
        if cust_temp.nil?
          message "Failed to find matching Customization Template:<#{cust_temp.inspect}>"
          prov.message = message
          log(:warn, message)
          #raise message
        else
          log(:info, "Found Customization Template ID:<#{cust_temp.id}> Name:<#{cust_temp.name}> Description:<#{cust_temp.description}>")
          prov.set_customization_template(cust_temp)
          log(:info, "Provisioning object <:customization_template_id> updated with <#{prov.get_option(:customization_template_id).inspect}>")
        end
      else
        log(:info, "Customization Template selected from dialog: <#{prov.get_option(:customization_template_id).inspect}>")
      end
    end # case mapping
  end # end process_amazon

  #################################
  #
  # Method: process_openstack
  #
  #################################
  def process_openstack(mapping, prov )

  end # end process_amazon

begin
  log(:info, "CFME Automate Method Started")

  # Get provisioning object
  prov = $evm.root["miq_provision"]

  log(:info, "Provision:<#{prov.id}> Request:<#{prov.miq_provision_request.id}> Type:<#{prov.type}>")

  # Build case statement to determine which type of processing is required
  case prov.type

  when 'MiqProvisionRedhat'
    ##########################################################
    # Red Hat Mapping for template Provisioning
    #
    # Possible values:
    #   0 - (DEFAULT No Mapping) This option skips the mapping of customization templates/cloud-init
    #
    ##########################################################
    mapping = 0
    process_redhat(mapping, prov)

  when 'MiqProvisionRedhatViaIso'
    ##########################################################
    # Red Hat Customization Template Mapping for ISO Provisioning
    #
    # Possible values:
    #   0 - (DEFAULT No Mapping) This option skips the mapping of iso images and customization templates
    #
    #   1 - CFME will look for a iso image and a customization template with
    #   the exact name as the template name if none were chosen from the provisioning dialog
    #
    #   2 - Include your own custom mapping logic here
    ##########################################################
    mapping = 1
    process_redhat_iso(mapping, prov)

  when 'MiqProvisionRedhatViaPxe'
    ##########################################################
    # Red Hat Customization Template Mapping for PXE Provisioning
    #
    # Possible values:
    #   0 - (DEFAULT No Mapping) This option skips the mapping of pxe images and customization templates
    #
    #   1 - CFME will look for a pxe image and a customization template with
    #   the exact name as the template name if none were chosen from the provisioning dialog
    #
    #   2 - Include your own custom mapping logic here
    ##########################################################
    mapping = 1
    process_redhat_pxe(mapping, prov)

  when 'MiqProvisionVmware'
    ##########################################################
    # VMware Customization Specification Mapping
    #
    # Possible values:
    #   0 - (Default No Mapping) This option is automatically chosen if it finds a customization
    #   specification mapping chosen from the dialog
    #
    #   1 - EVM will look for a customization specification with
    #   the exact name as the template name
    #
    #   2 - Use this option to use a combination of product name and bitness to
    #   select your customization specification
    #
    #   3 - Include your own custom mapping logic here
    ##########################################################
    mapping = 2
    process_vmware(mapping, prov)

  when 'MiqProvisionVmwareViaPxe'
    ##########################################################
    # VMware PXE Customization Specification Mapping
    #
    # Possible values:
    #   0 - (DEFAULT No Mapping) This option skips the mapping of pxe images and customization templates
    #
    #   1 - CFME will look for a pxe image and a customization template with
    #   the exact name as the template name if none were chosen from the provisioning dialog
    #
    #   2 - Include your own custom mapping logic here
    ##########################################################
    mapping = 1
    process_vmware_pxe(mapping, prov)

  when 'MiqProvisionAmazon'
    ##########################################################
    # Amazon Specification Mapping
    #
    # Placeholder for future enhancements:
    #
    ##########################################################
    mapping = 0
    process_amazon(mapping, prov)

  when 'MiqProvisionOpenstack'
    ##########################################################
    # Openstack Specification Mapping
    #
    # Placeholder for future enhancements:
    #
    ##########################################################
    mapping = 0
    process_openstack(mapping, prov)

  else
    log(:info, "Provisioning Type:<#{prov.type}> does not match, skipping processing")
  end

  # When an error occurs during provisioning, this is used by
  # update_provision_status to clean up resources that were allocated
  # earlier in the provisioning process.
  prov.set_option(:rollback_parameters, {})

  # Log all of the provisioning options to the automation.log
  prov.options.each { |k,v| log(:info, "Provisioning Option Key:<#{k.inspect}> Value:<#{v.inspect}>") }

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  log(:info, "Check PROVISIONED PROV TAGS: #{prov.get_tags}")

  # Set Ruby rescue behavior
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
