###################################
#
# CFME Automate Method: process_tags
#
# Author: Jason Dillaman
#
# Notes: Process any VM tags that were sent via ws_values on the provision request
#
# Inputs: miq_provision.options[]
#
# Mods:
#
###################################

def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg.to_s
  exit MIQ_OK
end

def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def extract_tags(ws_values)
  tag_hash = {}
  ws_values.each do |k, v|
    next unless match = k.to_s.match(/tag_(.+)/)
    tag_hash[match[1].to_sym] = v unless v.blank?
  end
  
  $evm.log(:info, "Extracted tags: [#{tag_hash}]")
  tag_hash
end

def process_tag(category, tag)
  single_value = ![:intuit_software_component].include?(category.to_sym)
  
  # Convert to lower case and replace all non-word characters with underscores
  category_name = category.to_s.downcase.gsub(/\W/,'_')
  tag_name = tag.downcase.gsub(/\W/,'_')
  $evm.log(:info, "Converted category name:<#{category_name}> Converted tag name: <#{tag_name}>")

  # if the category exists else create it
  unless $evm.execute('category_exists?', category_name)
    $evm.log(:info, "Category <#{category_name}> doesn't exist, creating category")
    $evm.execute('category_create', :name => category_name, :single_value => single_value, :description => "#{category}")
  end
  # if the tag exists else create it
  unless $evm.execute('tag_exists?', category_name, tag_name)
    $evm.log(:info, "Adding new tag <#{tag_name}> in Category <#{category_name}>")
    $evm.execute('tag_create', category_name, :name => tag_name, :description => "#{tag}")
  end
end

def process_tags(tag_hash)
  begin
    tag_hash.each { |category, tag| process_tag(category, tag) }
  rescue => err
    $evm.log(:warn, "Failed to create tag: [#{category}][#{tag}][#{err}]\n#{err.backtrace.join("\n")}")
  end
end

def tags_to_classification_ids(tag_hash)
  tag_hash.collect do |category, tag|
    tag = $evm.vmdb('classification').find_by_name("#{category}/#{tag}")
    if tag
      $evm.log(:info, "Found tag:<#{tag.description}> id:<#{tag.id}>")  
      tag.id
    end
  end.compact
end

begin
  dump_root()
  
  task = $evm.root['miq_provision']
  error("Missing required parameter: miq_provision") unless task
  
  tag_hash = extract_tags(task.options[:ws_values])
  process_tags(tag_hash)
  classificaton_ids = tags_to_classification_ids(tag_hash)
  
  $evm.log(:info, "Updating VM tags: [#{classificaton_ids}]")
  task.set_option(:vm_tags, classificaton_ids)
  
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
