###################################
#
# CFME Automate Method: update_ownership
#
# Author: Jason Dillaman
#
# Notes: Updates a VM's ownership to the requester
#
# Inputs: miq_provision
#
###################################

# logging convenience method
def log(level, msg)
  @method = 'update_ownership'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def get_miq_provision()
  prov = $evm.root['miq_provision']
  error("Unable to retrieve provision task") unless prov
  prov
end

begin
  prov = get_miq_provision()
  
  vm = prov.destination
  ws_values = prov.get_option(:ws_values) || {}
  
  owner_userid = ws_values[:owner_userid]
  unless owner_userid.blank?
    user = $evm.vmdb('user').find_by_userid(owner_userid)
    if user
      log(:info, "Updating VM user ownership: [#{vm.name}][#{user.userid}]")
      vm.owner = user
    end
  end
  
  owner_group = ws_values[:owner_group]
  unless owner_group.blank?
    miq_group = $evm.vmdb('miq_group').find_by_description(owner_group)
    if miq_group
      log(:info, "Updating VM group ownership: [#{vm.name}][#{miq_group.description}]")
      vm.group = miq_group
    end
  end

  tier_name = ws_values[:intuit_tier_name]
  unless tier_name.blank?
    log(:info, "Setting tier name: #{tier_name}")
    vm.custom_set('Tier Name', tier_name)
  end
  
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
