##################################
#
# CFME Automate Method: wait_for_ip
#
# Author: Nick Lane, Carsten Clasohm
#
# Notes: Waits for the previously acquired IP address to appear in the VMDB
#        VM object. We do this before we install the HPSA agent, to ensure
#        that the VMware customization has been applied and that the VM has
#        network connectivity.
#
# Inputs: miq_provision
#
# Mods:
#
###################################

# logging convenience method
def log(level, msg)
  @method = 'wait_for_ip'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg.to_s
  exit MIQ_OK
end

def retry_method(msg, retry_time=1.minute)
  log(:warn, "Retrying in #{retry_time} seconds: [#{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_reason'] = msg.to_s
  $evm.root['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

begin
  prov = $evm.root['miq_provision']
  vm = prov.vm
  tags = prov.get_tags
  current_ip_addresses = vm.ipaddresses
  configured_ip_address = prov.get_option(:ip_addr)

###TESTING FOR QDC-A ROUTING
  hosting_segment = tags[:intuit_hs]


  if current_ip_addresses.include? configured_ip_address
    if hosting_segment == "qdca"
      result = $evm.instantiate('/Intuit/Integration/ManagementNicRoute/Method/AddManagementRoute')
      log(:info, "Result Inspect: #{result.inspect}")
      retry_method(result['reason']) unless result['result'] == 'ok'
    end
    log(:info, "Found IP address #{configured_ip_address} for VM #{vm.name}")
  else
    vm.refresh
    retry_method("Waiting for IP #{configured_ip_address} for VM #{vm.name}, current IPs: #{current_ip_addresses}")
  end

  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''
  exit MIQ_OK

rescue => err
  retry_method("Intermittent provider or credential failure") if err.to_s.include? "no Provider or credentials defined"
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
