##################################
#
# CFME Automate Method: wait_for_tools
#
# Author: Wayne Chatham, Nick Lane, Daniel Trieu
#
# Notes: Waits for the VMware Tools uprgade to complete. Waits for toolsOk status.
# Inputs: miq_provision
#
# Mods:
#
###################################

# logging convenience method
def log(level, msg)
  @method = 'wait_for_tools'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg.to_s
  exit MIQ_OK
end

def retry_method(msg, retry_time=1.minute)
  log(:warn, "Retrying in #{retry_time} seconds: [#{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_reason'] = msg.to_s
  $evm.root['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

begin
  prov = $evm.root['miq_provision']
  vm = prov.vm
  tags = prov.get_tags
  #current_ip_addresses = vm.ipaddresses
  #configured_ip_address = prov.get_option(:ip_addr)
  current_tools_status = vm.tools_status


  if current_tools_status == 'toolsOk'

    log(:info, "Upgrade complete on vm: #{vm.name}, tools status: #{current_tools_status}")
  else
    vm.refresh
    retry_method("Waiting for tools upgrade on vm: #{vm.name}, current status: #{current_tools_status}")
  end

  $evm.root['ae_result'] = 'ok'
  $evm.root['ae_reason'] = ''
  exit MIQ_OK

rescue => err
  retry_method("Intermittent provider or credential failure") if err.to_s.include? "no Provider or credentials defined"
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
