###################################
#
# CFME Automate Method: get_retirement_state_machine
#
# Author: Jason Dillaman
#
# Notes: Routes a retirement request to the correct region
#
# Inputs: vm
#
###################################

GET_CURRENT_REGION_NUMBER_URI = 'Intuit/Integration/CloudForms/Actions/GetCurrentRegionNumber'
DEFAULT_REGION_FACTOR = 1_000_000_000_000

def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute: [#{uri}]") unless result
  error(result['reason']) unless result['result'] == 'ok'
  return result
end

def region_number(object)
  object.id.to_i / DEFAULT_REGION_FACTOR
end

def get_current_region_number()
  region_number = instantiate(GET_CURRENT_REGION_NUMBER_URI)['region_number']
  error("Unable to determine current Intuit data center region number") if region_number.blank?
  region_number
end

def get_virtual_machine()
  vm = $evm.root['vm']
  error("Failed to retrieve VM") unless vm
  $evm.log(:info, "Located VM: [#{vm.name}]")
  vm
end

begin
  dump_root()

  $evm.log(:info, "Determining Retirement State Machine")

  current_region_number = get_current_region_number()
  
  vm = get_virtual_machine()
  
  $evm.log(:info, "Got VM: #{vm.name} ID: #{vm.id}")
  
  vm_region_number = region_number(vm)

  $evm.log(:info, "VM Region Number: #{vm_region_number}")
  
  if current_region_number == vm_region_number
    # VM is local to the current region
    $evm.log(:info, "Local Region Retirement")
    $evm.current['retirement_state_machine'] = 'VMRetirementLocal'
  else
    # VM is remote 
    $evm.log(:info, "Global Region Retirement")
    $evm.current['retirement_state_machine'] = 'VMRetirementGlobal'
  end

  $evm.log(:info, "Using Retirement State Machine: [#{$evm.current['retirement_state_machine']}]")
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
