###################################
#
# CFME Automate Method: CheckPoweredOff
#
# Notes: This method checks to see if the VM has been deleted from the Provider
#
###################################
begin
  # Method for logging
  def log(level, message)
    @method = 'CheckPoweredOff'
    $evm.log(level, "#{@method} - #{message}")
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root

  # Get vm from root object
  vm = $evm.root['vm']

  unless vm.nil?
    power_state = vm.attributes['power_state']
    ems = vm.ext_management_system
    log(:info, "VM:<#{vm.name}> on Provider:<#{ems ? ems.name : nil} has Power State:<#{power_state}>")

    # If VM is powered off or suspended exit
    if power_state == "off" || power_state == "suspended"
      # Bump State
      $evm.root['ae_result'] = 'ok'
    elsif power_state == "never"
      # If never then this VM is a template so exit the retirement state machine
      $evm.root['ae_result'] = 'error'
    else
      $evm.root['ae_result']         = 'retry'
      $evm.root['ae_retry_interval'] = '15.seconds'
    end
  end

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Set Ruby rescue behavior
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
