###################################
#
# CFME Automate Method: CheckUnregisteredFromProvider
#
# Notes: This method checks to see if the VM is unregistered from the Provider
#
###################################

# Method for logging
def log(level, message)
  @method = 'CheckUnregisteredFromProvider'
  $evm.log(level, "#{@method}: #{message}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg, retry_time=1.minute)
  log(:warn, "Retrying in #{retry_time} seconds: [#{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

begin
  log(:info, "CFME Automate Method Started")

  vm = $evm.root['vm']
  
  unless vm.nil?
    if !vm.registered?
      # Bump State
      log(:info, "VM:<#{vm.name}> has been unregistered from Provider")
      $evm.root['ae_result'] = 'ok'
    else
      # CloudForms sometimes has removed the EMS from the VM by now. Make
      # sure to *not* use vm.host.name or vm.ext_management_system.name.

      retry_method("VM #{vm.name} still present on host [#{vm.host}], EMS [#{vm.ext_management_system}]")
    end
  end

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
