###################################
#
# CFME Automate Method: create_park_vm_request
#
# Author: Nick Lane
#
# Notes: Sends a VM retire automation request to the local region
#
# Inputs: vm: the target service or VM
#
###################################

require 'uri'

CREATE_AUTOMATION_REQUEST_URI = 'Intuit/Integration/CloudForms/Actions/CreateAutomationRequest'
GET_REGION_ENDPOINTS_URI = 'Intuit/Integration/CloudForms/Actions/GetRegionEndpoints'

def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def escape_uri(value)
  URI.escape("#{value}", Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def instantiate(uri, params = {})
  $evm.log(:info, "Instantiating: [#{uri}]")
  result = $evm.instantiate(uri)
  return nil if result.blank? && params[:ignore_error]
  error("Failed to execute: [#{uri}]") unless result
  error(result['reason']) unless result['result'] == 'ok'
  return result
end

def create_automation_request(vm, intuit_dc, region_endpoint_ip_addresses)
  uri = "#{CREATE_AUTOMATION_REQUEST_URI}?" \
    "intuit_dc=#{intuit_dc}&" \
    "ae_namespace=#{escape_uri("Intuit/Automation/StateMachines")}&" \
    "ae_class=#{escape_uri("ParkVM")}&" \
    "ae_instance=#{escape_uri("Default")}&" \
    "parameters=#{escape_uri("Vm::vm=#{vm.id}")}"
  region_endpoint_ip_addresses.each_with_index do |ip_address, idx|
    break if instantiate("#{uri}&ip_address=#{ip_address}",
                         :ignore_error => (idx == region_endpoint_ip_addresses.length - 1))
  end
end

def get_virtual_machine()
  vm = nil
  case $evm.root['vmdb_object_type']
  when 'vm'
    vm = $evm.root['vm']
  else
    error("Unsupported Source Object: [#{$evm.root['vmdb_object_type']}]")
  end
  $evm.log(:info, "Located VM: [#{vm.name}]")
  vm
end

def get_region_endpoints(vm)
  region_endpoint_ip_addresses = []
  uri = "#{GET_REGION_ENDPOINTS_URI}?vmdb_object_type=vm&Vm::vm=#{vm.id}"
  result = instantiate(uri)
  intuit_dc = result['intuit_dc']
  region_endpoint_ip_addresses = result['region_endpoint_ip_addresses']
  $evm.log(:info, "Mapped VM to CloudForms region: [#{vm.name}][#{intuit_dc}][#{region_endpoint_ip_addresses}]")
  return intuit_dc, region_endpoint_ip_addresses
end

begin
  dump_root()
  
  vm = get_virtual_machine()
  
  intuit_dc, region_endpoint_ip_addresses = get_region_endpoints(vm)
  create_automation_request(vm, intuit_dc, region_endpoint_ip_addresses)
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
