###################################
#
# CFME Automate Method: DeleteFromProvider
#
# Notes: This method deletes the VM from the Provider
#
###################################

# Method for logging
def log(level, message)
  @method = 'DeleteFromProvider'
  $evm.log(level, "#{@method} - #{message}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg, retry_time=1.minute)
  log(:warn, "Retrying in #{retry_time} seconds: [#{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

begin
  log(:info, "CFME Automate Method Started")

  # Get vm from root object
  vm = $evm.root['vm']
  error("Failed to retrieve VM") unless vm
  
  ems = vm.ext_management_system
  log(:info, "Deleting VM:<#{vm.name}> from EMS:<#{ems ? ems.name : nil}>")
  vm.remove_from_disk

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

rescue => err
  log(:info, "Error: #{err.inspect} (class: #{err.class.to_s})")

  if err.class.to_s == "MiqException::MiqVimBrokerUnavailable"
    retry_method("VIM broker temporarily unavailable [#{err}]")
  elsif err.class.to_s == "DRb::DRbUnknownError"
    retry_method("HTTPClient Exception [#{err}]")
  elsif err.is_a?(RuntimeError)
    if /waitForTask: task not found/.match(err.message)
      retry_method("VM destroy task not found via VIM [#{err.message}]")
    else
      error("[#{err.message}]\n#{err.backtrace.join("\n")}")
    end
  else
    error("[#{err}]\n#{err.backtrace.join("\n")}")
  end
end
