##################################
#
# CFME Automate Method: power_off_vm
#
# Author: Nick Lane
#
# Notes: Sends a VM retire automation request to the local region
#
# Inputs: vm: the target service or VM
#
###################################

require 'uri'

CREATE_AUTOMATION_REQUEST_URI = 'Intuit/Integration/CloudForms/Actions/CreateAutomationRequest'
GET_REGION_ENDPOINTS_URI = 'Intuit/Integration/CloudForms/Actions/GetRegionEndpoints'

def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def escape_uri(value)
  URI.escape("#{value}", Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end




def get_virtual_machine()
  vm = nil
  case $evm.root['vmdb_object_type']
  when 'vm'
    vm = $evm.root['vm']
  when 'automation_task'
    vm = $evm.root['vm']
  else
    error("Unsupported Source Object: [#{$evm.root['vmdb_object_type']}]")
  end
  $evm.log(:info, "Located VM: [#{vm.name}]")
  vm
end

def get_region_endpoints(vm)
  region_endpoint_ip_addresses = []
  uri = "#{GET_REGION_ENDPOINTS_URI}?vmdb_object_type=vm&Vm::vm=#{vm.id}"
  result = instantiate(uri)
  intuit_dc = result['intuit_dc']
  region_endpoint_ip_addresses = result['region_endpoint_ip_addresses']
  $evm.log(:info, "Mapped VM to CloudForms region: [#{vm.name}][#{intuit_dc}][#{region_endpoint_ip_addresses}]")
  return intuit_dc, region_endpoint_ip_addresses
end

begin
  dump_root()
  
  vm = get_virtual_machine()
  $evm.log(:info, "Got VM #{vm.inspect}") 


  ems_tags = vm.ext_management_system.tags
  intuit_dc = ems_tags.select {|e_tags| e_tags.include? "intuit_dc"}.first.split('/').last
  $evm.log(:info, "DC = #{intuit_dc}")
  $evm.root['intuit_dc'] = intuit_dc
  $evm.root['ip_address'] = vm.ipaddresses
  $evm.log(:info, "IP:#{$evm.root['ip_address']}")

  hostname = vm.name

 # vm.stop
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
