###################################
#
# CFME Automate Method: release_ip_addresses
#
# Author: Carsten Clasohm
#
# Notes: Remove Men & Mice records for all IP address used by a VM.
#
# Inputs: vm
#
# Mods:
#
###################################

require 'savon'
require 'rubygems'
require 'httpi'

# logging convenience method
def log (level, msg) 
  @method = 'release_ip_addresses'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, "#{msg}")
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def retry_method(msg, retry_time=1.minute)
  log(:warn, "Retrying in #{retry_time} seconds: [#{msg}]")
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = retry_time
  exit MIQ_OK
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute #{uri}") unless result
  
  case result['result']
  when 'error'
    error(result['reason'])
  when 'retry'
    retry_method(result['reason'])
  end
  
  result
end

def get_virtual_machine()
  case $evm.root['vmdb_object_type']
  when 'vm'
    vm = $evm.root['vm']
  end
  raise "VM not found" unless vm
  vm
end

def get_intuit_dc(vm)
  ems = vm.ext_management_system
  intuit_dc = ems.tags(:intuit_dc).first
  error("Intuit Data Center tag missing on provider: [#{ems.name}]") unless intuit_dc
  return intuit_dc
end

# --- BEGIN MAIN -----#
begin
  vm = get_virtual_machine()
  intuit_dc = get_intuit_dc(vm)
  dns_zone = vm.name.sub(%r{^[^.]+\.}, '') + '.'

  #unless vm.ipaddresses.blank?
    ip_addresses = vm.ipaddresses.join(",")
    log(:info, "Calling Release IP address for VM: #{vm.name} with IP addresses: #{vm.ipaddresses}")
    ae_uri = "/Intuit/Integration/MenAndMice/Actions/ReleaseIPAddress?" + \
             URI.encode_www_form([['intuit_dc', intuit_dc],
                                  ['dns_zone', dns_zone],
                                  ['vm_name', vm.name],
                                  ['ip_address', ip_addresses]])
    result = instantiate(ae_uri)

    error("failure in ReleaseIPAddress") if result.nil?
  #else
  #  log(:info, "MAIN: vm.ipaddresses blank: #{vm.ipaddresses}")
  #end

  log(:info, "Main: exiting main method")
  # --- END MAIN -----#
  exit MIQ_OK
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
