###################################
#
# CFME Automate Method: VMRetirement_Extend
#
# Notes: This method is used to add x days to retirement date when target
# Instance has a retires_on value and is not already retired
#
# Inputs: $evm.root['dialog_vm_retire_extend_days']
#
###################################
begin
  # Method for logging
  def log(level, message)
    @method = 'VMRetirement_Extend'
    $evm.log(level, "#{@method} - #{message}")
  end

  # dump_root
  def dump_root()
    log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
    $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
    log(:info, "Root:<$evm.root> End $evm.root.attributes")
    log(:info, "")
  end

  log(:info, "CFME Automate Method Started")

  # dump all root attributes to the log
  dump_root

  # Number of days to automatically extend retirement if none was provided (default to 14)
  vm_retire_extend_days = nil
  vm_retire_extend_days ||= $evm.root['dialog_vm_retire_extend_days'] || $evm.object['dialog_vm_retire_extend_days'] || 14
  raise "vm_retire_extend_days not found!" if vm_retire_extend_days.nil?

  log(:info,"Number of days to extend: <#{vm_retire_extend_days}>")

  obj = $evm.object("process")

  vm = obj["vm"] || $evm.root["vm"]

  # Log Current Retirement
  log(:info,"VM:<#{vm.name}> vm.retired:<#{vm.retired.inspect}> vm.retires_on:<#{vm.retires_on.inspect}> vm.retirement_warn:<#{vm.retirement_warn.inspect}> vm.retirement_last_warn:<#{vm.retirement_last_warn.inspect}>")

  # Bail out if VM does not have retirement date
  if vm.retires_on.nil? | vm.retires_on == ""
    log(:info,"VM '#{vm.name}' has no retirement date - extension bypassed")
    exit MIQ_OK
  end

  # If VM is already retired to do not continue
  if vm.retired then
    log(:info, "VM '#{vm.name}' is already marked as retired. No Action taken")
    #exit MIQ_OK
  end

  unless vm.retires_on.nil?
    log(:info, "Extending retirement <#{vm_retire_extend_days}> days for VM: <#{vm.name}>")

    # Set new retirement date here
    vm.retires_on += vm_retire_extend_days.to_i

    log(:info, "VM: <#{vm.name}> new retirement date is #{vm.retires_on}")

    # Log Retirement
    log(:info,"VM:<#{vm.name}> vm.retired:<#{vm.retired.inspect}> vm.retires_on:<#{vm.retires_on.inspect}> vm.retirement_warn:<#{vm.retirement_warn.inspect}> vm.retirement_last_warn:<#{vm.retirement_last_warn.inspect}>")

    ######################################
    #
    # VM Retirement Exended Email
    #
    ######################################

    # Get VM Owner Name and Email
    evm_owner_id = vm.attributes['evm_owner_id']
    owner = nil
    owner = $evm.vmdb('user', evm_owner_id) unless evm_owner_id.nil?
    #log(:info, "Inspecting VM Owner: #{owner.inspect}")

    # to_email_address from owner.email then from model if nil
    unless owner.nil?
      to = owner.email
    else
      to = $evm.object['to_email_address']
    end

    # Get from_email_address from model unless specified below
    from = nil
    from ||= $evm.object['from_email_address']

    # Get signature from model unless specified below
    signature = nil
    signature ||= $evm.object['signature']

    # email subject
    subject = "VM Retirement Extended for #{vm.name}"

    # Build email body
    body = "Hello, "
    body += "<br><br>The retirement date for your virtual machine: [#{vm.name}] has been extended to: [#{vm.retires_on}]."
    body += "<br><br> Thank you,"
    body += "<br> #{signature}"

    # Send email
    log(:info, "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
    $evm.execute('send_email', to, from, subject, body)
  end

  # Exit method
  log(:info, "CFME Automate Method Ended")
  exit MIQ_OK

  # Set Ruby rescue behavior
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
