###################################
#
# CFME Automate Method: get_hpsa_software_policies
#
# Author: Jason Dillaman
#
# Notes: Retrieve a list of all available HPSA Software Policies (by filter)
#
# Inputs: dialog_hpsa_software_policy_filter
#
###################################

require 'uri'

GET_CURRENT_INTUIT_DC_URI = 'Intuit/Integration/CloudForms/Actions/GetCurrentIntuitDc'
GET_HPSA_SOFTWARE_POLICIES_URI = 'Intuit/Integration/HPSA/Actions/GetSoftwarePolicies'

def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def escape_uri(value)
  URI.escape("#{value}", Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute: [#{uri}]") unless result
  error(result['reason']) unless result['result'] == 'ok'
  return result
end

def get_intuit_dc()
  intuit_dc = instantiate(GET_CURRENT_INTUIT_DC_URI)['intuit_dc']
  error("Unable to determine current Intuit data center") if intuit_dc.blank?
  intuit_dc
end

def get_hpsa_software_policies(intuit_dc, hpsa_software_policy_filter)
  result = instantiate("#{GET_HPSA_SOFTWARE_POLICIES_URI}?" \
    "intuit_dc=#{escape_uri(intuit_dc)}&" \
    "hpsa_software_policy_filter=#{escape_uri(hpsa_software_policy_filter)}")
  $evm.log(:info, "Discovered HPSA Software Policies: [#{result['hpsa_software_policies']}]")
  result['hpsa_software_policies']
end

def build_dialog_values(software_policies)
  # CFME does not properly handle sorted dynamic fields
  list_values = {
    'sort_by'    => :none,
    'data_type'  => :string,
    'required'   => true,
    'values'     => [[nil, nil]] + software_policies.sort_by { |x| x[:name] }.to_a.collect { |x| [x[:name], x[:id]] }
  }

  $evm.log('info', "HPSA Software Policies drop-down: [#{list_values}]")
  list_values.each { |k,v| $evm.object[k] = v }
end

begin
  dump_root()
  
  intuit_dc = get_intuit_dc()
  software_policies = get_hpsa_software_policies(intuit_dc, $evm.root['dialog_hpsa_software_policy_filter'])
  build_dialog_values(software_policies)
  
rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
