##################################
#
# CFME Automate Method: get_lifecycles
#
# Author: Nick Lane, Daniel Trieu
#
# Notes: Retrieve a list of all online security zones
#
# Changes:
#   2015-11-11 - ew - clean up from a original copy paste and add more debug logging
#   2015-12-02 Daniel Trieu updated the values in list_values to hash, as is req by CF3.2
###################################

# logging convenience method
def log (level, msg)
  @method = 'get_lifecycles'
  $evm.log(level, "#{@method}: #{msg}")
end

def error(msg)
  log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def dump_root()
  log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  log(:info, "Root:<$evm.root> End $evm.root.attributes")
  log(:info, "")
end

def get_lifecycles(clusters, hosting_segment)
  lifecycles = Hash.new
  lf_array = Array.new
  clusters.each { |cluster|
    log(:info, "cluster: #{cluster}")
    #next unless cluster.tagged_with?("intuit_sz", "prod") || cluster.tagged_with? ("intuit_sz", "preprod")
    sec_zones = cluster.tags('intuit_sz')
    sec_level = cluster.tags('intuit_security_level')
    fault_domains = cluster.tags('intuit_fd')
    log(:info, "FD:#{fault_domains}")
    log(:info, "SL: #{sec_level}")
    sec_zones.each {|sz|
      sec_level.each {|sl|
        fault_domains.each {|fd|
          lf_array << "#{hosting_segment}_#{sz.to_s}_#{sl.to_s}_#{fd.to_s}"
        }
      }
    }
  }

  log(:info, "lifecycles: #{lf_array}")

  return lf_array.uniq
end

def build_dialog_values(lf_array)
  # CFME does not properly handle sorted dynamic fields
  values = lf_array.map { |x| ["#{x}", x.split(/_/).map {|y| y.upcase}.join(' ')] }.sort_by { |z| z.first }
  log(:info, "lf_array = #{values}")

  values_array = [['','']] + values
  values_hash = {}
  values_array.each{ |k,v| values_hash.merge!({k=>v})}

  list_values = {
    'sort_by'       => :none,
    'data_type'     => :string,
    'required'      => true,
    'default_value' => '',
    'values'        => values_hash
  }

  log(:info, "Security Zone drop-down: [#{list_values}]")
  list_values.each { |k,v| $evm.object[k] = v }
end

begin
  dump_root()

  hosting_segment = $evm.inputs['hosting_segment']
  log(:info, "Hosting Segment: #{hosting_segment}")
  clusters = $evm.vmdb('ems_cluster').all.select { |cluster|
    cluster.tagged_with?('intuit_hs', hosting_segment)
  }

  lf_array = get_lifecycles(clusters, hosting_segment)

  build_dialog_values(lf_array)

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
