###################################
#
# CFME Automate Method: get_hosting_segment_list
#
# Author: Bryan Antigua
#
# Notes: Retrieve a list of hosting segments available from ServiceNow and populates a dropdown
#
# Inputs: 
#
###################################

require 'uri'

GET_SERVICE_NOW_HOSTING_SEGMENT_LIST_URI = '/Intuit/Integration/ServiceNow/Actions/GetHostingSegmentList'
GET_CURRENT_INTUIT_DC_URI = 'Intuit/Integration/CloudForms/Actions/GetCurrentIntuitDc'

def error(msg)
  $evm.log(:error, msg)
  $evm.root['ae_result'] = 'error'
  $evm.root['ae_reason'] = msg
  exit MIQ_OK
end

def escape_uri(value)
  URI.escape("#{value}", Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))
end

def dump_root()
  $evm.log(:info, "Root:<$evm.root> Begin $evm.root.attributes")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "Root:<$evm.root> Attribute - #{k}: #{v}")}
  $evm.log(:info, "Root:<$evm.root> End $evm.root.attributes")
  $evm.log(:info, "")
end

def instantiate(uri)
  result = $evm.instantiate(uri)
  error("Failed to execute: [#{uri}]") unless result
  error(result['reason']) unless result['result'] == 'ok'
  return result
end

def get_intuit_dc()
  intuit_dc = instantiate(GET_CURRENT_INTUIT_DC_URI)['intuit_dc']
  error("Unable to determine current Intuit data center") if intuit_dc.blank?
  intuit_dc
  #intuit_dc = 'qdc'
  #intuit_dc
end
    
def get_service_now_hosting_segment_list(intuit_dc)
  result = instantiate("#{GET_SERVICE_NOW_HOSTING_SEGMENT_LIST_URI}?" \
    "intuit_dc=#{escape_uri(intuit_dc)}");
  #$evm.log(:info, "ServiceNow Hosting Segment List [#{result['hosting_segment_list']}]")
  result['hosting_segment_list']
end

def build_dialog_values(hosting_segment_list)
  # CFME does not properly handle sorted dynamic fields
  list_values = {
    'sort_by'    => :none,
    'data_type'  => :string,
    'required'   => true,
    'values'     => [[nil, nil]] + hosting_segment_list.sort_by { |x| x['label'] }.collect { |x| [x['label'], x['label']] }
  }

  #$evm.log(:info, "ServiceNow Hosting Segment drop-down: [#{list_values}]")
  list_values.each { |k,v| $evm.object[k] = v }
end

begin
 # $evm.log(:info, "Automate Method Started")
  intuit_dc = get_intuit_dc()
  hosting_segment_list = get_service_now_hosting_segment_list(intuit_dc)
  build_dialog_values(hosting_segment_list)
 # $evm.log("info", "Automate Method Ended")

rescue => err
  error("[#{err}]\n#{err.backtrace.join("\n")}")
end
