#
#            EVM Automate Method
#
$evm.log("info", "EVM Check Multicast Starting")


# Change those variables based on desired thresholds and email recipient
max_multicast_vms = "5"
to ||= 'evm_demo@manageiq.com'
from = 'evm_demo@manageiq.com'

#
# Find the VM
vm = $evm.root['vm']
return if vm.nil?

event=$evm.root['ems_event']
$evm.log('info', "Event Dest Host Name: <#{event.dest_host_id}>")


# Only check for powered on VMs
if vm.power_state == "on"

  # Find the Host this VM is running on
  if event.dest_host_id.nil?
    # If it is not a VM relocation set the host to the current host
    host = vm.host
    return if host.nil?
  else
    # If it driven by a VM relocation event set the host to the destination host and the source host to where the VM is coming from
    host = vm.ext_management_system.hosts.detect { |h| h['id'] == event.dest_host_id.to_i }
  end


  # Under this Host, find all VMs tagged with category 'multicast' and tag of 'true' and running
  multicast_vms = host.vms.find_all { |v| v.tagged_with?('multicast', 'true') &&  v.power_state == "on"}

  $evm.log('info', "Host <#{host.name}> has #{multicast_vms.size} Multicast VMs running on this Host already!")

  # Check if over the limit and take appropriate action
  if multicast_vms.size > max_multicast_vms.to_i
    if event.dest_host_id.nil?
      $evm.log('info', "Stopping VM <#{vm.name}> on Host <#{host.name}> because there are #{multicast_vms.size} Multicast VMs running on this Host already!")
      vm.stop
      subject = "VM <#{vm.name}> on Host <#{host.name}> has been stopped because there are #{multicast_vms.size} Multicast VMs running on this Host already!"
    else
      # Get VM's Cluster
      cluster = vm.ems_cluster

      # Stop the VM if NOT in a cluster
      if cluster.nil?
        $evm.log('info', "Stopping VM <#{vm.name}> on Host <#{vm.host.name}> because it is not in a cluster")
        subject = "VM <#{vm.name}> on Host <#{host.name}> has been stopped because there are #{multicast_vms.size} Multicast VMs running on this Host already and this host is not in a cluster!"
        vm.stop
      else
       # Get all the hosts and their count of VMs that are tagged as multicast and powered on
       mhosts_array  = []
       cluster.hosts.each do |target_host|
         multicast_vms = target_host.vms.find_all { |v| v.tagged_with?('multicast', 'true') &&  v.power_state == "on"}.size
         # Do NOT add the VMs host to candidate hosts for vmotion
         next if target_host == host
         # Create a tuple (or tiny array) that we will put onto the mhosts_array
         tuple = [target_host, multicast_vms]
         # Add the tuple to the mhosts_array
         mhosts_array << tuple
       end
       # Make sure that there is at least 1 element in the array
       if mhosts_array.size == 0
         $evm.log('info', "Stopping VM <#{vm.name}> on Host <#{host.name}> because there are no other hosts in the cluster")
         vm.stop
         subject = "VM <#{vm.name}> on Host <#{host.name}> has been stopped because there are #{multicast_vms.size} Multicast VMs running on this Host already and no host to relocate it!"
       else
         # Sort the array by count of VMs that are tagged as multicast and powered on
         mhosts_array.sort! { |a,b| a.last <=> b.last }

         # Get the first tuple of the sorted array
         tuple = mhosts_array.first

         # Break up tuple into target host and count
         target_host  = tuple.first
         count = tuple.last
         $evm.log('info', "Target Host <#{target_host}> has #{count} Multicast VMs running!")
       end

       if count >= max_multicast_vms.to_i
          $evm.log('info', "Stopping VM <#{vm.name}> on Host <#{host.name}> because there is nowhere safe to vmotion it to")
          vm.stop
          subject = "VM <#{vm.name}> on Host <#{host.name}> has been stopped because there are #{multicast_vms.size} Multicast VMs running on this Host already and no host to relocate it!"
       else
          $evm.log('info', "Migrating VM <#{vm.name}> on Host <#{host.name}> to Host <#{target_host.name}>")
          vm.migrate(target_host)
          subject = "VM <#{vm.name}> on Host <#{host.name}> has been migrated Host <#{target_host.name}> because there are #{multicast_vms.size} Multicast VMs running on the Host already."
       end
      end
    end
    # Send email
    body = "VM Information: <br>
    VM: #{vm.name} <br>
    Host: #{host.name} <br>"
    $evm.execute('send_email', to, from, subject, body)
  end
else
$evm.log('info', "This method only apply to power on VMs. #{vm.name} is #{vm.power_state}")
end
