###################################
#
# EVM Automate Method: ConfigureChildDialog
#
# Notes: Sample method to configure/set child dialog values
#
###################################
begin
  @method = 'ConfigureChildDialog'
  $evm.log("info", "#{@method} - EVM Automate Method Started")

  # Turn of verbose logging
  @debug = true

  # Method Code Goes here
  $evm.log("info", "#{@method} - ===========================================") if @debug
  $evm.log("info", "#{@method} - Listing ROOT Attributes:") if @debug
  $evm.root.attributes.sort.each { |k, v| $evm.log("info", "#{@method} - \t#{k}: #{v}") if @debug}
  $evm.log("info", "#{@method} - ===========================================") if @debug

  stp_task = $evm.root["service_template_provision_task"]
  $evm.log("info", "#{@method} - ===========================================") if @debug
  $evm.log("info", "#{@method} - Listing task Attributes:") if @debug
  stp_task.attributes.sort.each { |k, v| $evm.log("info", "#{@method} - \t#{k}: #{v}") if @debug}
  $evm.log("info", "#{@method} - ===========================================") if @debug

  $evm.log("info","#{@method} - No child dialog options have been configured.") if @debug

  dialog_service_type = $evm.root['dialog_service_type']
  $evm.log("info","#{@method} - User selected Dialog option = [#{dialog_service_type}]") if @debug

  stp_miq_request_task = stp_task.miq_request_task
  #$evm.log("info","#{@method} - (parent) miq_request_task:  = [#{stp_miq_request_task}]") if @debug

  stp_miq_request_tasks = stp_task.miq_request_tasks
  #$evm.log("info","#{@method} - (children) miq_request_tasks count:  = [#{stp_miq_request_tasks.count}]") if @debug

  stp_miq_request_tasks.each do |t|
    $evm.log("info"," Setting dialog for: #{t.description}")
    service = t.source
    service_resource = t.service_resource
    #$evm.log("info","#{@method} - Child service resource name: #{service_resource.resource_name}") if @debug
    #$evm.log("info","#{@method} - Child service resource description: #{service_resource.resource_description}") if @debug

    service_tag_array = service.tags(:app_tier)
    service_tag = service_tag_array.first.to_s

    memory_size = nil

    case dialog_service_type
    when "Small"
      case service_tag
      when "app"
        memory_size = 1024
      when "web"
        memory_size = 1024
      when "db"
        memory_size = 4096
      else
        $evm.log("info","#{@method} - Unknown Dialog type") if @debug
      end
    when "Large"
      case service_tag
      when "app"
        memory_size = 4096
      when "web"
        memory_size = 4096
      when "db"
        memory_size = 8192
      else
        $evm.log("info","#{@method} - Unknown Dialog type") if @debug
      end
    else
      $evm.log("info","#{@method} - Unknown Dialog type - setting Dialog options here") if @debug
    end

    t.set_dialog_option('memory',memory_size) unless memory_size.nil?

    $evm.log("info","#{@method} - Set dialog for selection: [#{dialog_service_type}]  Service_Tier: [#{service_tag}] Memory size: [#{memory_size}]") if @debug
  end

  #
  # Exit method
  #
  $evm.log("info", "#{@method} - EVM Automate Method Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "#{@method} - [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
