######################################################################################################
  # EVM Extend Retirement Date method
  ######################################################################################################
  #
  #        Description: Add 14 days to retirement date when target VM has a retires_on value and is not already retired
  #        Version    : 1.0
  #        Last Update: 09102010
  #        Author:    : ManageIQ
  #
  #        1-setting "debug = true" will enable full visualizatoin of objects "process", "VM" and "user"
  #        2- default but dormant date threshold is 14 days
  #        3- threshold activation achieved by setting "activate_date_threshold" to true
  #            when threshold is active, then VMs with a retires_on value more than "future_threshold_in_days" value
  #             in the future will be logged but not changed.
  #
  #        Objects Referenced:
  #         - evm.root              -   default automation object
  #         - evm.object["process'] -           -   job object
  #         - evm.root["vm"]        -   vm to be processed
  #         - evm.root["user"]      -   information about operator requesting retirement extension

begin_time = Time.now             # used for timing of method & logging at method end
$evm.log("info", "===== EVM Automate Method 'extend_retirement_date' Started")
$evm.log("info", "created 9/10/2010 by tch (manageiq)")

def exit_method( begin_time)
$evm.log("info", "===== EVM Automate Method 'extend_retirement_date' Ended -> duration #{Time.now - begin_time} seconds")
exit MIQ_OK
end

debug = false
# if debug is true then the user and VM objects will be fully exposed, otherwise
# these sections of code will be skipped (tch 2010-09-09)

activate_date_threshold = false
# if activate_date_threshold = true then VMs with a retirement date greater than
# that threshold will not be modified.  the threshold is a count of the number of days
# between the current date and the date set as vm.retires_on.  If the count exceeds the
# threshold and the threshold is active, then no change to the retirement date will occur

# *  * * * * * * * * * * * * *  ***
future_threshold_in_days = 14

# future_threshold_in_days is provided as a limit as to how far into the future a retirement
# may be set and still allow for the retirement action to be performed.
# the code is in place to use this threshold, but is commented out since no threshold
# has as yet been requested.
# *  * * * * * * * * * * * * *  ***




$evm.log("info","current date info '#{DateTime.now}' - '#{Time.now()}'")
work_datetime = Time.at(Time.now.to_f + (future_threshold_in_days * 24 * 3600))
$evm.log("info","value of proposed future threshold date is '#{work_datetime}'")

future_limit = Date.new(work_datetime.year, work_datetime.month, work_datetime.day)

#future_limit is being constructed as a Date type field so that comparison with vm.retires_on is
# a like-for-like comparison and we can do differencing on the dates.

#if retirement > future_threshold_in_days days in future, don't retire - warning only
# it still will update the retirment date


if debug then
     $evm.log("info","future_limit is of type '#{future_limit.class.to_s}' with value of '#{future_limit}'")
end

#
#            Method Code Goes here
#
evm = $evm.object
if debug then
     $evm.log("info","evaluating and displaying 'evm.object' object")
     $evm.log("info"," evm.object is of type '#{evm.class.to_s}'")
     evm.attributes.sort.each { |k,v| $evm.log("info", "\t#{k}: \(#{k.class.to_s}\) -> #{v}") }
     $evm.log("info","===== end of evm.object elements =====\n")
end

obj = $evm.object("process")
if debug then
     $evm.log("info", "evaluating and displaying 'process' object")
     $evm.log("info"," evm.object\(\"process\"\) is of type '#{obj.class.to_s}'")
     obj.attributes.sort.each { |k, v|  $evm.log("info", "\t#{k}: \(#{k.class.to_s}\)->#{v}") }
end


vm = obj["vm"]
if debug then
     $evm.log("info", "evaluating and displaying obj[vm] object try 1")
     $evm.log("info", "display 'vm' object info -> 'vm' is of type #{vm.class.to_s}" )
end

vm = $evm.object["vm"]
if debug then
     $evm.log("info", "evaluating and displaying 'vm' object try 2")
     $evm.log("info", "display 'vm' object info -> 'vm' is of type #{vm.class.to_s}" )
end

vm = $evm.root["vm"]
if debug then
     $evm.log("info", "===== begin evaluating and displaying 'vm' object try 3 =====")
     $evm.log("info", "display 'vm' object info -> 'vm' is of type #{vm.class.to_s}" )
     vm.attributes.sort.each { |k,v| $evm.log("info", "\t#{k}: of type '#{k.class.to_s}' has value '#{v}'") }
     # $evm.log("info","vm.inspect yields ->'#{vm.inspect}' ")
     $evm.log("info", "===== end evaluating and displaying 'vm' object try 3 =====")
end
case vm.retires_on
    when nil then
          $evm.log("info","VM '#{vm.name}' has no retirement date - extension bypassed")
          exit_method(begin_time)
     when "" then
          $evm.log("info","VM '#{vm.name}' has no retirement date - extension bypassed")
          exit_method(begin_time)

end

# since the vm.retired attribute is not exposed for alteration in the automate code, we cannot unretire
# a VM, so we will also not allow the retirement date to be altered on a VM that is already retired.
if vm.retired then
     $evm.log("info", "vm '#{vm.name}' is already marked as retired. vm.retires_on date value is  #{vm.retires_on}. No Action taken")
     exit_method(begin_time)
end

if debug then
     $evm.log("info","value of vm.retires_on is '#{vm.retires_on}'")
end

#if debug then
#     $evm.log("info", "expose local variable pool")
#     local_variables.each { |k| $evm.log("info", "\t#{k} -> #{eval(k.class.to_s)} ->#{eval(k)} ") }
#     eval("local_variables").each { |k| $evm.log("info", "local variable ->\t#{k}") }
#end

if debug then
     $evm.log("info", "expose vm attributes and values")
     # vm.attributes.sort.each { |k,v| $evm.log("info", "\t#{k}: of type '#{k.class.to_s}' with value of '#{v}'" ) }
     $evm.log("info","#{vm.inspect}")
end

$evm.log("info", "vm retirement date is #{vm.retires_on}")

#if vm.retires_on.to_i > future_limit.to_i then # DateTime can't handle .to_f method
$evm.log("info","date delta calculation #{future_limit - vm.retires_on} days") if debug
if (future_limit - vm.retires_on) < 0 then
     $evm.log("info","VM '#{vm.name}' retirement date '#{vm.retires_on}' more than #{future_threshold_in_days} days in future ")
     if activate_date_threshold then
          $evm.log("info","VM '#{vm.name}' retirement date not changed")
          exit_method(begin_time)
      end
end



user = $evm.root["user"]
$evm.log("info","vm name '#{vm.name}' retirement date being altered by userid '#{user.userid}' role-> '#{user.name}'. ")

if debug then
    $evm.log("info"," ===== attempt to expose entire user object =====\n")
     user.attributes.sort.each { |k,v| $evm.log("info", "\t#{k}: #{v}") }
     $evm.log("info","===== end attempt to expose entire user object =====\n")
end


unless vm.retires_on.nil?
  # Set new retirement date here
  vm.retires_on = vm.retires_on + 14

  $evm.log("info", "vm new retirement date is #{vm.retires_on}")


  # Save
  #vm.save
end
#
#
#
exit_method(begin_time)
#$evm.log("info", "===== EVM Automate Method 'extend_retirement_date' Ended -> duration #{end_time - begin_time} seconds")
#exit MIQ_OK
