###################################
#
# EVM Automate Method: PreProvision_sample1
#
# Notes: This method is used to customize the provisioning object prior to provisioning
#
###################################
begin
  @method = 'PreProvision_sample1'
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Started")

  #
  # Get Variables
  #
  miq_provision = $evm.root["miq_provision"] || $evm.root['miq_provision']
  prov = $evm.root["miq_provision"]
  user = prov.miq_request.requester
  raise "User not specified" if user.nil?


  ###################################
  # Process Intake/RFC Number and set VM Summary Description
  ###################################
  intake = prov.get_option(:vm_description)
  intake = "Intake/RFC Number: #{intake}"
  prov.set_option(:vm_description,intake)


  ###################################
  # Set the customization spec here
  # If one selected in dialog it will be used,
  # else it will map template to customization spec based on template name
  ###################################
  if prov.get_option(:sysprep_custom_spec).nil?
    $evm.log("info", "Forcing the custom spec to match template name")
    customization_spec = miq_provision.vm_template.name # to match the template name
    miq_provision.set_customization_spec(customization_spec)
  end



  ###################################
  # Set the VLAN as follows:
  # miq_provision.set_vlan('default')  # Straight vLAN
  # miq_provision.set_dvs('vlan220')   # Distributed Virtual Switch
  ###################################
  template = miq_provision.vm_template
  virtual_center = template.ext_management_system


  ###################################
  # Was a VLAN selected in dialog?
  # If not you can set one here.
  ###################################
  if prov.get_option(:vlan).nil?
    # Set the Distributed Virtual Switch based on the VC
    if virtual_center.name.downcase.include?('vlab')
      #     miq_provision.set_vlan('vlan22')
      miq_provision.set_dvs('portgroup1')
    else
      #      miq_provision.set_vlan('VLAN207-VMs')
      miq_provision.set_dvs('portgroup2')
    end
  end


  ###################################
  # Set the VM Notes as follows:
  ###################################
  vm_notes = "#{intake}"
  vm_notes +=  "\nOwner: #{miq_provision.get_option(:owner_first_name)} #{miq_provision.get_option(:owner_last_name)}"
  vm_notes += "\nEmail: #{miq_provision.get_option(:owner_email)}"
  vm_notes += "\nSource Template: #{miq_provision.vm_template.name}"

  miq_provision.set_vm_notes(vm_notes)


  ###################################
  # Drop the VM in the targeted folder
  # In VC a folder must exist that matches the LDAP Group
  # VM will be placed in the Folder
  ###################################
  begin

    if prov.get_option(:placement_folder_name).nil?
      ###################################
      # If you want to use a Default folder, set folder = below to the default
      ###################################
      #    folder = "22F DC/LAB FARM/GSE/Intel/Infrastructure/ManageIQ/SelfServiceVMs"
      folder = "DC1/Infrastructure/ManageIQ/SelfService"
      $evm.log("info", "Placing VM in VC folder: <#{folder}")
      $evm.log("info", "Set_folder called with [#{folder.inspect}]")

      miq_provision.set_folder(folder)
    end

  rescue => err
    $evm.log("info", "Set_folder err [#{err}]\n#{err.backtrace.join("\n")}")
  end


  ###################################
  # Set the VM tags based on folder
  ###################################
  if miq_provision.get_tags.empty?

    ###################################
    # Below example is to set Tags based on VC selected
    ###################################
    if virtual_center.name.downcase.include?('prod')
      miq_provision.add_tag(:environment, 'prod')
    else
      miq_provision.add_tag(:environment, 'lab')
    end
    ###################################
    # Below example is to set Tags based on VC folder path
    # This needs to be mapped to how deep the Folder path is from section above
    ###################################
    #  if folder.kind_of?(Array)
    #    paths = folder.last.split('/')
    #    dept, group = paths[-2], paths[-1]
    #    miq_provision.add_tag(:department, dept)
    #    miq_provision.add_tag(:group, group)
    #  end
  end


  ####################################################
  # IP Adress Management
  #
  # Specify filename to read
  # File must contain the following construct
  # VLAN Name  IP Addr    Gateway       Router  status
  ############ #########  ############, ####### ######
  # VM Network,10.1.1.10,255.255.255.0,10.1.1.1,[used, free]
  $evm.log("info", "IP Address Management Starting")
  fname = "/var/www/miq/iplist.csv"
  raise "File '#{fname}' does not exist" unless File.exist?(fname)

  # Get VM Name to be provisioned
  vm_hostname = prov.get_option(:vm_target_name).strip.gsub(/\W/, '_')
  vm_vlan = prov.get_option(:vlan)
  $evm.log("info", "VM VLAN: <#{vm_vlan}>")
  $evm.log("info", "VM Name: <#{vm_hostname}>")


  # Regular Expression to match the following construct:
  # VLAN Name ,IP Addr  Subnet Mask  ,Gateway , [used,free]
  # VM Network,10.1.1.1,255.255.255.0,10.1.1.1,used
  regex_line = /^(vlan.*|vm\snet.*|VM2.*),(\d*).(\d*).(\d*).(\d*),(\d*).(\d*).(\d*).(\d*),(\d*).(\d*).(\d*).(\d*),(\w*)/i
  $evm.log("info", "regex_line: <#{regex_line}>")

  # Open file for reading and iterate through the file looking for a match
  File.open(fname) do |file|
    file.each do |line|
      # convert everything to lowercase and strip all leading/trailing whitespaces
      line = line.strip

      # if the regular expression successfully matches else skip the line
      if regex_line =~ line
        vlan    = $1
        octet1  = $2
        octet2  = $3
        octet3  = $4
        octet4  = $5
        mask1   = $6
        mask2   = $7
        mask3   = $8
        mask4   = $9
        gate1   = $10
        gate2   = $11
        gate3   = $12
        gate4   = $13
        tmp_name  = $14
        ipaddr  = "#{octet1}.#{octet2}.#{octet3}.#{octet4}"
        submask = "#{mask1}.#{mask2}.#{mask3}.#{mask4}"
        gateway = "#{gate1}.#{gate2}.#{gate3}.#{gate4}"

        $evm.log("info", "Processing Line: <#{line}> VLAN: <#{vlan}> IP: <#{ipaddr}> Mask: <#{submask}> Gateway: <#{gateway}> Name: <#{tmp_name}>")

        # If a the VLAN's match and there is a free slot found then switch it to the VM's hostname
        if vm_vlan == vlan && tmp_name == "free"
          newline = "#{vlan},#{ipaddr},#{submask},#{gateway},#{vm_hostname}"
          $evm.log("info", "Replacing Line: <#{line},> with : <#{newline}> in file: <#{fname}")

          # Use sed to do an in-place subtitution
          `sed -i 's/#{vlan},#{octet1}\.#{octet2}\.#{octet3}\.#{octet4},#{mask1}\.#{mask2}\.#{mask3}\.#{mask4},#{gate1}\.#{gate2}\.#{gate3}\.#{gate4},#{tmp_name}/#{vlan},#{octet1}\.#{octet2}\.#{octet3}\.#{octet4},#{mask1}\.#{mask2}\.#{mask3}\.#{mask4},#{gate1}\.#{gate2}\.#{gate3}\.#{gate4},#{vm_hostname}/' #{fname}`

          # Set provisioning options to override options
          prov.set_option(:sysprep_spec_override, [true, 1])
          # prov.set_option(:sysprep_timezone, "020")
          prov.set_option(:addr_mode, ["static", "Static"])
          prov.set_option(:ip_addr, ipaddr)
          prov.set_option(:subnet_mask, submask)
          prov.set_option(:gateway, gateway)
          $evm.log("info", "Provision Options: #{prov.options.inspect}")
          exit MIQ_OK
        else
          $evm.log("info", "Skipping Line: <#{line}>. Dialog VLAN: <#{vm_vlan}> File VLAN: <#{vlan}> and VM Name: <#{tmp_name}>")
        end #if status
      else
        $evm.log("info", "Line: <#{line}> does not match criteria in file: <#{fname}>")
      end
    end # file.each do
  end # file open do

  #
  # Exit method
  #
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "<#{@method}>: [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
