###################################
#
# EVM Automate Method: Retrieve_attachment
#
# This method is used to pull attachment from email and place in a directory
#
# ManageIQ: Bill Helgeson
###################################
begin
  @method = 'Retrieve_attachment'
  $evm.log("info", "Inline Method: <#{@method}>  EVM Automate Method Started")

  # Turn on verbose logging
  @debug = true

  # Load required ruby gem
  require 'mail'

  # Get values from class object
  mailhost = $evm.object['mailhost']
  mailuser = $evm.object['mailuser']
  mailpass = $evm.object['mailpass'].decrypt
  mailport = $evm.object['mailport']
  mailssl = $evm.object['mailssl']
  remoteDir = $evm.object['remoteDir']

  # Log into account pop3
  Mail.defaults do
    retriever_method :pop3, :address => mailhost,
                     :port => mailport,
                     :user_name => mailuser,
                     :password => mailpass,
                     :enable_ssl => mailssl
  end

  # Get all the emails
  emails = Mail.all

  # Do we have mail
  if emails.empty?
    $evm.log("info", "Inline Method: <#{@method}> - No Emails") if @debug
  end

  unless emails.empty?
    # We have Emails to process
    # Process Emails
    emails.each do |m|
      # Make sure email subject is for our report
      if m.subject == "Report From EVM"
        $evm.log("info", "Inline Method: <#{@method}> - Processing email from #{m.from} for report #{m.subject}") if @debug
        # Process all attachments for this email
        m.attachments.each do |attachment|
          filename = attachment.filename
          # Place the attachment on the remote share
          File.open("#{remoteDir}/#{filename}", "w+b", 0644) { |f| f.write attachment.body.decoded }
          $evm.log("info", "Inline Method: <#{@method}> - Report #{filename} retrieved") if @debug
        end
        # Mark processed email for delete
        m.mark_for_delete = true
      else
        $evm.log("info", "Inline Method: <#{@method}> - Report #{m.subject} does not qualify") if @debug
        next
      end
    end
    # Clean up emails
    Mail.find_and_delete
    $evm.log("info", "Inline Method: <#{@method}> - Reports delivered") if @debug
  end

  ############
  # Exit method
  #
  $evm.log("info", "Inline Method: <#{@method}>  EVM Automate Method Ended")
  exit MIQ_OK

#
# Set Ruby rescue behavior
#
rescue => err
  $evm.log("error", "Inline Method: <#{@method}> - [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
