###################################
#
# EVM Automate Method: vmname_sample1
#
# Names a VM with the following characteristics:
# XXDC-WOS-INP
# Where:
# XX = static 2 chars that will always be the prefix when naming a VM
# DC = 3 char from Tag Category Datacenter
# W = 1 char from Tag Category Function
# OS = 3 for Win2K3 8 for Win2K8 based on Template
# IN = 3 char user will be required to input from dialog where VM Naming was before. We have renamed and set limits on this dialog field accordingly.
#  $n(2) = 2 digit generated number 01, 02, 03, etc
#
###################################
begin
  @method = 'vmname_sample1'
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Started")

    # Turn of verbose logging
  @debug = true

  prov                            = $evm.root['miq_provision_request'] || $evm.root['miq_provision'] || $evm.root['miq_provision_request_template']
  vm_name                         = prov.get_option(:vm_name).to_s.strip
  number_of_vms_being_provisioned = prov.get_option(:number_of_vms)
  template                        = prov.vm_template
  virtual_center                  = template.ext_management_system


  #############################
  # Uncomment line below if tags are needed for VM Name
  tags = prov.get_tags
  $evm.log("info","Tags: #{tags.inspect}") if @debug
  #############################


  #
  # Method: dumpobj
  # Dump object and automation space root attributes
  #
  def dumpobj(obj=self)
    $evm.log("info", "dumpobj - Dumping <#{obj}> object")
    obj.instance_variables.each do |key|
      value = obj.instance_eval(key)
      $evm.log("info", "dumpobj-<#{obj}>---> #{key}: #{value.inspect}") if @debug
    end
    $evm.log("info", "dumpobj - Dumping <root> object") if @debug
    $evm.root.attributes.each { |key, value| $evm.log("info", "dumpobj-<root>---> #{key}: #{value.inspect}") } if @debug
  end


  #############################
  # Example: cb22f-m8-sql01
  #
  # XXDC-WOS-INP
  # XX = static 2 chars that will always be the prefix when naming a VM
  # DC = 3 char from Tag Category Datacenter
  # W = 1 char from Tag Category Function
  # OS = 3 for Win2K3 8 for Win2K8 based on Template
  # IN = 3 char user will be required to input from dialog where VM Naming was before. We have renamed and set limits on this dialog field accordingly.
  #  $n(2) = 2 digit generated number 01, 02, 03, etc
  #
  #############################

  # Prefix vmname with a static prefix
  prefix = 'cb'
  $evm.log("info","Prefix: <#{prefix}>")


  # Get datacenter tag
  dc = tags[:datacenter]
  $evm.log("info","Datacenter (dc) tag: <#{dc}>")

  # Get function/workload tag
  wl = tags[:function]
  $evm.log("info","Workload (wl) tag: <#{wl}>")


  # Get OS Type from the template platform
  product  = template.operating_system['product_name'] rescue ''
  $evm.log("info","Template product name: <#{product}>")

  if product.include?("2003")
    os = "3" # Win2k3
  elsif product.include?("2008")
    os = "8" # Win2k8
  elsif product.include?("Linux")
    os = "w" # unknown windows type
  end
  $evm.log("info","OS product type: <#{os}>")


  # Get 3 char user will be required to input from dialog
  appl = vm_name.downcase
  $evm.log("info","VM input name: <#{appl}>")


  # Octet separator
  sep = "-"
  # First (prefix + datacenter)
  oct1 = "#{prefix}#{dc}"
  # Second (workload + os)
  oct2 = "#{wl}#{os}"
  # third (application)
  oct3 = "#{appl}"

  # construct vmname
  derived_name = "#{oct1}#{sep}#{oct2}#{sep}#{oct3}"

  if number_of_vms_being_provisioned == 1
    derived_name = "#{derived_name}$n{2}"
  else
    derived_name = "#{derived_name}$n{2}"
  end

  # Set the evm object to the vmname
  $evm.object['vmname'] = derived_name

  $evm.log("info", "VM Naming Method Completed: vmname=#{derived_name}")


  #
  # Exit method
  #
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "<#{@method}>: [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
