###################################
#
# EVM Automate Method: vmname_sample2
#
# Notes: This method names the VM using the following:
# #{location}#{os_type}#{domain}$n{4}
#
#
###################################
begin
  @method = 'vmname_sample2'
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Started")

  # Turn of verbose logging
  @debug = true

  #
  # Get variables
  #
  prov = $evm.root['miq_provision_request'] || $evm.root['miq_provision'] || $evm.root['miq_provision_request_template']
  vm_name = prov.get_option(:vm_name).to_s.strip
  number_of_vms_being_provisioned = prov.get_option(:number_of_vms)
  template                        = prov.vm_template
  virtual_center                  = template.ext_management_system


  # Uncomment line below if tags are needed for VM Name
  # tags                            = prov.get_tags

  # Set a prefix called location
  location = 'sd'

  # Set a domain value based on the VC name
  os_type  = template.platform[0,1]
  if virtual_center.name.downcase.include?('vlab')
    domain = 'v'
  else
    domain = 'l'
  end

  # Name VM with location, os type, and domain
  if vm_name.empty?
    derived_name = "#{location}#{os_type}#{domain}$n{4}"
  elsif number_of_vms_being_provisioned == 1
    derived_name = "#{vm_name}"
  else
    derived_name =  "#{vm_name}$n{3}"
  end

  $evm.object['vmname'] = derived_name
  $evm.log("info", "VM Naming Method Completed: vmname=#{derived_name}")


  #
  # Exit method
  #
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "<#{@method}>: [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
