#
# EVM Automate Method: Create CMDB CI in Service-now
#
# Notes: This method is used to Create a CMDB in Service-now
#
###################################
begin
  @method = 'Service_now Create/Update CI'
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Started")

  # Turn off verbose logging
  @debug = true

  ##################################################################
  # Look in the current object for a VM
  #
  vm = $evm.object['vm']
  if vm.nil?
    vm_id = $evm.object['vm_id'].to_i
    vm = $evm.vmdb('vm', vm_id) unless vm_id == 0
  end

  #
  # Look in the Root Object for a VM
  #
  if vm.nil?
    vm = $evm.root['vm']
    if vm.nil?
      vm_id = $evm.root['vm_id'].to_i
      vm = $evm.vmdb('vm', vm_id) unless vm_id == 0
    end
  end

  # No VM Found, exit
  raise "VM not found" if vm.nil?
  ##################################################################

  # Get variables from Server object
  server = $evm.root['miq_server']
  evm_ipaddr    = server.ipaddress
  evm_hostname  = server.hostname
  evm_region    = server.region_number

  vm = $evm.root['vm']
  $evm.log("info", "Got VM from root object")
  $evm.log("info","Inspecting vm: #{vm.inspect}") if @debug
  vm_ipaddr = vm.ipaddresses.first
  vm_name = vm.name
  vm_id = vm['id']

  ems       = vm.ext_management_system
  $evm.log("info", "Got EMS <#{ems.name}> from VM <#{vm_name}>")
  $evm.log("info","Inspecting ems: #{ems.inspect}") if @debug
  ems_name = ems.hostname
  ems_ip = ems.ipaddress


  user = $evm.root['user']
  $evm.log("info","Inspecting User object: <#{user.inspect}>")
  username = user.name
  userid = user.userid

  
  host      = vm.host
  #$evm.log("info","Inspecting host: #{host.inspect}") if @debug
  $evm.log("info", "Got Host <#{host.name}> from VM <#{vm_name}>")
  host_name = host.name
  host_id = host['id']


process  = $evm.object("process")
hardware = vm.hardware
$evm.log("info","Inspecting hardware: #{hardware.inspect}") if @debug
owner               = vm.owner

os = vm.operating_system
$evm.log("info","Inspecting os: #{os.inspect}") if @debug



config_version      = hardware.config_version      #  varchar 255
virtual_hw_version  = hardware.virtual_hw_version  #  varchar 255
guest_os            = hardware.guest_os            #  varchar 255
numvcpus            = hardware.numvcpus            #  int4
bios                = hardware.bios                #  varchar 255
bios_location       = hardware.bios_location       #  varchar 255
time_sync           = hardware.time_sync           #  varchar 255
annotation          = hardware.annotation          #  text
vm_id               = hardware.vm_id               #  int8
memory_cpu          = hardware.memory_cpu          #  int4
host_id             = hardware.host_id             #  int8
cpu_speed           = hardware.cpu_speed           #  int4
cpu_type            = hardware.cpu_type            #  varchar 255
size_on_disk        = hardware.size_on_disk        #  int8
manufacturer        = hardware.manufacturer        #  varchar 255
model               = hardware.model               #  varchar 255
number_of_nics      = hardware.number_of_nics      #  int4
cpu_usage           = hardware.cpu_usage           #  int4
memory_usage        = hardware.memory_usage        #  int4
cores_per_socket    = hardware.cores_per_socket    #  int4
logical_cpus        = hardware.logical_cpus        #  int4
vmotion_enabled     = hardware.vmotion_enabled     #  int4
disk_free_space     = hardware.disk_free_space     #  int8
disk_capacity       = hardware.disk_capacity       #  int8
guest_os_full_name  = hardware.guest_os_full_name  #  varchar 255
memory_console      = hardware.memory_console      #  int4
product_name        = os.product_name
service_pack        = os.service_pack
os_width            = os.bitness
os_version          = os.version
os_type             = os.system_type
host_ipaddress      = host.ipaddress
#host_macaddress     = host.macaddress
host_name           = host.name
ems_name            = ems.name
ems_ipaddress       = ems.ipaddress
#ems_macaddress      = ems.macaddress







  ###################################
  #
  # Call Service-Now
  #
  ###################################



  agent      = "EVM Automate CMDB Server"
  topic      = "Create/Update CI for VM"
  name       = "VM:#{vm_name} | Host:#{host_name} | IP:#{vm_ipaddr}"
  source     = "EVM Region:#{evm_region} | Hostname:#{evm_hostname} | IP:#{evm_ipaddr}"

  $evm.log("info","Server IP: #{evm_ipaddr}")

  $evm.log("info","VM IP: #{vm_ipaddr}")

  comments   = "EVM Creating/Updating CI for VM: #{vm_name} Host:#{host_name} Host IP:#{evm_ipaddr}"
  vmurl      = "https://#{evm_ipaddr}/vm/show/#{vm_id}"
  hosturl    = "https://#{evm_ipaddr}/host/show/#{host_id}"
  description = "EVM Creating/Updating CI for VM: #{vm_name} Host:#{host_name} Host IP:#{evm_ipaddr}"

  require 'uri'
  args =  "agent=#{agent}&"
  args += "topic=#{topic}&"
  args += "source=#{source}&"
  args += "name=#{name}&"
  args += "comments=#{comments}&"
  args += "vmurl=#{vmurl}&"
  args += "hosturl=#{hosturl}&"
  args += "evm_vmname=#{vm_name}&"
  args += "hostname=#{host_name}&"
  args += "emsname=#{ems_name}&"
  args += "emsip=#{ems_ip}&"
  args += "username=#{username}&"
  args += "userid=#{userid}&"
  args += "config_version=#{config_version}&"
  args += "virtual_hw_version=#{virtual_hw_version}&"
  args += "guest_os=#{guest_os}&"
  args += "numvcpus=#{numvcpus}&"
  args += "bios=#{bios}&"
  args += "bios_location=#{bios_location}&"
  args += "time_sync=#{time_sync}&"
  args += "annotation=#{annotation}&"
  args += "vm_id=#{vm_id}&"
  args += "memory_cpu=#{memory_cpu}&"
  args += "host_id=#{host_id}&"
  args += "cpu_speed=#{cpu_speed}&"
  args += "cpu_type=#{cpu_type}&"
  args += "size_on_disk=#{size_on_disk}&"
  args += "manufacturer=#{manufacturer}&"
  args += "model=#{model}&"
  args += "number_of_nics=#{number_of_nics}&"
  args += "cpu_usage=#{cpu_usage}&"
  args += "memory_usage=#{memory_usage}&"
  args += "cores_per_socket=#{cores_per_socket}&"
  args += "logical_cpus=#{logical_cpus}&"
  args += "vmotion_enabled=#{vmotion_enabled}&"
  args += "disk_free_space=#{disk_free_space}&"
  args += "disk_capacity=#{disk_capacity}&"
  args += "guest_os_full_name=#{guest_os_full_name}&"
  args += "memory_console=#{memory_console}&"
  args += "product_name=#{product_name}&"
  args += "service_pack=#{service_pack}&"
  args += "os_width=#{os_width}&"
  args += "os_type=#{os_type}&"
  args += "ems_name=#{ems_name}&"
  args += "ems_ipaddress=#{ems_ipaddress}&"
  args += "host_ipaddress=#{host_ipaddress}&"
  args += "host_name=#{host_name}&"
  args += "vm_ipaddr=#{vm_ipaddr}&"
  args += "os_version=#{os_version}"

  args = URI.escape(args)

  $evm.log("info","Calling Service-Now with arguments: #{args}")
  $evm.instantiate("/Factory/ServiceNow/create_ci_in_eccq?#{args}")

  #
  # Exit method
  #
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "<#{@method}>: [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_STOP
end
