#
# EVM Automate Method: Create Incident in Service-now
#
# Notes: This method is used to Create an Incident in Service-now
#
###################################
begin
  @method = 'Service_now Create Incident'
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Started")

  # Turn off verbose logging
  @debug = true

  ##################################################################
  # Look in the current object for a VM
  #
  vm = $evm.object['vm']
  if vm.nil?
    vm_id = $evm.object['vm_id'].to_i
    vm = $evm.vmdb('vm', vm_id) unless vm_id == 0
  end

  #
  # Look in the Root Object for a VM
  #
  if vm.nil?
    vm = $evm.root['vm']
    if vm.nil?
      vm_id = $evm.root['vm_id'].to_i
      vm = $evm.vmdb('vm', vm_id) unless vm_id == 0
    end
  end

  # No VM Found, exit
  raise "VM not found" if vm.nil?
  ##################################################################

  # Get variables from Server object
  server = $evm.root['miq_server']
  evm_ipaddr    = server.ipaddress
  evm_hostname  = server.hostname
  evm_region    = server.region_number

  vm = $evm.root['vm']
  $evm.log("info", "Got VM from root object")
  $evm.log("info","Inspecting vm: #{vm.inspect}") if @debug
  vm_ipaddr = vm.ipaddresses.first
  vm_name = vm.name
  vm_id = vm['id']

  ems       = vm.ext_management_system
  $evm.log("info", "Got EMS <#{ems.name}> from VM <#{vm_name}>")
  $evm.log("info","Inspecting ems: #{ems.inspect}") if @debug
  ems_name = ems.hostname
  ems_ip = ems.ipaddress


  user = $evm.root['user']
  $evm.log("info","Inspecting User object: <#{user.inspect}>")
  username = user.name
  userid = user.userid

  
  host      = vm.host
  #$evm.log("info","Inspecting host: #{host.inspect}") if @debug
  $evm.log("info", "Got Host <#{host.name}> from VM <#{vm_name}>")
  host_name = host.name
  host_id = host['id']


  process                = $evm.object("process")
  evm_policy_id          = process.attributes["policy_id"].to_i
  evm_policy_name        = process.attributes["policy_name"] || 'create_incident'
  evm_policy_description = process.attributes["policy_description"] || "User: #{username} requested incident for this object"
  $evm.log("info", "Policy ID:          #{evm_policy_id}")
  $evm.log("info", "Policy Name:        #{evm_policy_name}")
  $evm.log("info", "Policy Description: #{evm_policy_description}")
  #policy    = $evm.vmdb('policy', policy_id)


  ###################################
  #
  # Call Service-Now
  #
  ###################################

  priority = 'low'
  severity = 3

  agent      = "EVM Automate Create Incident"
  topic      = "Violation in VM Policy:#{evm_policy_name}"
  name       = "VM:#{vm_name} | Host:#{host_name} | IP:#{vm_ipaddr}"
  source     = "EVM Region:#{evm_region} | Hostname:#{evm_hostname} | IP:#{evm_ipaddr}"

  $evm.log("info","Server IP: #{evm_ipaddr}")

  $evm.log("info","VM IP: #{vm_ipaddr}")

  short_desc = evm_policy_description
  comments   = "EVM detected a policy violation for VM: #{vm_name} Policy: #{evm_policy_description} Host:#{host_name} Host IP:#{evm_ipaddr}"
  vmurl      = "https://#{evm_ipaddr}/vm/show/#{vm_id}"
  hosturl    = "https://#{evm_ipaddr}/host/show/#{host_id}"
  description = "EVM detected a policy violation for VM: #{vm_name} \nPolicy: #{evm_policy_description} \nHost:#{host_name} \nHost IP:#{evm_ipaddr}"

  require 'uri'
  args = "agent=#{agent}&topic=#{topic}&source=#{source}&name=#{name}&comments=#{comments}&evm_policy_description=#{short_desc}&vmurl=#{vmurl}&hosturl=#{hosturl}&evm_vmname=#{vm_name}&evm_policy_id=#{evm_policy_id}&evm_policy_name=#{evm_policy_name}&hostname=#{host_name}&priority=#{priority}&severity=#{severity}&emsname=#{ems_name}&emsip=#{ems_ip}&username=#{username}&userid=#{userid}"
  args = URI.escape(args)

  $evm.log("info","Calling Service-Now with arguments: #{args}")
  $evm.instantiate("/Factory/ServiceNow/create_incident_in_eccq?#{args}")

  #
  # Exit method
  #
  $evm.log("info", "===== EVM Automate Method: <#{@method}> Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "<#{@method}>: [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_STOP
end
