###################################
#
# EVM Automate Method: ServiceNow_CI_SOAP_Insert
#
# Notes: This method uses a SOAP/XML call to update a CI in Service-Now
#
###################################
# Method for logging
def log(level, message)
  @method = 'ServiceNow_CI_SOAP_Insert'
  $evm.log(level, "#{@method} - #{message}")
end

begin
  log(:info, "EVM Automate Method Started")
  
  
  ###########################
  #
  # Method: insertCI
  # Inputs: 
  #
  ###########################
  def insertCI()
    # Require Savon Ruby Gem
    require "rubygems"
    require "savon"

    username = nil
    username ||= $evm.object['username']

    password = nil
    password ||= $evm.object.decrypt('password')

    wsdl_uri = nil
    wsdl_uri ||= $evm.object['wsdl_uri']
    
    
    # Set up SOAP Connection to WSDL
    client = Savon::Client.new do |wsdl,http|
      wsdl.document = wsdl_uri
      http.auth.basic username, password
    end

    log(:info,"Namespace: #{client.wsdl.namespace.inspect}")
    log(:info,"Endpoint: #{client.wsdl.endpoint.inspect}")
    
    # Incident Actions: [:insert, :update, :delete_multiple, :get_keys, :delete_record, :get, :get_records]
    log(:info,"Actions: #{client.wsdl.soap_actions.inspect}")

    # Call Remedy
    response = client.request :insert do |soap|
      # Build the body of the XML elements, Below are some samples
      
      
      soap.body = {
        # Note: DO NOT USE SYMBOLS - USE STRINGS
        'name' => 'vm12345',
        'short_description' => 'vm12345desc'
      }
    end
    
    # Convert xml response to a hash
    # Inspecting Service-Now Response:<{:sys_id=>"455719bbff976c80a19538abb67efe21", :name=>"kevinpc4"}>
        
    hash_response = response.to_hash[:insert_response]
    log(:info,"Inspecting Service-Now Response:<#{hash_response.inspect}>")

    log(:info,"Service-Now CI SOAP Insert sys_id=<#{hash_response[:sys_id]}>")
    log(:info,"Service-Now CI SOAP Insert name=<#{hash_response[:name]}>")
    return hash_response
  end
  
  servicecnowResponse = insertCI()
    
  #
  # Exit method
  #
  log(:info, "EVM Automate Method Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
