###################################
#
# EVM Automate Method: ServiceNow_Eccq_SOAP_Insert
#
# Notes: This method uses a SOAP/XML call to post data to 
#   Service-Now's Ecc Queue
#
###################################
begin
  @method = 'ServiceNow_Eccq_SOAP_Insert'
  @log_prefix = "[#{@method}] --"
  $evm.log("info", "#{@log_prefix} EVM Automate Method Started")

  # Turn of verbose logging
  @debug = true
  
  
  ###########################
  #
  # Method: eccqInsert
  # Inputs: 
  #
  ###########################
  def eccqInsert()
    # Require Savon Ruby Gem
    require "savon"

    username = nil
    username ||= $evm.object['username']

    password = nil
    password ||= $evm.object.decrypt('password')

    wsdl_uri = nil
    wsdl_uri ||= $evm.object['wsdl_uri']
    
    
    # Set up SOAP Connection to WSDL
    client = Savon::Client.new do |wsdl,http|
      wsdl.document = wsdl_uri
      http.auth.basic username, password
    end

    $evm.log("info","Namespace: #{client.wsdl.namespace.inspect}") if @debug
    $evm.log("info","Endpoint: #{client.wsdl.endpoint.inspect}") if @debug
    $evm.log("info","Actions: #{client.wsdl.soap_actions.inspect}") if @debug

    # Build the Payload in hash format
    payload = {}
    payload[:pay1] = 1
    payload[:pay2] = 'two'

    # Call Remedy
    response = client.request :insert do |soap|
      # Build the body of the XML elements 
      soap.body = {
        :agent             => 'EVM Agent',
        :agent_correlator  => 'EVM agent_correlator',
        :from_host         => 'EVM Host',
        :name              => 'EVM Name',
        :payload           => payload,
        :queue             => 'input',
        :source            => 'EVM Source',
        :topic             => 'EVM Topic',
      }
    end
    
    # Convert xml response to a hash
    # Service-Now hash Response:<{:insert_response=>{:sys_id=>"e3569c7bff1f2c80a19538abb67efe54", :topic=>"EVM Topic"}}>
    hash_response = response.to_hash[:insert_response]
    $evm.log("info","Inspecting Service-Now Response:<#{hash_response.inspect}>")

    $evm.log("info","Eccq Insert sys_id=<#{hash_response[:sys_id]}>")
    $evm.log("info","Eccq Insert topic=<#{hash_response[:topic]}>")
    return hash_response
  end
  
  servicecnowResponse = eccqInsert()
    
  #
  # Exit method
  #
  $evm.log("info", "#{@log_prefix} EVM Automate Method Ended")
  exit MIQ_OK

  #
  # Set Ruby rescue behavior
  #
rescue => err
  $evm.log("error", "#{@log_prefix} [#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
